@@ -1,6 +1,10 @@
+#ifndef WIN32
 #include <arpa/inet.h>
-#include <errno.h>
 #include <netinet/in.h>
+#else
+#include <windows.h>
+#endif
+#include <errno.h>
 #include <ndpi_api.h>
 #include <ndpi_main.h>
 #include <ndpi_typedefs.h>
@@ -12,6 +16,7 @@
 #include <string.h>
 #include <unistd.h>
 
+//#define VERBOSE 1
 #define MAX_FLOW_ROOTS_PER_THREAD 2048
 #define MAX_IDLE_FLOWS_PER_THREAD 64
 #define TICK_RESOLUTION 1000
@@ -47,12 +52,19 @@ struct nDPI_flow_info {
   union {
     struct {
       uint32_t src;
+      uint32_t pad_00[3];
       uint32_t dst;
+      uint32_t pad_01[3];
     } v4;
     struct {
       uint64_t src[2];
       uint64_t dst[2];
     } v6;
+
+    struct {
+      uint32_t src[4];
+      uint32_t dst[4];
+    } u32;
   } ip_tuple;
 
   unsigned long long int total_l4_data_len;
@@ -73,16 +85,12 @@ struct nDPI_flow_info {
   struct ndpi_proto guessed_protocol;
 
   struct ndpi_flow_struct * ndpi_flow;
-  struct ndpi_id_struct * ndpi_src;
-  struct ndpi_id_struct * ndpi_dst;
 };
 
 struct nDPI_workflow {
   pcap_t * pcap_handle;
 
-  uint8_t error_or_eof:1;
-  uint8_t reserved_00:7;
-  uint8_t reserved_01[3];
+  uint8_t error_or_eof;
 
   unsigned long long int packets_captured;
   unsigned long long int packets_processed;
@@ -130,12 +138,16 @@ static struct nDPI_workflow * init_workflow(char const * const file_or_device)
   if (access(file_or_device, R_OK) != 0 && errno == ENOENT) {
     workflow->pcap_handle = pcap_open_live(file_or_device, /* 1536 */ 65535, 1, 250, pcap_error_buffer);
   } else {
+#ifdef WIN32
+    workflow->pcap_handle = pcap_open_offline(file_or_device, pcap_error_buffer);
+#else
     workflow->pcap_handle = pcap_open_offline_with_tstamp_precision(file_or_device, PCAP_TSTAMP_PRECISION_MICRO,
 								    pcap_error_buffer);
+#endif
   }
 
   if (workflow->pcap_handle == NULL) {
-    fprintf(stderr, "pcap_open_live / pcap_open_offline_with_tstamp_precision: %.*s\n",
+    fprintf(stderr, "pcap_open_live / pcap_open_offline: %.*s\n",
 	    (int) PCAP_ERRBUF_SIZE, pcap_error_buffer);
     free_workflow(&workflow);
     return NULL;
@@ -176,8 +188,6 @@ static void ndpi_flow_info_freer(void * const node)
 {
   struct nDPI_flow_info * const flow = (struct nDPI_flow_info *)node;
 
-  ndpi_free(flow->ndpi_dst);
-  ndpi_free(flow->ndpi_src);
   ndpi_flow_free(flow->ndpi_flow);
   ndpi_free(flow);
 }
@@ -336,60 +346,72 @@ static void print_packet_info(struct nDPI_reader_thread const * const reader_thr
 }
 #endif
 
-static int ip_tuples_equal(struct nDPI_flow_info const * const A,
-                           struct nDPI_flow_info const * const B)
+static int ip_tuples_compare(struct nDPI_flow_info const * const A, struct nDPI_flow_info const * const B)
 {
-  if (A->l3_type == L3_IP && B->l3_type == L3_IP6) {
-    return A->ip_tuple.v4.src == B->ip_tuple.v4.src &&
-      A->ip_tuple.v4.dst == B->ip_tuple.v4.dst;
-  } else if (A->l3_type == L3_IP6 && B->l3_type == L3_IP6) {
-    return A->ip_tuple.v6.src[0] == B->ip_tuple.v6.src[0] &&
-      A->ip_tuple.v6.src[1] == B->ip_tuple.v6.src[1] &&
-      A->ip_tuple.v6.dst[0] == B->ip_tuple.v6.dst[0] &&
-      A->ip_tuple.v6.dst[1] == B->ip_tuple.v6.dst[1];
+  // generate a warning if the enum changes
+  switch (A->l3_type)
+  {
+    case L3_IP:
+    case L3_IP6:
+      break;
   }
-  return 0;
-}
 
-static int ip_tuples_compare(struct nDPI_flow_info const * const A,
-                             struct nDPI_flow_info const * const B)
-{
-  if (A->l3_type == L3_IP && B->l3_type == L3_IP6) {
-    if (A->ip_tuple.v4.src < B->ip_tuple.v4.src ||
-	A->ip_tuple.v4.dst < B->ip_tuple.v4.dst)
-      {
-	return -1;
-      }
-    if (A->ip_tuple.v4.src > B->ip_tuple.v4.src ||
-	A->ip_tuple.v4.dst > B->ip_tuple.v4.dst)
-      {
-	return 1;
-      }
-  } else if (A->l3_type == L3_IP6 && B->l3_type == L3_IP6) {
-    if ((A->ip_tuple.v6.src[0] < B->ip_tuple.v6.src[0] &&
-	 A->ip_tuple.v6.src[1] < B->ip_tuple.v6.src[1]) ||
-	(A->ip_tuple.v6.dst[0] < B->ip_tuple.v6.dst[0] &&
-	 A->ip_tuple.v6.dst[1] < B->ip_tuple.v6.dst[1]))
-      {
-	return -1;
-      }
-    if ((A->ip_tuple.v6.src[0] > B->ip_tuple.v6.src[0] &&
-	 A->ip_tuple.v6.src[1] > B->ip_tuple.v6.src[1]) ||
-	(A->ip_tuple.v6.dst[0] > B->ip_tuple.v6.dst[0] &&
-	 A->ip_tuple.v6.dst[1] > B->ip_tuple.v6.dst[1]))
-      {
-	return 1;
-      }
-  }
-  if (A->src_port < B->src_port ||
-      A->dst_port < B->dst_port)
+  if (A->l3_type == L3_IP && B->l3_type == L3_IP)
+  {
+    if (A->ip_tuple.v4.src < B->ip_tuple.v4.src)
     {
       return -1;
-    } else if (A->src_port > B->src_port ||
-               A->dst_port > B->dst_port)
+    }
+    if (A->ip_tuple.v4.src > B->ip_tuple.v4.src)
     {
       return 1;
     }
+    if (A->ip_tuple.v4.dst < B->ip_tuple.v4.dst)
+    {
+      return -1;
+    }
+    if (A->ip_tuple.v4.dst > B->ip_tuple.v4.dst)
+    {
+      return 1;
+    }
+  }
+  else if (A->l3_type == L3_IP6 && B->l3_type == L3_IP6)
+  {
+    if (A->ip_tuple.v6.src[0] < B->ip_tuple.v6.src[0] && A->ip_tuple.v6.src[1] < B->ip_tuple.v6.src[1])
+    {
+      return -1;
+    }
+    if (A->ip_tuple.v6.src[0] > B->ip_tuple.v6.src[0] && A->ip_tuple.v6.src[1] > B->ip_tuple.v6.src[1])
+    {
+      return 1;
+    }
+    if (A->ip_tuple.v6.dst[0] < B->ip_tuple.v6.dst[0] && A->ip_tuple.v6.dst[1] < B->ip_tuple.v6.dst[1])
+    {
+      return -1;
+    }
+    if (A->ip_tuple.v6.dst[0] > B->ip_tuple.v6.dst[0] && A->ip_tuple.v6.dst[1] > B->ip_tuple.v6.dst[1])
+    {
+      return 1;
+    }
+  }
+
+  if (A->src_port < B->src_port)
+  {
+    return -1;
+  }
+  if (A->src_port > B->src_port)
+  {
+    return 1;
+  }
+  if (A->dst_port < B->dst_port)
+  {
+    return -1;
+  }
+  if (A->dst_port > B->dst_port)
+  {
+    return 1;
+  }
+
   return 0;
 }
 
@@ -438,13 +460,6 @@ static int ndpi_workflow_node_cmp(void const * const A, void const * const B) {
     return(1);
   }
 
-  if (ip_tuples_equal(flow_info_a, flow_info_b) != 0 &&
-      flow_info_a->src_port == flow_info_b->src_port &&
-      flow_info_a->dst_port == flow_info_b->dst_port)
-    {
-      return(0);
-    }
-
   return ip_tuples_compare(flow_info_a, flow_info_b);
 }
 
@@ -486,10 +501,6 @@ static void ndpi_process_packet(uint8_t * const args,
   void * tree_result;
   struct nDPI_flow_info * flow_to_process;
 
-  int direction_changed = 0;
-  struct ndpi_id_struct * ndpi_src;
-  struct ndpi_id_struct * ndpi_dst;
-
   const struct ndpi_ethhdr * ethernet;
   const struct ndpi_iphdr * ip;
   struct ndpi_ipv6hdr * ip6;
@@ -687,7 +698,7 @@ static void ndpi_process_packet(uint8_t * const args,
   workflow->total_l4_data_len += l4_len;
 
 #ifdef VERBOSE
-  print_packet_info(reader_thread, header, l4_data_len, &flow);
+  print_packet_info(reader_thread, header, l4_len, &flow);
 #endif
 
   /* calculate flow hash for btree find, search(insert) */
@@ -713,27 +724,38 @@ static void ndpi_process_packet(uint8_t * const args,
   tree_result = ndpi_tfind(&flow, &workflow->ndpi_flows_active[hashed_index], ndpi_workflow_node_cmp);
   if (tree_result == NULL) {
     /* flow not found in btree: switch src <-> dst and try to find it again */
-    uint64_t orig_src_ip[2] = { flow.ip_tuple.v6.src[0], flow.ip_tuple.v6.src[1] };
-    uint64_t orig_dst_ip[2] = { flow.ip_tuple.v6.dst[0], flow.ip_tuple.v6.dst[1] };
+    uint32_t orig_src_ip[4] = { flow.ip_tuple.u32.src[0], flow.ip_tuple.u32.src[1],
+                                flow.ip_tuple.u32.src[2], flow.ip_tuple.u32.src[3] };
+    uint32_t orig_dst_ip[4] = { flow.ip_tuple.u32.dst[0], flow.ip_tuple.u32.dst[1],
+                                flow.ip_tuple.u32.dst[2], flow.ip_tuple.u32.dst[3] };
     uint16_t orig_src_port = flow.src_port;
     uint16_t orig_dst_port = flow.dst_port;
 
-    flow.ip_tuple.v6.src[0] = orig_dst_ip[0];
-    flow.ip_tuple.v6.src[1] = orig_dst_ip[1];
-    flow.ip_tuple.v6.dst[0] = orig_src_ip[0];
-    flow.ip_tuple.v6.dst[1] = orig_src_ip[1];
+    flow.ip_tuple.u32.src[0] = orig_dst_ip[0];
+    flow.ip_tuple.u32.src[1] = orig_dst_ip[1];
+    flow.ip_tuple.u32.src[2] = orig_dst_ip[2];
+    flow.ip_tuple.u32.src[3] = orig_dst_ip[3];
+
+    flow.ip_tuple.u32.dst[0] = orig_src_ip[0];
+    flow.ip_tuple.u32.dst[1] = orig_src_ip[1];
+    flow.ip_tuple.u32.dst[2] = orig_src_ip[2];
+    flow.ip_tuple.u32.dst[3] = orig_src_ip[3];
+
     flow.src_port = orig_dst_port;
     flow.dst_port = orig_src_port;
 
     tree_result = ndpi_tfind(&flow, &workflow->ndpi_flows_active[hashed_index], ndpi_workflow_node_cmp);
-    if (tree_result != NULL) {
-      direction_changed = 1;
-    }
 
-    flow.ip_tuple.v6.src[0] = orig_src_ip[0];
-    flow.ip_tuple.v6.src[1] = orig_src_ip[1];
-    flow.ip_tuple.v6.dst[0] = orig_dst_ip[0];
-    flow.ip_tuple.v6.dst[1] = orig_dst_ip[1];
+    flow.ip_tuple.u32.src[0] = orig_src_ip[0];
+    flow.ip_tuple.u32.src[1] = orig_src_ip[1];
+    flow.ip_tuple.u32.src[2] = orig_src_ip[2];
+    flow.ip_tuple.u32.src[3] = orig_src_ip[3];
+
+    flow.ip_tuple.u32.dst[0] = orig_dst_ip[0];
+    flow.ip_tuple.u32.dst[1] = orig_dst_ip[1];
+    flow.ip_tuple.u32.dst[2] = orig_dst_ip[2];
+    flow.ip_tuple.u32.dst[3] = orig_dst_ip[3];
+
     flow.src_port = orig_src_port;
     flow.dst_port = orig_dst_port;
   }
@@ -757,7 +779,7 @@ static void ndpi_process_packet(uint8_t * const args,
     workflow->cur_active_flows++;
     workflow->total_active_flows++;
     memcpy(flow_to_process, &flow, sizeof(*flow_to_process));
-    flow_to_process->flow_id = flow_id++;
+    flow_to_process->flow_id = __sync_fetch_and_add(&flow_id, 1);
 
     flow_to_process->ndpi_flow = (struct ndpi_flow_struct *)ndpi_flow_malloc(SIZEOF_FLOW_STRUCT);
     if (flow_to_process->ndpi_flow == NULL) {
@@ -767,20 +789,6 @@ static void ndpi_process_packet(uint8_t * const args,
     }
     memset(flow_to_process->ndpi_flow, 0, SIZEOF_FLOW_STRUCT);
 
-    flow_to_process->ndpi_src = (struct ndpi_id_struct *)ndpi_calloc(1, SIZEOF_ID_STRUCT);
-    if (flow_to_process->ndpi_src == NULL) {
-      fprintf(stderr, "[%8llu, %d, %4u] Not enough memory for src id struct\n",
-	      workflow->packets_captured, reader_thread->array_index, flow_to_process->flow_id);
-      return;
-    }
-
-    flow_to_process->ndpi_dst = (struct ndpi_id_struct *)ndpi_calloc(1, SIZEOF_ID_STRUCT);
-    if (flow_to_process->ndpi_dst == NULL) {
-      fprintf(stderr, "[%8llu, %d, %4u] Not enough memory for dst id struct\n",
-	      workflow->packets_captured, reader_thread->array_index, flow_to_process->flow_id);
-      return;
-    }
-
     printf("[%8llu, %d, %4u] new %sflow\n", workflow->packets_captured, thread_index,
 	   flow_to_process->flow_id,
 	   (flow_to_process->is_midstream_flow != 0 ? "midstream-" : ""));
@@ -788,19 +796,8 @@ static void ndpi_process_packet(uint8_t * const args,
       /* Possible Leak, but should not happen as we'd abort earlier. */
       return;
     }
-
-    ndpi_src = flow_to_process->ndpi_src;
-    ndpi_dst = flow_to_process->ndpi_dst;
   } else {
     flow_to_process = *(struct nDPI_flow_info **)tree_result;
-
-    if (direction_changed != 0) {
-      ndpi_src = flow_to_process->ndpi_dst;
-      ndpi_dst = flow_to_process->ndpi_src;
-    } else {
-      ndpi_src = flow_to_process->ndpi_src;
-      ndpi_dst = flow_to_process->ndpi_dst;
-    }
   }
 
   flow_to_process->packets_processed++;
@@ -851,7 +848,7 @@ static void ndpi_process_packet(uint8_t * const args,
   flow_to_process->detected_l7_protocol =
     ndpi_detection_process_packet(workflow->ndpi_struct, flow_to_process->ndpi_flow,
 				  ip != NULL ? (uint8_t *)ip : (uint8_t *)ip6,
-				  ip_size, time_ms, ndpi_src, ndpi_dst);
+				  ip_size, time_ms);
 
   if (ndpi_is_protocol_detected(workflow->ndpi_struct,
 				flow_to_process->detected_l7_protocol) != 0 &&
@@ -906,41 +903,43 @@ static void ndpi_process_packet(uint8_t * const args,
 	  flow_to_process->detected_l7_protocol.app_protocol == NDPI_PROTOCOL_TLS)
         {
 	  if (flow_to_process->tls_client_hello_seen == 0 &&
-	      flow_to_process->ndpi_flow->l4.tcp.tls.hello_processed != 0)
+	      flow_to_process->ndpi_flow->protos.tls_quic.hello_processed != 0)
             {
 	      uint8_t unknown_tls_version = 0;
+	      char buf_ver[16];
 	      printf("[%8llu, %d, %4d][TLS-CLIENT-HELLO] version: %s | sni: %s | alpn: %s\n",
 		     workflow->packets_captured,
 		     reader_thread->array_index,
 		     flow_to_process->flow_id,
-		     ndpi_ssl_version2str(flow_to_process->ndpi_flow,
-					  flow_to_process->ndpi_flow->protos.tls_quic_stun.tls_quic.ssl_version,
+		     ndpi_ssl_version2str(buf_ver, sizeof(buf_ver),
+					  flow_to_process->ndpi_flow->protos.tls_quic.ssl_version,
 					  &unknown_tls_version),
-		     flow_to_process->ndpi_flow->protos.tls_quic_stun.tls_quic.client_requested_server_name,
-		     (flow_to_process->ndpi_flow->protos.tls_quic_stun.tls_quic.alpn != NULL ?
-		      flow_to_process->ndpi_flow->protos.tls_quic_stun.tls_quic.alpn : "-"));
+		     flow_to_process->ndpi_flow->host_server_name,
+		     (flow_to_process->ndpi_flow->protos.tls_quic.alpn != NULL ?
+		      flow_to_process->ndpi_flow->protos.tls_quic.alpn : "-"));
 	      flow_to_process->tls_client_hello_seen = 1;
             }
 	  if (flow_to_process->tls_server_hello_seen == 0 &&
 	      flow_to_process->ndpi_flow->l4.tcp.tls.certificate_processed != 0)
             {
 	      uint8_t unknown_tls_version = 0;
+	      char buf_ver[16];
 	      printf("[%8llu, %d, %4d][TLS-SERVER-HELLO] version: %s | common-name(s): %.*s | "
 		     "issuer: %s | subject: %s\n",
 		     workflow->packets_captured,
 		     reader_thread->array_index,
 		     flow_to_process->flow_id,
-		     ndpi_ssl_version2str(flow_to_process->ndpi_flow,
-					  flow_to_process->ndpi_flow->protos.tls_quic_stun.tls_quic.ssl_version,
+		     ndpi_ssl_version2str(buf_ver, sizeof(buf_ver),
+					  flow_to_process->ndpi_flow->protos.tls_quic.ssl_version,
 					  &unknown_tls_version),
-		     (flow_to_process->ndpi_flow->protos.tls_quic_stun.tls_quic.server_names_len == 0 ?
-		      1 : flow_to_process->ndpi_flow->protos.tls_quic_stun.tls_quic.server_names_len),
-		     (flow_to_process->ndpi_flow->protos.tls_quic_stun.tls_quic.server_names == NULL ?
-		      "-" : flow_to_process->ndpi_flow->protos.tls_quic_stun.tls_quic.server_names),
-		     (flow_to_process->ndpi_flow->protos.tls_quic_stun.tls_quic.issuerDN != NULL ?
-		      flow_to_process->ndpi_flow->protos.tls_quic_stun.tls_quic.issuerDN : "-"),
-		     (flow_to_process->ndpi_flow->protos.tls_quic_stun.tls_quic.subjectDN != NULL ?
-		      flow_to_process->ndpi_flow->protos.tls_quic_stun.tls_quic.subjectDN : "-"));
+		     (flow_to_process->ndpi_flow->protos.tls_quic.server_names_len == 0 ?
+		      1 : flow_to_process->ndpi_flow->protos.tls_quic.server_names_len),
+		     (flow_to_process->ndpi_flow->protos.tls_quic.server_names == NULL ?
+		      "-" : flow_to_process->ndpi_flow->protos.tls_quic.server_names),
+		     (flow_to_process->ndpi_flow->protos.tls_quic.issuerDN != NULL ?
+		      flow_to_process->ndpi_flow->protos.tls_quic.issuerDN : "-"),
+		     (flow_to_process->ndpi_flow->protos.tls_quic.subjectDN != NULL ?
+		      flow_to_process->ndpi_flow->protos.tls_quic.subjectDN : "-"));
 	      flow_to_process->tls_server_hello_seen = 1;
             }
         }
@@ -957,7 +956,7 @@ static void run_pcap_loop(struct nDPI_reader_thread const * const reader_thread)
 
       fprintf(stderr, "Error while reading pcap file: '%s'\n",
 	      pcap_geterr(reader_thread->workflow->pcap_handle));
-      reader_thread->workflow->error_or_eof = 1;
+      __sync_fetch_and_add(&reader_thread->workflow->error_or_eof, 1);
     }
   }
 }
@@ -978,14 +977,14 @@ static void * processing_thread(void * const ndpi_thread_arg)
 
   printf("Starting Thread %d\n", reader_thread->array_index);
   run_pcap_loop(reader_thread);
-  reader_thread->workflow->error_or_eof = 1;
+  __sync_fetch_and_add(&reader_thread->workflow->error_or_eof, 1);
   return NULL;
 }
 
 static int processing_threads_error_or_eof(void)
 {
   for (int i = 0; i < reader_thread_count; ++i) {
-    if (reader_threads[i].workflow->error_or_eof == 0) {
+    if (__sync_fetch_and_add(&reader_threads[i].workflow->error_or_eof, 0) == 0) {
       return 0;
     }
   }
@@ -994,6 +993,7 @@ static int processing_threads_error_or_eof(void)
 
 static int start_reader_threads(void)
 {
+#ifndef WIN32
   sigset_t thread_signal_set, old_signal_set;
 
   sigfillset(&thread_signal_set);
@@ -1003,6 +1003,7 @@ static int start_reader_threads(void)
     fprintf(stderr, "pthread_sigmask: %s\n", strerror(errno));
     return 1;
   }
+#endif
 
   for (int i = 0; i < reader_thread_count; ++i) {
     reader_threads[i].array_index = i;
@@ -1030,6 +1031,7 @@ static int start_reader_threads(void)
 
 static int stop_reader_threads(void)
 {
+  unsigned long long int total_packets_captured = 0;
   unsigned long long int total_packets_processed = 0;
   unsigned long long int total_l4_data_len = 0;
   unsigned long long int total_flows_captured = 0;
@@ -1047,6 +1049,10 @@ static int stop_reader_threads(void)
       continue;
     }
 
+    if (pthread_join(reader_threads[i].thread_id, NULL) != 0) {
+      fprintf(stderr, "pthread_join: %s\n", strerror(errno));
+    }
+
     total_packets_processed += reader_threads[i].workflow->packets_processed;
     total_l4_data_len += reader_threads[i].workflow->total_l4_data_len;
     total_flows_captured += reader_threads[i].workflow->total_active_flows;
@@ -1059,27 +1065,25 @@ static int stop_reader_threads(void)
 	   reader_threads[i].workflow->total_l4_data_len, reader_threads[i].workflow->total_active_flows,
 	   reader_threads[i].workflow->total_idle_flows, reader_threads[i].workflow->detected_flow_protocols);
   }
+
   /* total packets captured: same value for all threads as packet2thread distribution happens later */
-  printf("Total packets captured.: %llu\n",
-	 reader_threads[0].workflow->packets_captured);
-  printf("Total packets processed: %llu\n", total_packets_processed);
-  printf("Total layer4 data size.: %llu\n", total_l4_data_len);
-  printf("Total flows captured...: %llu\n", total_flows_captured);
-  printf("Total flows timed out..: %llu\n", total_flows_idle);
-  printf("Total flows detected...: %llu\n", total_flows_detected);
+  total_packets_captured = reader_threads[0].workflow->packets_captured;
 
   for (int i = 0; i < reader_thread_count; ++i) {
     if (reader_threads[i].workflow == NULL) {
       continue;
     }
 
-    if (pthread_join(reader_threads[i].thread_id, NULL) != 0) {
-      fprintf(stderr, "pthread_join: %s\n", strerror(errno));
-    }
-
     free_workflow(&reader_threads[i].workflow);
   }
 
+  printf("Total packets captured.: %llu\n", total_packets_captured);
+  printf("Total packets processed: %llu\n", total_packets_processed);
+  printf("Total layer4 data size.: %llu\n", total_l4_data_len);
+  printf("Total flows captured...: %llu\n", total_flows_captured);
+  printf("Total flows timed out..: %llu\n", total_flows_idle);
+  printf("Total flows detected...: %llu\n", total_flows_detected);
+
   return 0;
 }
 
@@ -1087,12 +1091,8 @@ static void sighandler(int signum)
 {
   fprintf(stderr, "Received SIGNAL %d\n", signum);
 
-  if (main_thread_shutdown == 0) {
-    main_thread_shutdown = 1;
-    if (stop_reader_threads() != 0) {
-      fprintf(stderr, "Failed to stop reader threads!\n");
-      exit(EXIT_FAILURE);
-    }
+  if (__sync_fetch_and_add(&main_thread_shutdown, 0) == 0) {
+    __sync_fetch_and_add(&main_thread_shutdown, 1);
   } else {
     fprintf(stderr, "Reader threads are already shutting down, please be patient.\n");
   }
@@ -1126,11 +1126,11 @@ int main(int argc, char ** argv)
 
   signal(SIGINT, sighandler);
   signal(SIGTERM, sighandler);
-  while (main_thread_shutdown == 0 && processing_threads_error_or_eof() == 0) {
+  while (__sync_fetch_and_add(&main_thread_shutdown, 0) == 0 && processing_threads_error_or_eof() == 0) {
     sleep(1);
   }
 
-  if (main_thread_shutdown == 0 && stop_reader_threads() != 0) {
+  if (stop_reader_threads() != 0) {
     fprintf(stderr, "%s: stop_reader_threads\n", argv[0]);
     return 1;
   }
