@@ -322,6 +322,17 @@ typedef enum {
   NDPI_MALICIOUS_JA3,
   NDPI_MALICIOUS_SHA1_CERTIFICATE,
   NDPI_DESKTOP_OR_FILE_SHARING_SESSION,
+  NDPI_TLS_UNCOMMON_ALPN,
+  NDPI_TLS_CERT_VALIDITY_TOO_LONG,
+  NDPI_TLS_SUSPICIOUS_EXTENSION,
+  NDPI_TLS_FATAL_ALERT,
+  NDPI_SUSPICIOUS_ENTROPY,
+  NDPI_CLEAR_TEXT_CREDENTIALS,
+  NDPI_DNS_LARGE_PACKET,
+  NDPI_DNS_FRAGMENTED,
+  NDPI_INVALID_CHARACTERS,
+  NDPI_POSSIBLE_EXPLOIT,
+  NDPI_TLS_CERTIFICATE_ABOUT_TO_EXPIRE,
 
   /* Leave this as last member */
   NDPI_MAX_RISK
@@ -433,11 +444,6 @@ struct ndpi_id_struct {
      to compare this, use:
   **/
   NDPI_PROTOCOL_BITMASK detected_protocol_bitmask;
-  /* NDPI_PROTOCOL_RTSP */
-  ndpi_ip_addr_t rtsp_ip_address;
-
-  /* NDPI_PROTOCOL_YAHOO */
-  uint32_t yahoo_video_lan_timer;
 
   /* NDPI_PROTOCOL_IRC_MAXPORT % 2 must be 0 */
   /* NDPI_PROTOCOL_IRC */
@@ -449,65 +455,31 @@ struct ndpi_id_struct {
   /* NDPI_PROTOCOL_GNUTELLA */
   uint32_t gnutella_ts;
 
-  /* NDPI_PROTOCOL_THUNDER */
-  uint32_t thunder_ts;
-
-  /* NDPI_PROTOCOL_RTSP */
-  uint32_t rtsp_timer;
-
-  /* NDPI_PROTOCOL_ZATTOO */
-  uint32_t zattoo_ts;
-
   /* NDPI_PROTOCOL_UNENCRYPTED_JABBER */
   uint32_t jabber_stun_or_ft_ts;
 
   /* NDPI_PROTOCOL_DIRECTCONNECT */
   uint32_t directconnect_last_safe_access_time;
 
-  /* NDPI_PROTOCOL_SOULSEEK */
-  uint32_t soulseek_last_safe_access_time;
-
   /* NDPI_PROTOCOL_DIRECTCONNECT */
   uint16_t detected_directconnect_port;
   uint16_t detected_directconnect_udp_port;
   uint16_t detected_directconnect_ssl_port;
 
-  /* NDPI_PROTOCOL_BITTORRENT */
-#define NDPI_BT_PORTS 8
-  uint16_t bt_port_t[NDPI_BT_PORTS];
-  uint16_t bt_port_u[NDPI_BT_PORTS];
-
   /* NDPI_PROTOCOL_UNENCRYPTED_JABBER */
 #define JABBER_MAX_STUN_PORTS 6
   uint16_t jabber_voice_stun_port[JABBER_MAX_STUN_PORTS];
   uint16_t jabber_file_transfer_port[2];
 
-  /* NDPI_PROTOCOL_GNUTELLA */
-  uint16_t detected_gnutella_port;
-
   /* NDPI_PROTOCOL_GNUTELLA */
   uint16_t detected_gnutella_udp_port1;
   uint16_t detected_gnutella_udp_port2;
 
-  /* NDPI_PROTOCOL_SOULSEEK */
-  uint16_t soulseek_listen_port;
-
   /* NDPI_PROTOCOL_IRC */
   uint8_t irc_number_of_port;
 
   /* NDPI_PROTOCOL_UNENCRYPTED_JABBER */
   uint8_t jabber_voice_stun_used_ports;
-
-  /* NDPI_PROTOCOL_SIP */
-  /* NDPI_PROTOCOL_YAHOO */
-  uint32_t yahoo_video_lan_dir:1;
-
-  /* NDPI_PROTOCOL_YAHOO */
-  uint32_t yahoo_conf_logged_in:1;
-  uint32_t yahoo_voice_conf_logged_in:1;
-
-  /* NDPI_PROTOCOL_RTSP */
-  uint32_t rtsp_ts_set:1;
 };
 
 struct ndpi_flow_tcp_struct {
@@ -517,18 +489,11 @@ struct ndpi_flow_tcp_struct {
   /* NDPI_PROTOCOL_MAIL_POP */
   uint16_t pop_command_bitmask;
 
-  /* NDPI_PROTOCOL_QQ */
-  uint16_t qq_nxt_len;
-
   /* NDPI_PROTOCOL_WHATSAPP */
   uint8_t wa_matched_so_far;
 
-  /* NDPI_PROTOCOL_TDS */
-  uint8_t tds_login_version;
-
   /* NDPI_PROTOCOL_IRC */
   uint8_t irc_stage;
-  uint8_t irc_port;
 
   /* NDPI_PROTOCOL_H323 */
   uint8_t h323_valid_packets;
@@ -542,38 +507,16 @@ struct ndpi_flow_tcp_struct {
   uint32_t irc_direction:2;
   uint32_t irc_0x1000_full:1;
 
-  /* NDPI_PROTOCOL_SOULSEEK */
-  uint32_t soulseek_stage:2;
-
-  /* NDPI_PROTOCOL_TDS */
-  uint32_t tds_stage:3;
-
   /* NDPI_PROTOCOL_USENET */
   uint32_t usenet_stage:2;
 
-  /* NDPI_PROTOCOL_IMESH */
-  uint32_t imesh_stage:4;
-
   /* NDPI_PROTOCOL_HTTP */
-  uint32_t http_setup_dir:2;
   uint32_t http_stage:2;
   uint32_t http_empty_line_seen:1;
-  uint32_t http_wait_for_retransmission:1;
 
   /* NDPI_PROTOCOL_GNUTELLA */
   uint32_t gnutella_stage:2;		       // 0 - 2
 
-  /* NDPI_CONTENT_MMS */
-  uint32_t mms_stage:2;
-
-  /* NDPI_PROTOCOL_YAHOO */
-  uint32_t yahoo_sip_comm:1;
-  uint32_t yahoo_http_proxy_stage:2;
-
-  /* NDPI_PROTOCOL_MSN */
-  uint32_t msn_stage:3;
-  uint32_t msn_ssl_ft:2;
-
   /* NDPI_PROTOCOL_SSH */
   uint32_t ssh_stage:3;
 
@@ -589,10 +532,8 @@ struct ndpi_flow_tcp_struct {
       unsigned buffer_len, buffer_used;
     } message;
 
-    void* srv_cert_fingerprint_ctx; /* SHA-1 */
-
     /* NDPI_PROTOCOL_TLS */
-    uint8_t hello_processed:1, certificate_processed:1, subprotocol_detected:1, fingerprint_set:1, _pad:4; 
+    uint8_t certificate_processed:1, fingerprint_set:1, _pad:6;
     uint8_t sha1_certificate_fingerprint[20], num_tls_blocks;
     int16_t tls_application_blocks_len[NDPI_MAX_NUM_TLS_APPL_BLOCKS];
   } tls;
@@ -600,8 +541,7 @@ struct ndpi_flow_tcp_struct {
   /* NDPI_PROTOCOL_POSTGRES */
   uint32_t postgres_stage:3;
 
-  /* NDPI_PROTOCOL_DIRECT_DOWNLOAD_LINK */
-  uint32_t ddlink_server_direction:1;
+  /* Part of the TCP header. */
   uint32_t seen_syn:1;
   uint32_t seen_syn_ack:1;
   uint32_t seen_ack:1;
@@ -618,9 +558,6 @@ struct ndpi_flow_tcp_struct {
   /* NDPI_PROTOCOL_WORLDOFWARCRAFT */
   uint32_t wow_stage:2;
 
-  /* NDPI_PROTOCOL_HTTP_APPLICATION_VEOHTV */
-  uint32_t veoh_tv_stage:2;
-
   /* NDPI_PROTOCOL_SHOUTCAST */
   uint32_t shoutcast_stage:2;
 
@@ -660,12 +597,6 @@ struct ndpi_flow_tcp_struct {
 };
 
 struct ndpi_flow_udp_struct {
-  /* NDPI_PROTOCOL_SNMP */
-  uint32_t snmp_msg_id;
-
-  /* NDPI_PROTOCOL_SNMP */
-  uint32_t snmp_stage:2;
-
   /* NDPI_PROTOCOL_PPSTREAM */
   uint32_t ppstream_stage:3;		  // 0 - 7
 
@@ -681,9 +612,6 @@ struct ndpi_flow_udp_struct {
   /* NDPI_PROTOCOL_XBOX */
   uint32_t xbox_stage:1;
 
-  /* NDPI_PROTOCOL_WINDOWS_UPDATE */
-  uint32_t wsus_stage:1;
-
   /* NDPI_PROTOCOL_SKYPE */
   uint8_t skype_packet_id;
 
@@ -704,6 +632,21 @@ struct ndpi_flow_udp_struct {
   /* NDPI_PROTOCOL_WIREGUARD */
   uint8_t wireguard_stage;
   uint32_t wireguard_peer_index[2];
+
+  /* NDPI_PROTOCOL_QUIC */
+  u_int8_t *quic_reasm_buf;
+  u_int32_t quic_reasm_buf_len;
+
+  /* NDPI_PROTOCOL_CSGO */
+  uint8_t csgo_strid[18],csgo_state,csgo_s2;
+  uint32_t csgo_id2;
+
+  /* NDPI_PROTOCOL_RDP */
+  u_int8_t rdp_to_srv[3], rdp_from_srv[3], rdp_to_srv_pkts, rdp_from_srv_pkts;
+
+  /* NDPI_PROTOCOL_IMO */
+  uint8_t imo_last_one_byte_pkt, imo_last_byte;
+
 };
 
 struct ndpi_int_one_line_struct {
@@ -722,7 +665,6 @@ struct ndpi_packet_struct {
   uint64_t current_time_ms;
 
   uint16_t detected_protocol_stack[2];
-  uint16_t protocol_stack_info;
 
   struct ndpi_int_one_line_struct line[64];
   /* HTTP headers */
@@ -746,19 +688,13 @@ struct ndpi_packet_struct {
   uint8_t http_num_headers; /* number of found (valid) header lines in HTTP request or response */
 
   uint16_t l3_packet_len;
-  uint16_t l4_packet_len;
   uint16_t payload_packet_len;
-  uint16_t actual_payload_len;
-  uint16_t num_retried_bytes;
   uint16_t parsed_lines;
-  uint16_t parsed_unix_lines;
   uint16_t empty_line_position;
   uint8_t tcp_retransmission;
-  uint8_t l4_protocol;
 
-  uint8_t tls_certificate_detected:4, tls_certificate_num_checks:4;
   uint8_t packet_lines_parsed_complete:1,
-  packet_direction:1, empty_line_position_set:1, pad:5;
+  packet_direction:1, empty_line_position_set:1, http_check_content:1, pad:4;
 };
 
 struct ndpi_detection_module_struct;
@@ -952,8 +888,6 @@ struct ndpi_detection_module_struct {
 
   /* HTTP/DNS/HTTPS host matching */
   ndpi_automa host_automa,                     /* Used for DNS/HTTPS */
-    content_automa,                            /* Used for HTTP subprotocol_detection */
-    subprotocol_automa,                        /* Used for HTTP subprotocol_detection */
     bigrams_automa, impossible_bigrams_automa; /* TOR */
   /* IMPORTANT: please update ndpi_finalize_initalization() whenever you add a new automa */
 
@@ -970,18 +904,7 @@ struct ndpi_detection_module_struct {
   uint32_t irc_timeout;
   /* gnutella parameters */
   uint32_t gnutella_timeout;
-  /* thunder parameters */
-  uint32_t thunder_timeout;
-  /* SoulSeek parameters */
-  uint32_t soulseek_connection_ip_tick_timeout;
-  /* rtsp parameters */
-  uint32_t rtsp_connection_timeout;
   /* rstp */
-  uint32_t orb_rstp_ts_timeout;
-  /* yahoo */
-  uint8_t yahoo_detect_http_connections;
-  uint32_t yahoo_lan_video_timeout;
-  uint32_t zattoo_connection_timeout;
   uint32_t jabber_stun_timeout;
   uint32_t jabber_file_transfer_timeout;
   uint8_t ip_version_limit;
@@ -1008,6 +931,9 @@ struct ndpi_detection_module_struct {
 
   uint8_t direction_detect_disable:1, /* disable internal detection of packet direction */
     _pad:7;
+
+  /* Current packet */
+  struct ndpi_packet_struct packet;
 };
 
 #define NDPI_CIPHER_SAFE                        0
@@ -1022,7 +948,6 @@ typedef enum {
 
 struct ndpi_flow_struct {
   uint16_t detected_protocol_stack[2];
-  uint16_t protocol_stack_info;
   /* init parameter, internal used to set up timestamp,... */
   uint16_t guessed_protocol_id, guessed_host_protocol_id, guessed_category, guessed_header_category;
   uint8_t l4_proto, protocol_id_already_guessed:1, host_already_guessed:1,
@@ -1049,13 +974,8 @@ struct ndpi_flow_struct {
   /* Place textual flow info here */
   char flow_extra_info[16];
 
-  /*
-    Pointer to src or dst that identifies the
-    server of this connection
-  */
-  struct ndpi_id_struct *server_id;
-  /* HTTP host or DNS query */
-  uint8_t host_server_name[240];
+  char host_server_name[80];
+
   uint8_t initial_binary_bytes[8], initial_binary_bytes_len;
   uint8_t risk_checked;
   ndpi_risk risk; /* Issues found with this flow [bitmask of ndpi_risk] */
@@ -1069,11 +989,11 @@ struct ndpi_flow_struct {
   */
   struct {
     ndpi_http_method method;
-    char *url, *content_type, *user_agent;
-    uint8_t num_request_headers, num_response_headers;
     uint8_t request_version; /* 0=1.0 and 1=1.1. Create an enum for this? */
     uint16_t response_status_code; /* 200, 404, etc. */
-    uint8_t detected_os[32]; /* Via HTTP/QUIC User-Agent */
+    char *url, *content_type, *user_agent;
+    char *detected_os; /* Via HTTP/QUIC User-Agent */
+    char *nat_ip; /* Via HTTP X-Forwarded-For */
 
   } http;
 
@@ -1086,6 +1006,18 @@ struct ndpi_flow_struct {
     char *pktbuf;
     uint16_t pktbuf_maxlen, pktbuf_currlen;
   } kerberos_buf;
+
+  struct {
+    u_int8_t num_udp_pkts, num_binding_requests;
+    u_int16_t num_processed_pkts;
+  } stun;
+
+  /* TODO: something clever to save memory */
+  struct {
+      uint8_t auth_found:1, auth_failed:1, auth_tls:1, auth_done:1, _pad:4;
+      char username[32], password[16];
+  } ftp_imap_pop_smtp;
+
   union {
     /* the only fields useful for nDPI and ntopng */
     struct {
@@ -1104,37 +1036,25 @@ struct ndpi_flow_struct {
     } kerberos;
 
     struct {
-      struct {
-      char ssl_version_str[12];
-      uint16_t ssl_version, server_names_len;
-      char client_requested_server_name[64], *server_names,
-      *alpn, *tls_supported_versions, *issuerDN, *subjectDN;
+      char *server_names, *alpn, *tls_supported_versions, *issuerDN, *subjectDN;
       uint32_t notBefore, notAfter;
       char ja3_client[33], ja3_server[33];
       uint16_t server_cipher;
+      u_int8_t sha1_certificate_fingerprint[20];
+      u_int8_t hello_processed:1, subprotocol_detected:1, _pad:6;
       struct {
         uint16_t cipher_suite;
         char *esni;
       } encrypted_sni;
       ndpi_cipher_weakness server_unsafe_cipher;
-      } ssl;
-
-      struct {
-      uint8_t num_udp_pkts, num_processed_pkts, num_binding_requests;
-      } stun;
-
-      /* We can have STUN over SSL/TLS thus they need to live together */
-    } stun_ssl;
+      uint16_t ssl_version, server_names_len;
+    } tls_quic;
 
     struct {
       char client_signature[48], server_signature[48];
       char hassh_client[33], hassh_server[33];
     } ssh;
 
-    struct {
-      uint8_t last_one_byte_pkt, last_byte;
-    } imo;
-
     struct {
       uint8_t username_detected:1, username_found:1,
       password_detected:1, password_found:1,
@@ -1151,16 +1071,6 @@ struct ndpi_flow_struct {
       char version[32];
     } ubntac2;
 
-    struct {
-      /* Via HTTP X-Forwarded-For */
-      uint8_t nat_ip[24];
-    } http;
-
-    struct {
-      uint8_t auth_found:1, auth_failed:1, _pad:5;
-      char username[16], password[16];
-    } ftp_imap_pop_smtp;
-
     struct {
       /* Bittorrent hash */
       uint8_t hash[20];
@@ -1191,24 +1101,15 @@ struct ndpi_flow_struct {
   /* NDPI_PROTOCOL_DIRECTCONNECT */
   uint8_t directconnect_stage:2;	      // 0 - 1
 
-  /* NDPI_PROTOCOL_YAHOO */
-  uint8_t sip_yahoo_voice:1;
-
   /* NDPI_PROTOCOL_HTTP */
   uint8_t http_detected:1;
 
   /* NDPI_PROTOCOL_RTSP */
-  uint8_t rtsprdt_stage:2, rtsp_control_flow:1;
-
-  /* NDPI_PROTOCOL_YAHOO */
-  uint8_t yahoo_detection_finished:2;
+  uint8_t rtsprdt_stage:2;
 
   /* NDPI_PROTOCOL_ZATTOO */
   uint8_t zattoo_stage:3;
 
-  /* NDPI_PROTOCOL_QQ */
-  uint8_t qq_stage:3;
-
   /* NDPI_PROTOCOL_THUNDER */
   uint8_t thunder_stage:2;		        // 0 - 3
 
@@ -1243,16 +1144,13 @@ struct ndpi_flow_struct {
   uint8_t ovpn_session_id[8];
   uint8_t ovpn_counter;
 
+  /* Flow key used to search a match into the mining cache */
+  u_int32_t key_mining_cache;
+
   /* NDPI_PROTOCOL_TINC */
   uint8_t tinc_state;
   struct tinc_cache_entry tinc_cache_entry;
 
-  /* NDPI_PROTOCOL_CSGO */
-  uint8_t csgo_strid[18],csgo_state,csgo_s2;
-  uint32_t csgo_id2;
-  /* internal structures to save functions calls */
-  struct ndpi_packet_struct packet;
-  struct ndpi_flow_struct *flow;
   struct ndpi_id_struct *src;
   struct ndpi_id_struct *dst;
 };
