@@ -251,22 +251,12 @@ NDPIDetectionModuleStruct._fields_ = [
     ("ndpi_num_supported_protocols", c_uint),
     ("ndpi_num_custom_protocols", c_uint),
     ("host_automa", NDPIAutoma),
-    ("content_automa", NDPIAutoma),
-    ("subprotocol_automa", NDPIAutoma),
     ("bigrams_automa", NDPIAutoma),
     ("impossible_bigrams_automa", NDPIAutoma),
     ("custom_categories", CustomCategories),
     ("protocols_ptree", c_void_p),
     ("irc_timeout", c_uint32),
     ("gnutella_timeout", c_uint32),
-    ("battlefield_timeout", c_uint32),
-    ("thunder_timeout", c_uint32),
-    ("soulseek_connection_ip_tick_timeout", c_uint32),
-    ("rtsp_connection_timeout", c_uint32),
-    ("tvants_connection_timeout", c_uint32),
-    ("orb_rstp_ts_timeout", c_uint32),
-    ("yahoo_detect_http_connections", c_uint8),
-    ("yahoo_lan_video_timeout", c_uint32),
     ("zattoo_connection_timeout", c_uint32),
     ("jabber_stun_timeout", c_uint32),
     ("jabber_file_transfer_timeout", c_uint32),
@@ -279,11 +269,9 @@ NDPIDetectionModuleStruct._fields_ = [
     ("tinc_cache", POINTER(Cache)),
     ("proto_defaults", NDPIProtoDefaultsT * (ndpi.ndpi_wrap_ndpi_max_supported_protocols() +
                                              ndpi.ndpi_wrap_ndpi_max_num_custom_protocols())),
-    ("http_dont_dissect_response", c_uint8, 1),
-    ("dns_dont_dissect_response", c_uint8, 1),
     ("direction_detect_disable", c_uint8, 1),
-    ("disable_metadata_export", c_uint8, 1),
-    ("hyperscan", c_void_p)
+    ('_pad', c_uint8, 7),
+    ('packet', NDPIPacketStruct),
 ]
 
 
@@ -312,38 +300,24 @@ class NDPIIpAddrT(Union):
 class NDPIIdStruct(Structure):
     _fields_ = [
         ('detected_protocol_bitmask', NDPIProtocolBitMask),
-        ('rtsp_ip_address', NDPIIpAddrT),
-        ('yahoo_video_lan_timer', c_uint32),
         ('irc_port', c_uint16 * 8),
         ('last_time_port_used', c_uint32 * 8),
         ('irc_ts', c_uint32),
         ('gnutella_ts', c_uint32),
         ('battlefield_ts', c_uint32),
-        ('thunder_ts', c_uint32),
         ('rtsp_timer', c_uint32),
-        ('oscar_last_safe_access_time', c_uint32),
-        ('zattoo_ts', c_uint32),
         ('jabber_stun_or_ft_ts', c_uint32),
         ('directconnect_last_safe_access_time', c_uint32),
-        ('soulseek_last_safe_access_time', c_uint32),
         ('detected_directconnect_port', c_uint16),
         ('detected_directconnect_udp_port', c_uint16),
         ('detected_directconnect_ssl_port', c_uint16),
-        ('bt_port_t', c_uint16 * 8),
-        ('bt_port_u', c_uint16 * 8),
         ('jabber_voice_stun_port', c_uint16 * 6),
         ('jabber_file_transfer_port', c_uint16 * 2),
-        ('detected_gnutella_port', c_uint16),
         ('detected_gnutella_udp_port1', c_uint16),
         ('detected_gnutella_udp_port2', c_uint16),
-        ('soulseek_listen_port', c_uint16),
         ('irc_number_of_port', c_uint8),
         ('oscar_ssl_session_id', c_uint8 * 33),
         ('jabber_voice_stun_used_ports', c_uint8),
-        ('yahoo_video_lan_dir', c_uint32, 1),
-        ('yahoo_conf_logged_in', c_uint32, 1),
-        ('yahoo_voice_conf_logged_in', c_uint32, 1),
-        ('rtsp_ts_set', c_uint32, 1),
     ]
 
 
@@ -352,35 +326,21 @@ class NDPIFlowTcpStruct(Structure):
     _fields_ = [
         ('smtp_command_bitmask', c_uint16),
         ('pop_command_bitmask', c_uint16),
-        ('qq_nxt_len', c_uint16),
         ('wa_matched_so_far', c_uint8),
-        ('tds_login_version', c_uint8),
         ('irc_stage', c_uint8),
-        ('irc_port', c_uint8),
         ('h323_valid_packets', c_uint8),
         ('gnutella_msg_id', c_uint8 * 3),
         ('irc_3a_counter', c_uint32, 3),
         ('irc_stage2', c_uint32, 5),
         ('irc_direction', c_uint32, 2),
         ('irc_0x1000_full', c_uint32, 1),
-        ('soulseek_stage', c_uint32, 2),
-        ('tds_stage', c_uint32, 3),
         ('usenet_stage', c_uint32, 2),
-        ('imesh_stage', c_uint32, 4),
-        ('http_setup_dir', c_uint32, 2),
         ('http_stage', c_uint32, 2),
         ('http_empty_line_seen', c_uint32, 1),
-        ('http_wait_for_retransmission', c_uint32, 1),
         ('gnutella_stage', c_uint32, 2),
-        ('mms_stage', c_uint32, 2),
-        ('yahoo_sip_comm', c_uint32, 1),
-        ('yahoo_http_proxy_stage', c_uint32, 2),
-        ('msn_stage', c_uint32, 3),
-        ('msn_ssl_ft', c_uint32, 2),
         ('ssh_stage', c_uint32, 3),
         ('vnc_stage', c_uint32, 2),
         ('telnet_stage', c_uint32, 2),
-        ('tls_srv_cert_fingerprint_ctx', c_void_p),
         ('tls_seen_client_cert', c_uint8, 1),
         ('tls_seen_server_cert', c_uint8, 1),
         ('tls_seen_certificate', c_uint8, 1),
@@ -391,7 +351,6 @@ class NDPIFlowTcpStruct(Structure):
         ('tls_fingerprint_len', c_int16),
         ('tls_sha1_certificate_fingerprint', c_uint8 * 20),
         ('postgres_stage', c_uint32, 3),
-        ('ddlink_server_direction', c_uint32, 1),
         ('seen_syn', c_uint32, 1),
         ('seen_syn_ack', c_uint32, 1),
         ('seen_ack', c_uint32, 1),
@@ -399,7 +358,6 @@ class NDPIFlowTcpStruct(Structure):
         ('dofus_stage', c_uint32, 1),
         ('fiesta_stage', c_uint32, 2),
         ('wow_stage', c_uint32, 2),
-        ('veoh_tv_stage', c_uint32, 2),
         ('shoutcast_stage', c_uint32, 2),
         ('rtp_special_packets_seen', c_uint32, 1),
         ('mail_pop_stage', c_uint32, 2),
@@ -420,16 +378,11 @@ class NDPIFlowTcpStruct(Structure):
 class NDPIFlowUdpStruct(Structure):
     _pack_ = 1
     _fields_ = [
-        ('battlefield_msg_id', c_uint32),
-        ('snmp_msg_id', c_uint32),
-        ('battlefield_stage', c_uint32, 3),
-        ('snmp_stage', c_uint32, 2),
         ('ppstream_stage', c_uint32, 3),
         ('halflife2_stage', c_uint32, 2),
         ('tftp_stage', c_uint32, 2),
         ('aimini_stage', c_uint32, 5),
         ('xbox_stage', c_uint32, 1),
-        ('wsus_stage', c_uint32, 1),
         ('skype_packet_id', c_uint8),
         ('teamviewer_stage', c_uint8),
         ('eaq_pkt_id', c_uint8),
@@ -439,6 +392,18 @@ class NDPIFlowUdpStruct(Structure):
         ('memcached_matches', c_uint8),
         ('wireguard_stage', c_uint8),
         ('wireguard_peer_index', c_uint32 * 2),
+        ('quic_reasm_buf', POINTER(c_uint8)),
+        ('quic_reasm_buf_len', c_uint32),
+        ('csgo_strid', c_uint8 * 18),
+        ('csgo_state', c_uint8),
+        ('csgo_s2', c_uint8),
+        ('csgo_id2', c_uint32),
+        ('rdp_to_srv', c_uint8 * 3),
+        ('rdp_from_srv', c_uint8 * 3),
+        ('rdp_to_srv_pkts,', c_uint8),
+        ('rdp_from_srv_pkts', c_uint8),
+        ('imo_last_one_byte_pkt,', c_uint8),
+        ('imo_last_byte', c_uint8),
     ]
 
 
@@ -449,13 +414,13 @@ class L4(Union):
 class Http(Structure):
     _fields_ = [
         ("method", c_int),
-        ("url", c_char_p),
-        ("content_type", c_char_p),
-        ("num_request_headers", c_uint8),
-        ("num_response_headers", c_uint8),
         ("request_version", c_uint8),
         ("response_status_code", c_uint16),
-        ("detected_os", c_char * 32),
+        ("url", c_char_p),
+        ("content_type", c_char_p),
+        ("user_agent", c_char_p),
+        ("detected_os", c_char_p),
+        ("nat_ip", c_char_p),
     ]
 
 
@@ -482,7 +447,7 @@ class Kerberos(Structure):
                 ("realm", c_char * 24)]
 
 
-class Ssl(Structure):
+class QuicSsl(Structure):
     _fields_ = [
         ("ssl_version", c_uint16),
         ("client_certificate", c_char * 64),
@@ -505,10 +470,6 @@ class Stun(Structure):
     ]
 
 
-class StunSsl(Structure):
-    _fields_ = [("ssl", Ssl), ("stun", Stun)]
-
-
 class Ssh(Structure):
     _fields_ = [
         ("client_signature", c_char * 48),
@@ -518,13 +479,6 @@ class Ssh(Structure):
     ]
 
 
-class Imo(Structure):
-    _fields_ = [
-        ("last_one_byte_pkt", c_uint8),
-        ("last_byte", c_uint8)
-    ]
-
-
 class Mdns(Structure):
     _fields_ = [("answer", c_char * 96)]
 
@@ -533,12 +487,17 @@ class Ubntac2(Structure):
     _fields_ = [("version", c_char * 32)]
 
 
-class Http2(Structure):
+class FtpImapPopSmtp(Structure):
     _fields_ = [
-        ("nat_ip", c_char * 24)
+        ("auth_found", c_uint8, 1),
+        ("auth_failed", c_uint8, 1),
+        ("auth_tls", c_uint8, 1),
+        ("auth_done", c_uint8, 1),
+        ("_pad", c_uint8, 4),
+        ("username", c_char * 32),
+        ("password", c_char * 16)
     ]
 
-
 class Bittorrent(Structure):
     _fields_ = [("hash", c_char * 20)]
 
@@ -554,12 +513,10 @@ class Protos(Union):
     _fields_ = [
         ("dns", Dns),
         ("kerberos", Kerberos),
-        ("stun_ssl", StunSsl),
+        ("quic_ssl", QuicSsl),
         ("ssh", Ssh),
-        ("imo", Imo),
         ("mdns", Mdns),
         ("ubntac2", Ubntac2),
-        ("http", Http2),
         ("bittorrent", Bittorrent),
         ("dhcp", Dhcp)
     ]
@@ -649,13 +606,6 @@ class NDPIUdpHdr(Structure):
     ]
 
 
-class NDPIPacketStructStack(Structure):
-    _pack_ = 1
-    _fields_ = [
-        ('protocol_stack_info', c_uint16)
-    ]
-
-
 class NDPIPacketStruct(Structure):
     _fields_ = [
         ('iph', POINTER(NDPIIphdr)),
@@ -664,10 +614,8 @@ class NDPIPacketStruct(Structure):
         ('udp', POINTER(NDPIUdpHdr)),
         ('generic_l4_ptr', POINTER(c_uint8)),
         ('payload', POINTER(c_uint8)),
-        ('tick_timestamp', c_uint32),
-        ('tick_timestamp_l', c_uint64),
+        ('current_timestamp_ms', c_uint64),
         ('detected_protocol_stack', c_uint16 * ndpi.ndpi_wrap_ndpi_procol_size()),
-        ('ndpi_packet_stack', NDPIPacketStructStack),
         ('line', NDPIIntOneLineStruct * 64),
         ('host_line', NDPIIntOneLineStruct),
         ('forwarded_line', NDPIIntOneLineStruct),
@@ -687,21 +635,15 @@ class NDPIPacketStruct(Structure):
         ('http_response', NDPIIntOneLineStruct),
         ('http_num_headers', c_uint8),
         ('l3_packet_len', c_uint16),
-        ('l4_packet_len', c_uint16),
         ('payload_packet_len', c_uint16),
-        ('actual_payload_len', c_uint16),
-        ('num_retried_bytes', c_uint16),
         ('parsed_lines', c_uint16),
-        ('parsed_unix_lines', c_uint16),
         ('empty_line_position', c_uint16),
         ('tcp_retransmission', c_uint8),
-        ('l4_protocol', c_uint8),
-        ('ssl_certificate_detected', c_uint8, 4),
-        ('ssl_certificate_num_checks', c_uint8, 4),
         ('packet_lines_parsed_complete', c_uint8, 1),
         ('packet_direction', c_uint8, 1),
         ('empty_line_position_set', c_uint8, 1),
-        ('pad', c_uint8, 5),
+        ('http_check_content', c_uint8, 1),
+        ('pad', c_uint8, 4),
     ]
 
 
@@ -709,7 +651,6 @@ class NDPIFlowStructStack(Structure):
     _pack_ = 1
     _fields_ = [
         ("detected_protocol_stack", c_uint16 * ndpi.ndpi_wrap_ndpi_procol_size()),
-        ("protocol_stack_info", c_uint16)
     ]
 
 
@@ -732,9 +673,10 @@ NDPIFlowStruct._fields_ = [
     ("num_processed_pkts", c_uint8),
     ("extra_packets_func", CFUNCTYPE(c_int, POINTER(NDPIDetectionModuleStruct), POINTER(NDPIFlowStruct))),
     ("l4", L4),
-    ("server_id", POINTER(NDPIIdStruct)),
-    ("host_server_name", c_ubyte * 256),
+    ("host_server_name", c_char * 80),
     ("http", Http),
+    ("stun", Stun),
+    ("ftp_imap_pop_smtp", FtpImapPopSmtp),
     ("protos", Protos),
     ("excluded_protocol_bitmask", NDPIProtocolBitMask),
     ("category", c_int),
@@ -745,15 +687,11 @@ NDPIFlowStruct._fields_ = [
     ('byte_counter', c_uint16 * 2),
     ('bittorrent_stage', c_uint8),
     ('directconnect_stage', c_uint8, 2),
-    ('sip_yahoo_voice', c_uint8, 1),
     ('http_detected', c_uint8, 1),
     ('http_upper_protocol', c_uint16),
     ('http_lower_protocol', c_uint16),
     ('rtsprdt_stage', c_uint8, 2),
-    ('rtsp_control_flow', c_uint8, 1),
-    ('yahoo_detection_finished', c_uint8, 2),
     ('zattoo_stage', c_uint8, 3),
-    ('qq_stage', c_uint8, 3),
     ('thunder_stage', c_uint8, 2),
     ('oscar_ssl_voice_stage', c_uint8, 3),
     ('oscar_video_voice', c_uint8, 1),
@@ -773,16 +711,9 @@ NDPIFlowStruct._fields_ = [
     ('starcraft_udp_stage', c_uint8, 3),
     ('ovpn_session_id', c_uint8 * 8),
     ('ovpn_counter', c_uint8),
+    ('key_mining_cache', c_uint32),
     ('tinc_state', c_uint8),
     ('TincCacheEntry', TincCacheEntry),
-    ('csgo_strid', c_uint8 * 18),
-    ('csgo_state', c_uint8),
-    ('csgo_s2', c_uint8),
-    ('csgo_id2', c_uint32),
-    ('kxun_counter', c_uint16),
-    ('iqiyi_counter', c_uint16),
-    ('packet', NDPIPacketStruct),
-    ('flow', POINTER(NDPIFlowStruct)),
     ('src', POINTER(NDPIIdStruct)),
     ('dst', POINTER(NDPIIdStruct))
 ]
