@@ -1,7 +1,7 @@
 /*
  * ndpi_typedefs.h
  *
- * Copyright (C) 2011-21 - ntop.org
+ * Copyright (C) 2011-22 - ntop.org
  *
  * This file is part of nDPI, an open source deep packet inspection
  * library based on the OpenDPI and PACE technology by ipoque GmbH
@@ -31,10 +31,6 @@
 /* Used by both nDPI core and patricia code under third-party */
 #include "ndpi_patricia_typedefs.h"
 
-#ifdef HAVE_MAXMINDDB
-#include <maxminddb.h>
-#endif
-
 /* NDPI_LOG_LEVEL */
 typedef enum {
 	      NDPI_LOG_ERROR,
@@ -64,9 +60,18 @@ typedef enum {
   When the typedef below is modified don't forget to update
   - nDPI/wireshark/ndpi.lua
   - ndpi_risk2str (in ndpi_utils.c)
-  - https://github.com/ntop/ntopng/blob/dev/scripts/lua/modules/flow_risk_utils.lua
+  - doc/flow_risks.rst
   - ndpi_risk_enum (in python/ndpi.py)
-  - ndpi_known_risks (ndpi_utils.c)
+  - ndpi_known_risks (ndpi_main.c)
+
+  To make sure the risk is also seen by ntopng:
+  1. Add a new flow alert key to the enum FlowAlertTypeEnum in include/ntop_typedefs.h
+  2. Add the very same flow alert key to the table flow_alert_keys in scripts/lua/modules/alert_keys/flow_alert_keys.lua
+  3. Add the risk to the array risk_enum_to_alert_type in src/FlowRiskAlerts.cpp
+     - To initialize .alert_type use the flow alert key added in 1. and an AlertCategory
+     - To initialize .alert_lua_name use a unique string
+
+  Example: https://github.com/ntop/ntopng/commit/aecc1e3e6505a0522439dbb2b295a3703d3d0f9a
  */
 typedef enum {
   NDPI_NO_RISK = 0,
@@ -102,7 +107,15 @@ typedef enum {
   NDPI_DESKTOP_OR_FILE_SHARING_SESSION, /* 30 */
   NDPI_TLS_UNCOMMON_ALPN,
   NDPI_TLS_CERT_VALIDITY_TOO_LONG,
-  NDPI_TLS_EXTENSION_SUSPICIOUS,
+  NDPI_TLS_SUSPICIOUS_EXTENSION,
+  NDPI_TLS_FATAL_ALERT,
+  NDPI_SUSPICIOUS_ENTROPY,
+  NDPI_CLEAR_TEXT_CREDENTIALS,
+  NDPI_DNS_LARGE_PACKET,
+  NDPI_DNS_FRAGMENTED,
+  NDPI_INVALID_CHARACTERS,
+  NDPI_POSSIBLE_EXPLOIT, /* Log4J and other exploits */
+  NDPI_TLS_CERTIFICATE_ABOUT_TO_EXPIRE,
 
   /* Leave this as last member */
   NDPI_MAX_RISK /* must be <= 63 due to (**) */
@@ -138,7 +151,6 @@ typedef struct {
   risk_percentage default_client_risk_pctg; /* 0-100 */
 } ndpi_risk_info;
 
-
 /* NDPI_VISIT */
 typedef enum {
    ndpi_preorder,
@@ -478,50 +490,10 @@ struct ndpi_vxlanhdr {
 
 typedef struct message {
   u_int8_t *buffer;
-  u_int buffer_len, buffer_used, max_expected;
+  u_int buffer_len, buffer_used;
   u_int32_t next_seq[2]; /* Directions */
 } message_t;
 
-/* NDPI_PROTOCOL_BITTORRENT */
-typedef struct spinlock {
-  volatile int    val;
-} spinlock_t;
-
-typedef struct atomic {
-  volatile int counter;
-} atomic_t;
-
-struct hash_ip4p_node {
-  struct hash_ip4p_node   *next,*prev;
-  time_t                  lchg;
-  u_int16_t               port,count:12,flag:4;
-  u_int32_t               ip;
-  // + 12 bytes for ipv6
-};
-
-struct hash_ip4p {
-  struct hash_ip4p_node   *top;
-  spinlock_t              lock;
-  size_t                  len;
-};
-
-struct hash_ip4p_table {
-  size_t                  size;
-  int			  ipv6;
-  spinlock_t              lock;
-  atomic_t                count;
-  struct hash_ip4p        tbl;
-};
-
-struct bt_announce {              // 192 bytes
-  u_int32_t		hash[5];
-  u_int32_t		ip[4];
-  u_int32_t		time;
-  u_int16_t		port;
-  u_int8_t		name_len,
-    name[192 - 4*10 - 2 - 1];     // 149 bytes
-};
-
 /* NDPI_PROTOCOL_TINC */
 #define TINC_CACHE_MAX_SIZE 10
 
@@ -558,112 +530,31 @@ struct ndpi_lru_cache {
   struct ndpi_lru_cache_entry *entries;
 };
 
-struct ndpi_id_struct {
-  /**
-     detected_protocol_bitmask:
-     access this bitmask to find out whether an id has used skype or not
-     if a flag is set here, it will not be reset
-     to compare this, use:
-  **/
-  NDPI_PROTOCOL_BITMASK detected_protocol_bitmask;
-  /* NDPI_PROTOCOL_RTSP */
-  ndpi_ip_addr_t rtsp_ip_address;
-
-  /* NDPI_PROTOCOL_YAHOO */
-  u_int32_t yahoo_video_lan_timer;
-
-  /* NDPI_PROTOCOL_IRC_MAXPORT % 2 must be 0 */
-  /* NDPI_PROTOCOL_IRC */
-#define NDPI_PROTOCOL_IRC_MAXPORT 8
-  u_int16_t irc_port[NDPI_PROTOCOL_IRC_MAXPORT];
-  u_int32_t last_time_port_used[NDPI_PROTOCOL_IRC_MAXPORT];
-  u_int32_t irc_ts;
-
-  /* NDPI_PROTOCOL_GNUTELLA */
-  u_int32_t gnutella_ts;
-
-  /* NDPI_PROTOCOL_THUNDER */
-  u_int32_t thunder_ts;
-
-  /* NDPI_PROTOCOL_RTSP */
-  u_int32_t rtsp_timer;
-
-  /* NDPI_PROTOCOL_ZATTOO */
-  u_int32_t zattoo_ts;
-
-  /* NDPI_PROTOCOL_JABBER */
-  u_int32_t jabber_stun_or_ft_ts;
-
-  /* NDPI_PROTOCOL_DIRECTCONNECT */
-  u_int32_t directconnect_last_safe_access_time;
-
-  /* NDPI_PROTOCOL_SOULSEEK */
-  u_int32_t soulseek_last_safe_access_time;
-
-  /* NDPI_PROTOCOL_DIRECTCONNECT */
-  u_int16_t detected_directconnect_port;
-  u_int16_t detected_directconnect_udp_port;
-  u_int16_t detected_directconnect_ssl_port;
-
-  /* NDPI_PROTOCOL_BITTORRENT */
-#define NDPI_BT_PORTS 8
-  u_int16_t bt_port_t[NDPI_BT_PORTS];
-  u_int16_t bt_port_u[NDPI_BT_PORTS];
-
-  /* NDPI_PROTOCOL_JABBER */
-#define JABBER_MAX_STUN_PORTS 6
-  u_int16_t jabber_voice_stun_port[JABBER_MAX_STUN_PORTS];
-  u_int16_t jabber_file_transfer_port[2];
-
-  /* NDPI_PROTOCOL_GNUTELLA */
-  u_int16_t detected_gnutella_port;
-
-  /* NDPI_PROTOCOL_GNUTELLA */
-  u_int16_t detected_gnutella_udp_port1;
-  u_int16_t detected_gnutella_udp_port2;
-
-  /* NDPI_PROTOCOL_SOULSEEK */
-  u_int16_t soulseek_listen_port;
-
-  /* NDPI_PROTOCOL_IRC */
-  u_int8_t irc_number_of_port;
-
-  /* NDPI_PROTOCOL_JABBER */
-  u_int8_t jabber_voice_stun_used_ports;
-
-  /* NDPI_PROTOCOL_SIP */
-  /* NDPI_PROTOCOL_YAHOO */
-  u_int32_t yahoo_video_lan_dir:1;
-
-  /* NDPI_PROTOCOL_YAHOO */
-  u_int32_t yahoo_conf_logged_in:1;
-  u_int32_t yahoo_voice_conf_logged_in:1;
-
-  /* NDPI_PROTOCOL_RTSP */
-  u_int32_t rtsp_ts_set:1;
-};
-
 /* ************************************************** */
 
 struct ndpi_flow_tcp_struct {
+
+  /* NDPI_PROTOCOL_MAIL_SMTP */
+  /* NDPI_PROTOCOL_MAIL_POP */
+  /* NDPI_PROTOCOL_MAIL_IMAP */
+  /* NDPI_PROTOCOL_MAIL_FTP */
+  /* TODO: something clever to save memory */
+  struct {
+    u_int8_t auth_found:1, auth_failed:1, auth_tls:1, auth_done:1, _pad:4;
+    char username[32], password[16];
+  } ftp_imap_pop_smtp;
+
   /* NDPI_PROTOCOL_MAIL_SMTP */
   u_int16_t smtp_command_bitmask;
 
   /* NDPI_PROTOCOL_MAIL_POP */
   u_int16_t pop_command_bitmask;
 
-  /* NDPI_PROTOCOL_QQ */
-  u_int16_t qq_nxt_len;
-
   /* NDPI_PROTOCOL_WHATSAPP */
   u_int8_t wa_matched_so_far;
 
-  /* NDPI_PROTOCOL_TDS */
-  u_int8_t tds_login_version;
-
   /* NDPI_PROTOCOL_IRC */
   u_int8_t irc_stage;
-  u_int8_t irc_port;
 
   /* NDPI_PROTOCOL_H323 */
   u_int8_t h323_valid_packets;
@@ -677,38 +568,16 @@ struct ndpi_flow_tcp_struct {
   u_int32_t irc_direction:2;
   u_int32_t irc_0x1000_full:1;
 
-  /* NDPI_PROTOCOL_SOULSEEK */
-  u_int32_t soulseek_stage:2;
-
-  /* NDPI_PROTOCOL_TDS */
-  u_int32_t tds_stage:3;
-
   /* NDPI_PROTOCOL_USENET */
   u_int32_t usenet_stage:2;
 
-  /* NDPI_PROTOCOL_IMESH */
-  u_int32_t imesh_stage:4;
-
   /* NDPI_PROTOCOL_HTTP */
-  u_int32_t http_setup_dir:2;
   u_int32_t http_stage:2;
   u_int32_t http_empty_line_seen:1;
-  u_int32_t http_wait_for_retransmission:1;
 
   /* NDPI_PROTOCOL_GNUTELLA */
   u_int32_t gnutella_stage:2;		       // 0 - 2
 
-  /* NDPI_CONTENT_MMS */
-  u_int32_t mms_stage:2;
-
-  /* NDPI_PROTOCOL_YAHOO */
-  u_int32_t yahoo_sip_comm:1;
-  u_int32_t yahoo_http_proxy_stage:2;
-
-  /* NDPI_PROTOCOL_MSN */
-  u_int32_t msn_stage:3;
-  u_int32_t msn_ssl_ft:2;
-
   /* NDPI_PROTOCOL_SSH */
   u_int32_t ssh_stage:3;
 
@@ -721,11 +590,8 @@ struct ndpi_flow_tcp_struct {
   struct {
     message_t message;
 
-    void* srv_cert_fingerprint_ctx; /* SHA-1 */
-
     /* NDPI_PROTOCOL_TLS */
-    u_int8_t hello_processed:1, certificate_processed:1, subprotocol_detected:1,
-	fingerprint_set:1, _pad:4;
+    u_int8_t certificate_processed:1, fingerprint_set:1, _pad:6;
     u_int8_t num_tls_blocks;
     int16_t tls_application_blocks_len[NDPI_MAX_NUM_TLS_APPL_BLOCKS]; /* + = src->dst, - = dst->src */
   } tls;
@@ -733,8 +599,7 @@ struct ndpi_flow_tcp_struct {
   /* NDPI_PROTOCOL_POSTGRES */
   u_int32_t postgres_stage:3;
 
-  /* NDPI_PROTOCOL_DIRECT_DOWNLOAD_LINK */
-  u_int32_t ddlink_server_direction:1;
+  /* Part of the TCP header. */
   u_int32_t seen_syn:1;
   u_int32_t seen_syn_ack:1;
   u_int32_t seen_ack:1;
@@ -751,9 +616,6 @@ struct ndpi_flow_tcp_struct {
   /* NDPI_PROTOCOL_WORLDOFWARCRAFT */
   u_int32_t wow_stage:2;
 
-  /* NDPI_PROTOCOL_HTTP_APPLICATION_VEOHTV */
-  u_int32_t veoh_tv_stage:2;
-
   /* NDPI_PROTOCOL_SHOUTCAST */
   u_int32_t shoutcast_stage:2;
 
@@ -798,12 +660,6 @@ struct ndpi_flow_tcp_struct {
 /* ************************************************** */
 
 struct ndpi_flow_udp_struct {
-  /* NDPI_PROTOCOL_SNMP */
-  u_int32_t snmp_msg_id;
-
-  /* NDPI_PROTOCOL_SNMP */
-  u_int32_t snmp_stage:2;
-
   /* NDPI_PROTOCOL_PPSTREAM */
   u_int32_t ppstream_stage:3;		  // 0 - 7
 
@@ -819,9 +675,6 @@ struct ndpi_flow_udp_struct {
   /* NDPI_PROTOCOL_XBOX */
   u_int32_t xbox_stage:1;
 
-  /* NDPI_PROTOCOL_WINDOWS_UPDATE */
-  u_int32_t wsus_stage:1;
-
   /* NDPI_PROTOCOL_SKYPE */
   u_int8_t skype_packet_id;
   u_int8_t skype_crc[4];
@@ -847,6 +700,16 @@ struct ndpi_flow_udp_struct {
   /* NDPI_PROTOCOL_QUIC */
   u_int8_t *quic_reasm_buf;
   u_int32_t quic_reasm_buf_len;
+
+  /* NDPI_PROTOCOL_CSGO */
+  u_int8_t csgo_strid[18],csgo_state,csgo_s2;
+  u_int32_t csgo_id2;
+
+  /* NDPI_PROTOCOL_RDP */
+  u_int8_t rdp_to_srv[3], rdp_from_srv[3], rdp_to_srv_pkts, rdp_from_srv_pkts;   
+
+  /* NDPI_PROTOCOL_IMO */
+  u_int8_t imo_last_one_byte_pkt, imo_last_byte;
 };
 
 /* ************************************************** */
@@ -884,9 +747,6 @@ struct ndpi_packet_struct {
 
   u_int64_t current_time_ms;
 
-  u_int16_t detected_protocol_stack[NDPI_PROTOCOL_SIZE];
-  u_int16_t protocol_stack_info;
-
   struct ndpi_int_one_line_struct line[NDPI_MAX_PARSE_LINES_PER_PACKET];
   /* HTTP headers */
   struct ndpi_int_one_line_struct host_line;
@@ -895,6 +755,7 @@ struct ndpi_packet_struct {
   struct ndpi_int_one_line_struct content_line;
   struct ndpi_int_one_line_struct content_disposition_line;
   struct ndpi_int_one_line_struct accept_line;
+  struct ndpi_int_one_line_struct authorization_line;
   struct ndpi_int_one_line_struct user_agent_line;
   struct ndpi_int_one_line_struct http_url_name;
   struct ndpi_int_one_line_struct http_encoding;
@@ -910,17 +771,11 @@ struct ndpi_packet_struct {
   u_int8_t http_num_headers; /* number of found (valid) header lines in HTTP request or response */
 
   u_int16_t l3_packet_len;
-  u_int16_t l4_packet_len;
   u_int16_t payload_packet_len;
-  u_int16_t actual_payload_len;
-  u_int16_t num_retried_bytes;
   u_int16_t parsed_lines;
-  u_int16_t parsed_unix_lines;
   u_int16_t empty_line_position;
   u_int8_t tcp_retransmission;
-  u_int8_t l4_protocol;
 
-  u_int8_t tls_certificate_detected:4, tls_certificate_num_checks:4;
   u_int8_t packet_lines_parsed_complete:1,
     packet_direction:1, empty_line_position_set:1, http_check_content:1, pad:4;
 };
@@ -945,6 +800,27 @@ typedef struct {
   u_int16_t port_low, port_high;
 } ndpi_port_range;
 
+typedef enum {
+  NDPI_CONFIDENCE_UNKNOWN = 0,		/* Unknown classification */
+  NDPI_CONFIDENCE_MATCH_BY_PORT,	/* Classification obtained looking only at the L4 ports */
+  NDPI_CONFIDENCE_MATCH_BY_IP,		/* Classification obtained looking only at the L3 addresses */
+  NDPI_CONFIDENCE_DPI_CACHE,		/* Classification results based on same LRU cache (i.e. correlation among sessions) */
+  NDPI_CONFIDENCE_DPI,			/* Deep packet inspection */
+
+  /*
+    IMPORTANT
+
+    Please keep in sync with
+
+    ndpi_confidence_get_name()
+
+    in ndpi_main.c
+  */
+
+  /* Last one */
+  NDPI_CONFIDENCE_MAX,
+} ndpi_confidence_t;
+
 typedef enum {
   NDPI_PROTOCOL_SAFE = 0,              /* Surely doesn't provide risks for the network. (e.g., a news site) */
   NDPI_PROTOCOL_ACCEPTABLE,            /* Probably doesn't provide risks, but could be malicious (e.g., Dropbox) */
@@ -1046,7 +922,8 @@ typedef enum {
 typedef struct ndpi_proto_defaults {
   char *protoName;
   ndpi_protocol_category_t protoCategory;
-  u_int16_t * subprotocols;
+  u_int8_t isClearTextProto;
+  u_int16_t *subprotocols;
   u_int32_t subprotocol_count;
   u_int16_t protoId, protoIdx;
   u_int16_t tcp_default_ports[MAX_DEFAULT_PORTS], udp_default_ports[MAX_DEFAULT_PORTS];
@@ -1087,20 +964,16 @@ typedef struct ndpi_proto {
 #define _NDPI_CONFIG_H_
 #endif
 
-#ifdef HAVE_PCRE
-#include <pcre.h>
-
-struct pcre_struct {
-  pcre *compiled;
-  pcre_extra *optimized;
-};
-#endif
-
 typedef enum {
   ndpi_stun_cache,
   ndpi_hangout_cache
 } ndpi_lru_cache_type;
 
+typedef struct ndpi_list_struct {
+  char *value;
+  struct ndpi_list_struct *next;
+} ndpi_list;
+
 struct ndpi_detection_module_struct {
   NDPI_PROTOCOL_BITMASK detection_bitmask;
   NDPI_PROTOCOL_BITMASK generic_http_packet_bitmask;
@@ -1109,7 +982,8 @@ struct ndpi_detection_module_struct {
   u_int32_t ticks_per_second;
   u_int16_t num_tls_blocks_to_follow;
   u_int8_t skip_tls_blocks_until_change_cipher:1, enable_ja3_plus:1, _notused:6;
-
+  u_int8_t tls_certificate_expire_in_x_days;
+  
 #ifdef NDPI_ENABLE_DEBUG_MESSAGES
   void *user_data;
 #endif
@@ -1157,13 +1031,13 @@ struct ndpi_detection_module_struct {
   int ac_automa_finalized;
   /* HTTP/DNS/HTTPS/QUIC host matching */
   ndpi_automa host_automa,                     /* Used for DNS/HTTPS */
-    content_automa,                            /* Used for HTTP subprotocol_detection */
-    subprotocol_automa,                        /* Used for HTTP subprotocol_detection */
     risky_domain_automa, tls_cert_subject_automa,
     malicious_ja3_automa, malicious_sha1_automa,
     host_risk_mask_automa, common_alpns_automa;  
   /* IMPORTANT: please update ndpi_finalize_initialization() whenever you add a new automa */
 
+  ndpi_list *trusted_issuer_dn;
+  
   void *ip_risk_mask_ptree;
   
   struct {
@@ -1179,27 +1053,10 @@ struct ndpi_detection_module_struct {
   u_int32_t irc_timeout;
   /* gnutella parameters */
   u_int32_t gnutella_timeout;
-  /* thunder parameters */
-  u_int32_t thunder_timeout;
-  /* SoulSeek parameters */
-  u_int32_t soulseek_connection_ip_tick_timeout;
-  /* rtsp parameters */
-  u_int32_t rtsp_connection_timeout;
   /* rstp */
-  u_int32_t orb_rstp_ts_timeout;
-  /* yahoo */
-  u_int8_t yahoo_detect_http_connections;
-  u_int32_t yahoo_lan_video_timeout;
-  u_int32_t zattoo_connection_timeout;
   u_int32_t jabber_stun_timeout;
   u_int32_t jabber_file_transfer_timeout;
   u_int8_t ip_version_limit;
-  /* NDPI_PROTOCOL_BITTORRENT */
-  struct hash_ip4p_table *bt_ht, *bt6_ht;
-
-  /* BT_ANNOUNCE */
-  struct bt_announce *bt_ann;
-  int    bt_ann_len;
 
   /* NDPI_PROTOCOL_OOKLA */
   struct ndpi_lru_cache *ookla_cache;
@@ -1207,6 +1064,12 @@ struct ndpi_detection_module_struct {
   /* NDPI_PROTOCOL_TINC */
   struct cache *tinc_cache;
 
+  /* NDPI_PROTOCOL_BITTORRENT */
+  struct ndpi_lru_cache *bittorrent_cache;
+
+  /* NDPI_PROTOCOL_ZOOM */
+  struct ndpi_lru_cache *zoom_cache;
+
   /* NDPI_PROTOCOL_STUN and subprotocols */
   struct ndpi_lru_cache *stun_cache;
 
@@ -1229,10 +1092,12 @@ struct ndpi_detection_module_struct {
   #include "../../../nDPI-custom/custom_ndpi_typedefs.h"
 #endif
 
-#ifdef HAVE_MAXMINDDB
-  MMDB_s mmdb_city, mmdb_as;
+  /* GeoIP */
+  void *mmdb_city, *mmdb_as;
   u_int8_t mmdb_city_loaded, mmdb_as_loaded;
-#endif
+
+  /* Current packet */
+  struct ndpi_packet_struct packet;
 };
 
 #endif /* NDPI_LIB_COMPILATION */
@@ -1260,12 +1125,12 @@ struct tls_heuristics {
  */
 struct ndpi_flow_struct {
   u_int16_t detected_protocol_stack[NDPI_PROTOCOL_SIZE];
-  u_int16_t protocol_stack_info;
 
   /* init parameter, internal used to set up timestamp,... */
   u_int16_t guessed_protocol_id, guessed_host_protocol_id, guessed_category, guessed_header_category;
   u_int8_t l4_proto, protocol_id_already_guessed:1, host_already_guessed:1, fail_with_unknown:1,
-    init_finished:1, setup_packet_direction:1, packet_direction:1, check_extra_packets:1;
+    init_finished:1, setup_packet_direction:1, packet_direction:1, check_extra_packets:1, is_ipv6:1;
+  ndpi_confidence_t confidence; /* ndpi_confidence_t */
 
   /*
     if ndpi_struct->direction_detect_disable == 1
@@ -1273,6 +1138,15 @@ struct ndpi_flow_struct {
   */
   u_int32_t next_tcp_seq_nr[2];
 
+  /* Flow addresses (used mainly for LRU lookups in ndpi_detection_giveup())
+     and ports. All in *network* byte order
+
+     TODO
+     - IPv6. Note that LRU is ipv4 only, for the time being 
+   */
+  u_int32_t saddr, daddr;
+  u_int16_t sport, dport;
+  
   // -----------------------------------------
 
   u_int8_t max_extra_packets_to_check;
@@ -1281,6 +1155,8 @@ struct ndpi_flow_struct {
 
   int (*extra_packets_func) (struct ndpi_detection_module_struct *, struct ndpi_flow_struct *flow);
 
+  u_int64_t last_packet_time_ms;
+
   /*
     the tcp / udp / other l4 value union
     used to reduce the number of bytes for tcp or udp protocol states
@@ -1290,16 +1166,20 @@ struct ndpi_flow_struct {
     struct ndpi_flow_udp_struct udp;
   } l4;
 
+  /* Some protocols calculate the entropy. */
+  float entropy;
+
   /* Place textual flow info here */
   char flow_extra_info[16];
 
-  /*
-    Pointer to src or dst that identifies the
-    server of this connection
-  */
-  struct ndpi_id_struct *server_id;
-  /* HTTP host or DNS query */
-  u_char host_server_name[240];
+  /* General purpose field used to save mainly hostname/SNI information.
+   * In details it used for: DNS and NETBIOS name, HTTP and DHCP hostname,
+   * WHOIS request, TLS/QUIC server name and STUN realm.
+   *
+   * Please, think *very* hard before increasing its size!
+   */
+  char host_server_name[80];
+
   u_int8_t initial_binary_bytes[8], initial_binary_bytes_len;
   u_int8_t risk_checked:1, ip_risk_mask_evaluated:1, host_risk_mask_evaluated:1, _notused:5;
   ndpi_risk risk_mask; /* Stores the flow risk mask for flow peers */
@@ -1314,11 +1194,11 @@ struct ndpi_flow_struct {
   */
   struct {
     ndpi_http_method method;
-    char *url, *content_type /* response */, *request_content_type /* e.g. for POST */, *user_agent;
-    u_int8_t num_request_headers, num_response_headers;
     u_int8_t request_version; /* 0=1.0 and 1=1.1. Create an enum for this? */
     u_int16_t response_status_code; /* 200, 404, etc. */
-    u_char detected_os[32]; /* Via HTTP/QUIC User-Agent */
+    char *url, *content_type /* response */, *request_content_type /* e.g. for POST */, *user_agent;
+    char *detected_os; /* Via HTTP/QUIC User-Agent */
+    char *nat_ip; /* Via HTTP X-Forwarded-For */
   } http;
 
   /*
@@ -1331,6 +1211,11 @@ struct ndpi_flow_struct {
     u_int16_t pktbuf_maxlen, pktbuf_currlen;
   } kerberos_buf;
 
+  struct {
+    u_int8_t num_udp_pkts, num_binding_requests;
+    u_int16_t num_processed_pkts;
+  } stun;
+
   union {
     /* the only fields useful for nDPI and ntopng */
     struct {
@@ -1349,48 +1234,35 @@ struct ndpi_flow_struct {
     } kerberos;
 
     struct {
-      struct {
-        char ssl_version_str[12];
-	u_int16_t ssl_version, server_names_len;
-	char client_requested_server_name[256], /* SNI hostname length: RFC 4366 */
-	  *server_names, *alpn, *tls_supported_versions, *issuerDN, *subjectDN;
-	u_int32_t notBefore, notAfter;
-	char ja3_client[33], ja3_server[33];
-	u_int16_t server_cipher;
-	u_int8_t sha1_certificate_fingerprint[20];
+      char *server_names, *alpn, *tls_supported_versions, *issuerDN, *subjectDN;
+      u_int32_t notBefore, notAfter;
+      char ja3_client[33], ja3_server[33];
+      u_int16_t server_cipher;
+      u_int8_t sha1_certificate_fingerprint[20];
+      u_int8_t hello_processed:1, subprotocol_detected:1, _pad:6;
 
 #ifdef TLS_HANDLE_SIGNATURE_ALGORITMS
-	/* Under #ifdef to save memory for those who do not need them */
-	u_int8_t num_tls_signature_algorithms;
-	u_int16_t client_signature_algorithms[MAX_NUM_TLS_SIGNATURE_ALGORITHMS];
+      /* Under #ifdef to save memory for those who do not need them */
+      u_int8_t num_tls_signature_algorithms;
+      u_int16_t client_signature_algorithms[MAX_NUM_TLS_SIGNATURE_ALGORITHMS];
 #endif
 
-	struct tls_heuristics browser_heuristics;
+      struct tls_heuristics browser_heuristics;
 
-	struct {
-	  u_int16_t cipher_suite;
-	  char *esni;
-	} encrypted_sni;
-	ndpi_cipher_weakness server_unsafe_cipher;
-      } tls_quic;
+      u_int16_t ssl_version, server_names_len;
 
       struct {
-		    u_int8_t num_udp_pkts, num_binding_requests;
-        u_int16_t num_processed_pkts;
-      } stun;
-
-      /* We can have STUN over SSL/TLS thus they need to live together */
-    } tls_quic_stun;
+        u_int16_t cipher_suite;
+        char *esni;
+      } encrypted_sni;
+      ndpi_cipher_weakness server_unsafe_cipher;
+    } tls_quic;
 
     struct {
       char client_signature[48], server_signature[48];
       char hassh_client[33], hassh_server[33];
     } ssh;
 
-    struct {
-      u_int8_t last_one_byte_pkt, last_byte;
-    } imo;
-
     struct {
       u_int8_t username_detected:1, username_found:1,
 	password_detected:1, password_found:1,
@@ -1403,16 +1275,9 @@ struct ndpi_flow_struct {
       char version[32];
     } ubntac2;
 
-    struct {
-      /* Via HTTP X-Forwarded-For */
-      u_char nat_ip[24];
-    } http;
-
-    struct {
-      u_int8_t auth_found:1, auth_failed:1, _pad:5;
-      char username[16], password[16];
-    } ftp_imap_pop_smtp;
-
+    /* In TLS.Bittorent flows there is no hash.
+       Nonetheless, we must pay attention to NOT write to /read from this field
+       with these flows */
     struct {
       /* Bittorrent hash */
       u_char hash[20];
@@ -1439,6 +1304,7 @@ struct ndpi_flow_struct {
   u_int16_t byte_counter[2];
   /* NDPI_PROTOCOL_BITTORRENT */
   u_int8_t bittorrent_stage;		      // can be 0 - 255
+  u_int8_t bt_check_performed : 1;
 
   /* NDPI_PROTOCOL_DIRECTCONNECT */
   u_int8_t directconnect_stage:2;	      // 0 - 1
@@ -1447,17 +1313,11 @@ struct ndpi_flow_struct {
   u_int8_t http_detected:1;
 
   /* NDPI_PROTOCOL_RTSP */
-  u_int8_t rtsprdt_stage:2, rtsp_control_flow:1;
-
-  /* NDPI_PROTOCOL_YAHOO */
-  u_int8_t yahoo_detection_finished:2;
+  u_int8_t rtsprdt_stage:2;
 
   /* NDPI_PROTOCOL_ZATTOO */
   u_int8_t zattoo_stage:3;
 
-  /* NDPI_PROTOCOL_QQ */
-  u_int8_t qq_stage:3;
-
   /* NDPI_PROTOCOL_THUNDER */
   u_int8_t thunder_stage:2;		        // 0 - 3
 
@@ -1492,24 +1352,16 @@ struct ndpi_flow_struct {
   /* NDPI_PROTOCOL_TINC */
   u_int8_t tinc_state;
   struct tinc_cache_entry tinc_cache_entry;
-
-  /* NDPI_PROTOCOL_CSGO */
-  u_int8_t csgo_strid[18],csgo_state,csgo_s2;
-  u_int32_t csgo_id2;
-
-  /* internal structures to save functions calls */
-  struct ndpi_packet_struct packet;
-  struct ndpi_flow_struct *flow;
-  struct ndpi_id_struct *src;
-  struct ndpi_id_struct *dst;
 };
 
+#define NDPI_PROTOCOL_DEFAULT_LEVEL	0
+
 typedef struct {
   char *string_to_match, *proto_name;
   u_int16_t protocol_id;
   ndpi_protocol_category_t protocol_category;
   ndpi_protocol_breed_t protocol_breed;
-  int level; /* 0 by default */
+  int level; /* NDPI_PROTOCOL_DEFAULT_LEVEL (0) by default */
 } ndpi_protocol_match;
 
 typedef struct {
@@ -1525,7 +1377,7 @@ typedef struct {
 typedef struct {
   u_int32_t network;
   u_int8_t cidr;
-  u_int8_t value;
+  u_int16_t value;
 } ndpi_network;
 
 typedef u_int32_t ndpi_init_prefs;
@@ -1687,7 +1539,8 @@ enum ndpi_bin_family {
 };
 
 struct ndpi_bin {
-  u_int8_t num_bins, is_empty;
+  u_int8_t is_empty;
+  u_int16_t num_bins;
   enum ndpi_bin_family family;
 
   union {
@@ -1772,4 +1625,9 @@ typedef int (*ndpi_custom_dga_predict_fctn)(const char* domain, int domain_lengt
 
 /* **************************************** */
 
+typedef void ndpi_bitmap;
+typedef void ndpi_bitmap_iterator;
+
+/* **************************************** */
+
 #endif /* __NDPI_TYPEDEFS_H__ */
