@@ -1,7 +1,7 @@
 /*
  * ndpi_analyze.c
  *
- * Copyright (C) 2019-21 - ntop.org
+ * Copyright (C) 2019-22 - ntop.org
  *
  * This file is part of nDPI, an open source deep packet inspection
  * library.
@@ -299,7 +299,7 @@ double ndpi_hll_count(struct ndpi_hll *hll) {
 /* ********************************************************************************* */
 /* ********************************************************************************* */
 
-int ndpi_init_bin(struct ndpi_bin *b, enum ndpi_bin_family f, u_int8_t num_bins) {
+int ndpi_init_bin(struct ndpi_bin *b, enum ndpi_bin_family f, u_int16_t num_bins) {
   b->num_bins = num_bins, b->family = f, b->is_empty = 1;
 
   switch(f) {
@@ -327,13 +327,13 @@ int ndpi_init_bin(struct ndpi_bin *b, enum ndpi_bin_family f, u_int8_t num_bins)
 void ndpi_free_bin(struct ndpi_bin *b) {
   switch(b->family) {
   case ndpi_bin_family8:
-    free(b->u.bins8);
+    ndpi_free(b->u.bins8);
     break;
   case ndpi_bin_family16:
-    free(b->u.bins16);
+    ndpi_free(b->u.bins16);
     break;
   case ndpi_bin_family32:
-    free(b->u.bins32);
+    ndpi_free(b->u.bins32);
     break;
   }
 }
@@ -350,7 +350,7 @@ struct ndpi_bin* ndpi_clone_bin(struct ndpi_bin *b) {
   switch(out->family) {
   case ndpi_bin_family8:
     if((out->u.bins8 = (u_int8_t*)ndpi_calloc(out->num_bins, sizeof(u_int8_t))) == NULL) {
-      free(out);
+      ndpi_free(out);
       return(NULL);
     } else
       memcpy(out->u.bins8, b->u.bins8, out->num_bins*sizeof(u_int8_t));
@@ -358,7 +358,7 @@ struct ndpi_bin* ndpi_clone_bin(struct ndpi_bin *b) {
 
   case ndpi_bin_family16:
     if((out->u.bins16 = (u_int16_t*)ndpi_calloc(out->num_bins, sizeof(u_int16_t))) == NULL) {
-      free(out);
+      ndpi_free(out);
       return(NULL);
     } else
       memcpy(out->u.bins16, b->u.bins16, out->num_bins*sizeof(u_int16_t));
@@ -366,7 +366,7 @@ struct ndpi_bin* ndpi_clone_bin(struct ndpi_bin *b) {
 
   case ndpi_bin_family32:
     if((out->u.bins32 = (u_int32_t*)ndpi_calloc(out->num_bins, sizeof(u_int32_t))) == NULL) {
-      free(out);
+      ndpi_free(out);
       return(NULL);
     } else
       memcpy(out->u.bins32, b->u.bins32, out->num_bins*sizeof(u_int32_t));
@@ -378,7 +378,7 @@ struct ndpi_bin* ndpi_clone_bin(struct ndpi_bin *b) {
 
 /* ********************************************************************************* */
 
-void ndpi_set_bin(struct ndpi_bin *b, u_int8_t slot_id, u_int32_t val) {
+void ndpi_set_bin(struct ndpi_bin *b, u_int16_t slot_id, u_int32_t val) {
   if(slot_id >= b->num_bins) slot_id = 0;
 
   switch(b->family) {
@@ -396,7 +396,7 @@ void ndpi_set_bin(struct ndpi_bin *b, u_int8_t slot_id, u_int32_t val) {
 
 /* ********************************************************************************* */
 
-void ndpi_inc_bin(struct ndpi_bin *b, u_int8_t slot_id, u_int32_t val) {
+void ndpi_inc_bin(struct ndpi_bin *b, u_int16_t slot_id, u_int32_t val) {
   b->is_empty = 0;
 
   if(slot_id >= b->num_bins) slot_id = 0;
@@ -416,7 +416,7 @@ void ndpi_inc_bin(struct ndpi_bin *b, u_int8_t slot_id, u_int32_t val) {
 
 /* ********************************************************************************* */
 
-u_int32_t ndpi_get_bin_value(struct ndpi_bin *b, u_int8_t slot_id) {
+u_int32_t ndpi_get_bin_value(struct ndpi_bin *b, u_int16_t slot_id) {
   if(slot_id >= b->num_bins) slot_id = 0;
 
   switch(b->family) {
@@ -457,7 +457,7 @@ void ndpi_reset_bin(struct ndpi_bin *b) {
   Each bin slot is transformed in a % with respect to the value total
  */
 void ndpi_normalize_bin(struct ndpi_bin *b) {
-  u_int8_t i;
+  u_int16_t i;
   u_int32_t tot = 0;
 
   if(b->is_empty) return;
@@ -495,7 +495,7 @@ void ndpi_normalize_bin(struct ndpi_bin *b) {
 /* ********************************************************************************* */
 
 char* ndpi_print_bin(struct ndpi_bin *b, u_int8_t normalize_first, char *out_buf, u_int out_buf_len) {
-  u_int8_t i;
+  u_int16_t i;
   u_int len = 0;
 
   if(!out_buf) return(out_buf); else out_buf[0] = '\0';
@@ -555,10 +555,14 @@ char* ndpi_print_bin(struct ndpi_bin *b, u_int8_t normalize_first, char *out_buf
    0 = alike
    ...
    the higher the more different
-*/
-float ndpi_bin_similarity(struct ndpi_bin *b1, struct ndpi_bin *b2, u_int8_t normalize_first) {
-  u_int8_t i;
 
+   if similarity_max_threshold != 0, we assume that bins arent similar
+*/
+float ndpi_bin_similarity(struct ndpi_bin *b1, struct ndpi_bin *b2,
+			  u_int8_t normalize_first, float similarity_max_threshold) {
+  u_int16_t i;
+  float threshold = similarity_max_threshold*similarity_max_threshold;
+  
   if(
      // (b1->family != b2->family) ||
      (b1->num_bins != b2->num_bins))
@@ -594,7 +598,10 @@ float ndpi_bin_similarity(struct ndpi_bin *b1, struct ndpi_bin *b2, u_int8_t nor
 
       if(a != b) sum += pow(diff, 2);
 
-      // printf("[a: %u][b: %u][sum: %u]\n", a, b, sum);
+      if(threshold && (sum > threshold))
+	return(-2); /* Sorry they are not similar */
+	 
+      // printf("%u/%u) [a: %u][b: %u][sum: %u]\n", i, b1->num_bins, a, b, sum);
     }
 
     /* The lower the more similar */
@@ -720,7 +727,7 @@ int ndpi_cluster_bins(struct ndpi_bin *bins, u_int16_t num_bins,
 
 	if(centroids[j].is_empty) continue;
 
-	similarity = ndpi_bin_similarity(&bins[i], &centroids[j], 0);
+	similarity = ndpi_bin_similarity(&bins[i], &centroids[j], 0, 0);
 
 	if(j == cluster_ids[i])
 	  current_similarity = similarity;
@@ -840,8 +847,8 @@ int ndpi_alloc_rsi(struct ndpi_rsi_struct *s, u_int16_t num_learning_values) {
     s->last_value = 0;
     return(0);
   } else {
-    if(s->gains)  free(s->gains);
-    if(s->losses) free(s->losses);
+    if(s->gains)  ndpi_free(s->gains);
+    if(s->losses) ndpi_free(s->losses);
     return(-1);
   }
 }
@@ -987,7 +994,7 @@ int ndpi_hw_init(struct ndpi_hw_struct *hw,
     return(-1);
 
   if((hw->s = (double*)ndpi_calloc(hw->params.num_season_periods, sizeof(double))) == NULL) {
-    free(hw->y);
+    ndpi_free(hw->y);
     return(-1);
   }
 
@@ -1226,12 +1233,67 @@ int ndpi_ses_add_value(struct ndpi_ses_struct *ses, const u_int64_t _value, doub
 
 #ifdef SES_DEBUG
   printf("[num_values: %u][[error: %.3f][forecast: %.3f][sqe: %.3f][sq: %.3f][confidence_band: %.3f]\n",
-	   ses->num_values, error, *forecast, ses->sum_square_error, sq, *confidence_band);
+	   ses->num_values, error, *forecast, ses->sum_square_error, sq_error, *confidence_band);
 #endif
 
   return(rc);
 }
 
+/* *********************************************************** */
+
+/*
+  Computes the best alpha value using the specified values used for training
+*/
+void ndpi_ses_fitting(double *values, u_int32_t num_values, float *ret_alpha) {
+  u_int i;
+  float alpha, best_alpha;
+  double sse, lowest_sse;
+  int trace = 0;
+
+  lowest_sse = 0, best_alpha = 0;
+
+  for(alpha=0.1; alpha<0.99; alpha += 0.05) {
+    struct ndpi_ses_struct ses;
+      
+    ndpi_ses_init(&ses, alpha, 0.05);
+
+    if(trace)
+      printf("\nDouble Exponential Smoothing [alpha: %.2f]\n", alpha);
+
+    sse = 0;
+
+    for(i=0; i<num_values; i++) {
+      double prediction, confidence_band;
+      double diff;
+
+      if(ndpi_ses_add_value(&ses, values[i], &prediction, &confidence_band) != 0) {
+	diff = fabs(prediction-values[i]);
+
+	if(trace)
+	  printf("%2u)\t%12.3f\t%.3f\t%.3f\n", i, values[i], prediction, diff);
+
+	sse += diff*diff;
+      }
+    }
+
+    if(lowest_sse == 0)
+      lowest_sse = sse, best_alpha = alpha; /* first run */
+    else {
+      if(sse <= lowest_sse)
+	lowest_sse = sse, best_alpha = alpha;
+    }
+
+    if(trace)
+      printf("[alpha: %.2f] - SSE: %.2f [BEST: alpha: %.2f/SSE: %.2f]\n", alpha, sse,
+	     best_alpha, lowest_sse);
+  } /* for (alpha) */
+
+  if(trace)
+    printf("BEST [alpha: %.2f][SSE: %.2f]\n", best_alpha, lowest_sse);
+
+  *ret_alpha = best_alpha;
+}
+
 /* *********************************************************** */
 /* *********************************************************** */
 
@@ -1243,7 +1305,8 @@ int ndpi_des_init(struct ndpi_des_struct *des, double alpha, double beta, float
   memset(des, 0, sizeof(struct ndpi_des_struct));
 
   des->params.alpha = alpha;
-  
+  des->params.beta = beta;
+	
   if((significance < 0) || (significance > 1)) significance = 0.05;
   des->params.ro         = ndpi_normal_cdf_inverse(1 - (significance / 2.));
 
@@ -1305,3 +1368,60 @@ int ndpi_des_add_value(struct ndpi_des_struct *des, const u_int64_t _value, doub
   
   return(rc);
 }
+
+/* *********************************************************** */
+
+/*
+  Computes the best alpha and beta values using the specified values used for training
+*/
+void ndpi_des_fitting(double *values, u_int32_t num_values, float *ret_alpha, float *ret_beta) {
+  u_int i;
+  float alpha, best_alpha, best_beta, beta = 0;
+  double sse, lowest_sse;
+  int trace = 0;
+
+  lowest_sse = 0, best_alpha = 0, best_beta = 0;
+
+  for(beta=0.1; beta<0.99; beta += 0.05) {
+    for(alpha=0.1; alpha<0.99; alpha += 0.05) {
+      struct ndpi_des_struct des;
+      
+      ndpi_des_init(&des, alpha, beta, 0.05);
+
+      if(trace)
+	printf("\nDouble Exponential Smoothing [alpha: %.2f][beta: %.2f]\n", alpha, beta);
+
+      sse = 0;
+
+      for(i=0; i<num_values; i++) {
+	double prediction, confidence_band;
+	double diff;
+
+	if(ndpi_des_add_value(&des, values[i], &prediction, &confidence_band) != 0) {
+	  diff = fabs(prediction-values[i]);
+
+	  if(trace)
+	    printf("%2u)\t%12.3f\t%.3f\t%.3f\n", i, values[i], prediction, diff);
+
+	  sse += diff*diff;
+	}
+      }
+
+      if(lowest_sse == 0)
+	lowest_sse = sse, best_alpha = alpha, best_beta = beta; /* first run */
+      else {
+	if(sse <= lowest_sse)
+	  lowest_sse = sse, best_alpha = alpha, best_beta = beta;
+      }
+
+      if(trace)
+	printf("[alpha: %.2f][beta: %.2f] - SSE: %.2f [BEST: alpha: %.2f/beta: %.2f/SSE: %.2f]\n", alpha, beta, sse,
+	       best_alpha, best_beta, lowest_sse);
+    } /* for (alpha) */
+  } /* for (beta) */
+
+  if(trace)
+    printf("BEST [alpha: %.2f][beta: %.2f][SSE: %.2f]\n", best_alpha, best_beta, lowest_sse);
+
+  *ret_alpha = best_alpha, *ret_beta = best_beta;
+}
