@@ -1,7 +1,7 @@
 /*
  * ndpi_serializer.c
  *
- * Copyright (C) 2011-20 - ntop.org
+ * Copyright (C) 2011-22 - ntop.org
  *
  * nDPI is free software: you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -47,14 +47,17 @@
 #define NDPI_U64_FORMAT "%llu"
 #endif
 
+#define OPTIMIZE_NUMERIC_KEYS      /* Convert numeric string keys into int keys */
+#define OPTIMIZE_CSV_SERIALIZATION /* Optimize serialization speed in case of CSV */
+
 /* ********************************** */
 
 u_int64_t ndpi_htonll(u_int64_t v) {
   union { u_int32_t lv[2]; u_int64_t llv; } u;
-  
+
   u.lv[0] = htonl(v >> 32);
   u.lv[1] = htonl(v & 0xFFFFFFFFULL);
-  
+
   return(u.llv);
 }
 
@@ -62,22 +65,24 @@ u_int64_t ndpi_htonll(u_int64_t v) {
 
 u_int64_t ndpi_ntohll(u_int64_t v) {
   union { u_int32_t lv[2]; u_int64_t llv; } u;
-  
+
   u.llv = v;
-  
+
   return((u_int64_t)ntohl(u.lv[0]) << 32) | (u_int64_t)ntohl(u.lv[1]);
 }
 
 /* ********************************** */
 
+#ifdef OPTIMIZE_NUMERIC_KEYS
 static int ndpi_is_number(const char *str, u_int32_t str_len) {
-  int i;
-  
+  unsigned int i;
+
   for(i = 0; i < str_len; i++)
     if(!isdigit(str[i])) return(0);
-  
+
   return(1);
 }
+#endif
 
 /* ********************************** */
 
@@ -159,7 +164,7 @@ static inline int ndpi_snappend(char *buf, size_t size, const char *fmt, ...) {
   else if (wlen >= size)
     wlen = size-1;
 
-  return wlen;
+  return(wlen);
 }
 #endif
 
@@ -188,12 +193,21 @@ void ndpi_reset_serializer(ndpi_serializer *_serializer) {
 
 /* ********************************** */
 
+void ndpi_serializer_skip_header(ndpi_serializer *_serializer) {
+  ndpi_private_serializer *serializer = (ndpi_private_serializer*)_serializer;
+
+  serializer->status.flags |= NDPI_SERIALIZER_STATUS_HDR_DONE;
+}
+
+/* ********************************** */
+
 static int ndpi_init_serializer_buffer(ndpi_private_serializer_buffer *buffer, u_int32_t buffer_size) {
   buffer->initial_size = buffer->size = buffer_size;
   buffer->data = (u_int8_t *) calloc(buffer->size, sizeof(u_int8_t));
   if(buffer->data == NULL)
-    return -1;
-  return 0;
+    return(-1);
+
+  return(0);
 }
 
 /* ********************************** */
@@ -209,7 +223,7 @@ int ndpi_init_serializer_ll(ndpi_serializer *_serializer,
 
   if (ndpi_init_serializer_buffer(&serializer->buffer, buffer_size) != 0)
     return(-1);
-  
+
   if(serializer->fmt == ndpi_serialization_format_json) {
     /* nothing to do */
 
@@ -227,7 +241,7 @@ int ndpi_init_serializer_ll(ndpi_serializer *_serializer,
 
   ndpi_reset_serializer(_serializer);
 
-  return(1);
+  return(0);
 }
 
 /* ********************************** */
@@ -272,12 +286,12 @@ static inline int ndpi_serializer_check_header_room(ndpi_private_serializer *ser
   u_int32_t buff_diff = serializer->header.size - serializer->status.header.size_used;
 
   if (buff_diff < needed)
-    if (ndpi_extend_serializer_buffer(&serializer->header, needed - buff_diff) < 0) 
-      return -1;
+    if (ndpi_extend_serializer_buffer(&serializer->header, needed - buff_diff) < 0)
+      return(-1);
 
   buff_diff = serializer->header.size - serializer->status.header.size_used;
 
-  return buff_diff;
+  return(buff_diff);
 }
 
 /* ********************************** */
@@ -286,17 +300,17 @@ static inline int ndpi_serializer_header_uint32(ndpi_private_serializer *seriali
   int room;
 
   if (serializer->status.flags & NDPI_SERIALIZER_STATUS_HDR_DONE)
-    return 0;
+    return(0);
 
   room = ndpi_serializer_check_header_room(serializer, 12);
 
   if (room < 0)
-    return -1;
+    return(-1);
 
-  serializer->status.header.size_used += snprintf((char *) &serializer->header.data[serializer->status.header.size_used], 
+  serializer->status.header.size_used += snprintf((char *) &serializer->header.data[serializer->status.header.size_used],
     room, "%s%u", (serializer->status.header.size_used > 0) ? serializer->csv_separator : "", key);
 
-  return 0;
+  return(0);
 }
 
 /* ********************************** */
@@ -305,12 +319,12 @@ static inline int ndpi_serializer_header_string(ndpi_private_serializer *seriali
   int room;
 
   if (serializer->status.flags & NDPI_SERIALIZER_STATUS_HDR_DONE)
-    return 0;
+    return(0);
 
   room = ndpi_serializer_check_header_room(serializer, klen + 4);
 
   if (room < 0)
-    return -1;
+    return(-1);
 
   if (serializer->status.header.size_used > 0) {
     int slen = strlen(serializer->csv_separator);
@@ -325,7 +339,7 @@ static inline int ndpi_serializer_header_string(ndpi_private_serializer *seriali
 
   serializer->header.data[serializer->status.header.size_used] = '\0';
 
-  return 0;
+  return(0);
 }
 
 /* ********************************** */
@@ -385,7 +399,7 @@ char* ndpi_serializer_get_header(ndpi_serializer *_serializer, u_int32_t *buffer
 
   if(buf == NULL) {
     *buffer_len = 0;
-    return "";
+    return("");
   }
 
   /* NULL terminate the buffer */
@@ -401,7 +415,8 @@ char* ndpi_serializer_get_header(ndpi_serializer *_serializer, u_int32_t *buffer
 
 ndpi_serialization_format ndpi_serializer_get_format(ndpi_serializer *_serializer) {
   ndpi_private_serializer *serializer = (ndpi_private_serializer*)_serializer;
-  return serializer->fmt;
+
+  return(serializer->fmt);
 }
 
 /* ********************************** */
@@ -579,7 +594,7 @@ int ndpi_serialize_raw_record(ndpi_serializer *_serializer,
   u_int32_t buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
   u_int16_t needed = record_len;
   u_int8_t add_comma = 0;
-    
+
   if(serializer->fmt == ndpi_serialization_format_json) {
     needed += 1;
 
@@ -588,21 +603,21 @@ int ndpi_serialize_raw_record(ndpi_serializer *_serializer,
     else
       needed += 2, add_comma = 1;
   }
-  
+
   if(buff_diff < needed) {
     if(ndpi_extend_serializer_buffer(&serializer->buffer, needed - buff_diff) < 0)
       return(-1);
-    
+
     buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
   }
 
-  if(serializer->fmt == ndpi_serialization_format_json) {   
+  if(serializer->fmt == ndpi_serialization_format_json) {
     if(add_comma)
       serializer->buffer.data[serializer->status.buffer.size_used-1] = ',';
     else
       serializer->status.buffer.size_used--;
   }
-  
+
   memcpy(&serializer->buffer.data[serializer->status.buffer.size_used], record, record_len);
   serializer->status.buffer.size_used += record_len;
 
@@ -610,9 +625,9 @@ int ndpi_serialize_raw_record(ndpi_serializer *_serializer,
     serializer->buffer.data[serializer->status.buffer.size_used] = ']';
     if(add_comma) serializer->status.buffer.size_used++;
   }
-  
+
   ndpi_serialize_end_of_record(_serializer);
-  
+
   return(0);
 }
 
@@ -653,7 +668,7 @@ int ndpi_serialize_end_of_record(ndpi_serializer *_serializer) {
   }
 
   serializer->status.flags &= ~NDPI_SERIALIZER_STATUS_NOT_EMPTY;
-  
+
   return(0);
 }
 
@@ -665,10 +680,10 @@ static inline void ndpi_serialize_csv_pre(ndpi_private_serializer *serializer) {
   } else if (serializer->status.buffer.size_used == 0) {
     /* nothing to do */
   } else {
-    u_int32_t buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
-    serializer->status.buffer.size_used += snprintf((char *) 
-      &serializer->buffer.data[serializer->status.buffer.size_used], buff_diff, "%s",
-      serializer->csv_separator);
+    if(serializer->buffer.size > serializer->status.buffer.size_used)
+      serializer->buffer.data[serializer->status.buffer.size_used] = serializer->csv_separator[0];
+
+    serializer->status.buffer.size_used++;
   }
 }
 
@@ -692,7 +707,7 @@ static inline void ndpi_serialize_json_pre(ndpi_serializer *_serializer) {
       serializer->status.buffer.size_used--; /* Remove ']' */
       if(serializer->status.flags & NDPI_SERIALIZER_STATUS_SOL)
         serializer->status.flags &= ~NDPI_SERIALIZER_STATUS_SOL;
-      else 
+      else
         serializer->buffer.data[serializer->status.buffer.size_used++] = ',';
     } else {
       if(serializer->status.flags & NDPI_SERIALIZER_STATUS_SOB)
@@ -763,13 +778,13 @@ int ndpi_serialize_uint32_uint32(ndpi_serializer *_serializer,
 
     if (!(serializer->status.flags & NDPI_SERIALIZER_STATUS_LIST)) {
       serializer->status.buffer.size_used += snprintf((char *)
-        &serializer->buffer.data[serializer->status.buffer.size_used], 
+        &serializer->buffer.data[serializer->status.buffer.size_used],
         buff_diff, "\"%u\":", key);
       buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
     }
 
     serializer->status.buffer.size_used += snprintf((char *)
-      &serializer->buffer.data[serializer->status.buffer.size_used], 
+      &serializer->buffer.data[serializer->status.buffer.size_used],
       buff_diff, "%u", value);
 
     ndpi_serialize_json_post(_serializer);
@@ -830,13 +845,13 @@ int ndpi_serialize_uint32_uint64(ndpi_serializer *_serializer,
     ndpi_serialize_json_pre(_serializer);
 
     if (!(serializer->status.flags & NDPI_SERIALIZER_STATUS_LIST)) {
-      serializer->status.buffer.size_used += snprintf((char *) 
+      serializer->status.buffer.size_used += snprintf((char *)
         &serializer->buffer.data[serializer->status.buffer.size_used], buff_diff,
         "\"%u\":", key);
       buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
     }
 
-    serializer->status.buffer.size_used += snprintf((char *) 
+    serializer->status.buffer.size_used += snprintf((char *)
       &serializer->buffer.data[serializer->status.buffer.size_used], buff_diff,
       NDPI_U64_FORMAT, (unsigned long long)value);
 
@@ -892,16 +907,16 @@ int ndpi_serialize_uint32_int32(ndpi_serializer *_serializer,
 
   if(serializer->fmt == ndpi_serialization_format_json) {
     ndpi_serialize_json_pre(_serializer);
-    
+
     if (!(serializer->status.flags & NDPI_SERIALIZER_STATUS_LIST)) {
       serializer->status.buffer.size_used += snprintf((char *)
-        &serializer->buffer.data[serializer->status.buffer.size_used], 
+        &serializer->buffer.data[serializer->status.buffer.size_used],
         buff_diff, "\"%u\":", key);
       buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
     }
 
     serializer->status.buffer.size_used += snprintf((char *)
-      &serializer->buffer.data[serializer->status.buffer.size_used], 
+      &serializer->buffer.data[serializer->status.buffer.size_used],
       buff_diff, "%d", value);
 
     ndpi_serialize_json_post(_serializer);
@@ -909,7 +924,7 @@ int ndpi_serialize_uint32_int32(ndpi_serializer *_serializer,
     if (ndpi_serializer_header_uint32(serializer, key) < 0) return(-1);
     ndpi_serialize_csv_pre(serializer);
     buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
-    serializer->status.buffer.size_used += snprintf((char *) 
+    serializer->status.buffer.size_used += snprintf((char *)
       &serializer->buffer.data[serializer->status.buffer.size_used], buff_diff,
       "%d", value);
   } else {
@@ -962,14 +977,14 @@ int ndpi_serialize_uint32_int64(ndpi_serializer *_serializer,
     ndpi_serialize_json_pre(_serializer);
 
     if (!(serializer->status.flags & NDPI_SERIALIZER_STATUS_LIST)) {
-      serializer->status.buffer.size_used += snprintf((char *) 
-        &serializer->buffer.data[serializer->status.buffer.size_used], 
+      serializer->status.buffer.size_used += snprintf((char *)
+        &serializer->buffer.data[serializer->status.buffer.size_used],
         buff_diff, "\"%u\":", key);
       buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
     }
 
-    serializer->status.buffer.size_used += snprintf((char *) 
-      &serializer->buffer.data[serializer->status.buffer.size_used], 
+    serializer->status.buffer.size_used += snprintf((char *)
+      &serializer->buffer.data[serializer->status.buffer.size_used],
       buff_diff, NDPI_I64_FORMAT, (long long int)value);
 
     ndpi_serialize_json_post(_serializer);
@@ -1084,7 +1099,7 @@ static int ndpi_serialize_uint32_binary(ndpi_serializer *_serializer,
     ndpi_serialize_json_pre(_serializer);
 
     if (!(serializer->status.flags & NDPI_SERIALIZER_STATUS_LIST)) {
-      serializer->status.buffer.size_used += snprintf((char *) &serializer->buffer.data[serializer->status.buffer.size_used], 
+      serializer->status.buffer.size_used += snprintf((char *) &serializer->buffer.data[serializer->status.buffer.size_used],
         buff_diff, "\"%u\":", key);
       buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
     }
@@ -1096,9 +1111,9 @@ static int ndpi_serialize_uint32_binary(ndpi_serializer *_serializer,
     if (ndpi_serializer_header_uint32(serializer, key) < 0) return(-1);
     ndpi_serialize_csv_pre(serializer);
     buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
-    serializer->status.buffer.size_used += snprintf((char *) 
-      &serializer->buffer.data[serializer->status.buffer.size_used], buff_diff,
-      "%s", value);
+    serializer->status.buffer.size_used += snprintf((char *)
+						    &serializer->buffer.data[serializer->status.buffer.size_used], buff_diff,
+						    "%s", value);
   } else {
     ndpi_serialization_type kt;
     u_int8_t type = 0;
@@ -1135,7 +1150,7 @@ int ndpi_serialize_uint32_boolean(ndpi_serializer *_serializer,
 
   if(serializer->fmt != ndpi_serialization_format_json &&
      serializer->fmt != ndpi_serialization_format_csv)
-    return -1;
+    return(-1);
 
   if(buff_diff < needed) {
     if(ndpi_extend_serializer_buffer(&serializer->buffer, needed - buff_diff) < 0)
@@ -1147,14 +1162,14 @@ int ndpi_serialize_uint32_boolean(ndpi_serializer *_serializer,
     ndpi_serialize_json_pre(_serializer);
 
     if (!(serializer->status.flags & NDPI_SERIALIZER_STATUS_LIST)) {
-      serializer->status.buffer.size_used += snprintf((char *) 
-        &serializer->buffer.data[serializer->status.buffer.size_used], 
+      serializer->status.buffer.size_used += snprintf((char *)
+        &serializer->buffer.data[serializer->status.buffer.size_used],
         buff_diff, "\"%u\":", key);
       buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
     }
 
-    serializer->status.buffer.size_used += snprintf((char *) 
-      &serializer->buffer.data[serializer->status.buffer.size_used], 
+    serializer->status.buffer.size_used += snprintf((char *)
+      &serializer->buffer.data[serializer->status.buffer.size_used],
       buff_diff, "%s", value ? "true" : "false");
 
     ndpi_serialize_json_post(_serializer);
@@ -1180,8 +1195,10 @@ int ndpi_serialize_binary_int32(ndpi_serializer *_serializer,
   u_int32_t buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
   u_int32_t needed;
 
+#ifdef OPTIMIZE_NUMERIC_KEYS
   if(ndpi_is_number(key, klen))
     return(ndpi_serialize_uint32_int32(_serializer, atoi(key), value));
+#endif
 
   needed =
     sizeof(u_int8_t) /* type */ +
@@ -1204,13 +1221,12 @@ int ndpi_serialize_binary_int32(ndpi_serializer *_serializer,
     if (!(serializer->status.flags & NDPI_SERIALIZER_STATUS_LIST)) {
       serializer->status.buffer.size_used += ndpi_json_string_escape(key, klen,
         (char *) &serializer->buffer.data[serializer->status.buffer.size_used], buff_diff);
-      buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
-      serializer->status.buffer.size_used += snprintf((char *) 
-        &serializer->buffer.data[serializer->status.buffer.size_used], buff_diff, ":");
+      serializer->buffer.data[serializer->status.buffer.size_used] = ':';
+      serializer->status.buffer.size_used++;
       buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
     }
-    
-    serializer->status.buffer.size_used += snprintf((char *) 
+
+    serializer->status.buffer.size_used += snprintf((char *)
       &serializer->buffer.data[serializer->status.buffer.size_used], buff_diff, "%d", value);
 
     ndpi_serialize_json_post(_serializer);
@@ -1238,7 +1254,7 @@ int ndpi_serialize_binary_int32(ndpi_serializer *_serializer,
   }
 
   serializer->status.flags |= NDPI_SERIALIZER_STATUS_NOT_EMPTY;
-  
+
   return(0);
 }
 
@@ -1246,6 +1262,38 @@ int ndpi_serialize_binary_int32(ndpi_serializer *_serializer,
 
 int ndpi_serialize_string_int32(ndpi_serializer *_serializer,
 				const char *key, int32_t value) {
+#ifdef OPTIMIZE_CSV_SERIALIZATION
+  ndpi_private_serializer *serializer = (ndpi_private_serializer*)_serializer;
+
+  if(serializer->fmt == ndpi_serialization_format_csv) {
+    /* Key is ignored */
+    u_int32_t buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
+    u_int16_t needed = 11 /* 10 (billion) + CVS separator */;
+    int rc;
+
+    if(buff_diff < needed) {
+      if(ndpi_extend_serializer_buffer(&serializer->buffer, needed - buff_diff) < 0)
+	return(-1);
+      else
+	buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
+    }
+
+    if(!(serializer->status.flags & NDPI_SERIALIZER_STATUS_HDR_DONE)) {
+      if(ndpi_serializer_header_string(serializer, key, strlen(key)) < 0)
+	return(-1);
+    }
+
+    ndpi_serialize_csv_pre(serializer);
+    needed--;
+
+    rc = snprintf((char*)&serializer->buffer.data[serializer->status.buffer.size_used],
+    needed, "%u", value);
+
+    if(rc > 0)
+      serializer->status.buffer.size_used += rc;
+    return(0);
+  } else
+#endif
   return(ndpi_serialize_binary_int32(_serializer, key, strlen(key), value));
 }
 
@@ -1258,8 +1306,10 @@ int ndpi_serialize_binary_int64(ndpi_serializer *_serializer,
   u_int32_t buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
   u_int32_t needed;
 
+#ifdef OPTIMIZE_NUMERIC_KEYS
   if(ndpi_is_number(key, klen))
     return(ndpi_serialize_uint32_int64(_serializer, atoi(key), value));
+#endif
 
   needed =
     sizeof(u_int8_t) /* type */ +
@@ -1283,13 +1333,12 @@ int ndpi_serialize_binary_int64(ndpi_serializer *_serializer,
       serializer->status.buffer.size_used += ndpi_json_string_escape(key, klen,
         (char *) &serializer->buffer.data[serializer->status.buffer.size_used], buff_diff);
       buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
-      serializer->status.buffer.size_used += snprintf((char *) 
-        &serializer->buffer.data[serializer->status.buffer.size_used], buff_diff,
-        ":");
+      serializer->buffer.data[serializer->status.buffer.size_used] = ':';
+      serializer->status.buffer.size_used++;
       buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
     }
 
-    serializer->status.buffer.size_used += snprintf((char *) 
+    serializer->status.buffer.size_used += snprintf((char *)
       &serializer->buffer.data[serializer->status.buffer.size_used], buff_diff,
       NDPI_I64_FORMAT, (long long int)value);
 
@@ -1306,7 +1355,7 @@ int ndpi_serialize_binary_int64(ndpi_serializer *_serializer,
     } else {
       serializer->buffer.data[serializer->status.buffer.size_used++] = (ndpi_serialization_string << 4) | ndpi_serialization_int64;
       ndpi_serialize_single_string(serializer, key, klen);
-      ndpi_serialize_single_uint32(serializer, value);
+      ndpi_serialize_single_uint64(serializer, value);
     }
   }
 
@@ -1329,17 +1378,48 @@ int ndpi_serialize_binary_uint32(ndpi_serializer *_serializer,
   u_int32_t buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
   u_int32_t needed;
 
+#ifdef OPTIMIZE_CSV_SERIALIZATION
+  if(serializer->fmt == ndpi_serialization_format_csv) {
+    /* Key is ignored */
+    u_int32_t buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
+    u_int16_t needed;
+    char _value[16];
+
+    snprintf(_value, sizeof(_value), "%u", value);
+    needed = strlen(_value) + 1 /* CVS separator */;
+
+    if(buff_diff < needed) {
+      if(ndpi_extend_serializer_buffer(&serializer->buffer, needed - buff_diff) < 0)
+	return(-1);
+      else
+	buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
+    }
+
+    if(!(serializer->status.flags & NDPI_SERIALIZER_STATUS_HDR_DONE)) {
+      if(ndpi_serializer_header_string(serializer, key, klen) < 0)
+	return(-1);
+    }
+
+    ndpi_serialize_csv_pre(serializer);
+    needed--;
+    memcpy(&serializer->buffer.data[serializer->status.buffer.size_used], _value, needed);
+    serializer->status.buffer.size_used += needed;
+    return(0);
+  }
+#endif
+
+#ifdef OPTIMIZE_NUMERIC_KEYS
   if(ndpi_is_number(key, klen))
     return(ndpi_serialize_uint32_uint32(_serializer, atoi(key), value));
+#endif
 
   needed =
     sizeof(u_int8_t) /* type */ +
     sizeof(u_int16_t) /* key len */ +
     klen /* key */ +
-    sizeof(u_int32_t);
-
-  if(serializer->fmt == ndpi_serialization_format_json)
-    needed += 16 + klen;
+    sizeof(u_int32_t) + /* value (as int) */
+    20 /* value (as string, in case of CSV) */ + 
+    16 /* extra overhead for JSON */;
 
   if(buff_diff < needed) {
     if(ndpi_extend_serializer_buffer(&serializer->buffer, needed - buff_diff) < 0)
@@ -1354,16 +1434,14 @@ int ndpi_serialize_binary_uint32(ndpi_serializer *_serializer,
       serializer->status.buffer.size_used += ndpi_json_string_escape(key, klen,
       (char *) &serializer->buffer.data[serializer->status.buffer.size_used], buff_diff);
       buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
-
-      serializer->status.buffer.size_used += snprintf((char *) 
-        &serializer->buffer.data[serializer->status.buffer.size_used],
-        buff_diff, ":");
+      serializer->buffer.data[serializer->status.buffer.size_used] = ':';
+      serializer->status.buffer.size_used++;
       buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
     }
 
-    serializer->status.buffer.size_used += snprintf((char *) 
-      &serializer->buffer.data[serializer->status.buffer.size_used],
-      buff_diff, "%u", value);
+    serializer->status.buffer.size_used += snprintf((char *)
+						    &serializer->buffer.data[serializer->status.buffer.size_used],
+						    buff_diff, "%u", value);
 
     ndpi_serialize_json_post(_serializer);
   } else if(serializer->fmt == ndpi_serialization_format_csv) {
@@ -1371,8 +1449,8 @@ int ndpi_serialize_binary_uint32(ndpi_serializer *_serializer,
     ndpi_serialize_csv_pre(serializer);
     buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
     serializer->status.buffer.size_used += snprintf((char *)
-      &serializer->buffer.data[serializer->status.buffer.size_used], buff_diff,
-      "%u", value);
+						    &serializer->buffer.data[serializer->status.buffer.size_used], buff_diff,
+						    "%u", value);
   } else {
     if(value <= 0xff) {
       serializer->buffer.data[serializer->status.buffer.size_used++] = (ndpi_serialization_string << 4) | ndpi_serialization_uint8;
@@ -1431,8 +1509,10 @@ int ndpi_serialize_binary_uint64(ndpi_serializer *_serializer,
   u_int32_t buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
   u_int32_t needed;
 
+#ifdef OPTIMIZE_NUMERIC_KEYS
   if(ndpi_is_number(key, klen))
     return(ndpi_serialize_uint32_uint64(_serializer, atoi(key), value));
+#endif
 
   needed =
     sizeof(u_int8_t) /* type */ +
@@ -1456,13 +1536,12 @@ int ndpi_serialize_binary_uint64(ndpi_serializer *_serializer,
       serializer->status.buffer.size_used += ndpi_json_string_escape(key, klen,
         (char *) &serializer->buffer.data[serializer->status.buffer.size_used], buff_diff);
       buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
-      serializer->status.buffer.size_used += snprintf((char *) 
-        &serializer->buffer.data[serializer->status.buffer.size_used], buff_diff,
-        ":");
+      serializer->buffer.data[serializer->status.buffer.size_used] = ':';
+      serializer->status.buffer.size_used++;
       buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
     }
 
-    serializer->status.buffer.size_used += snprintf((char *) 
+    serializer->status.buffer.size_used += snprintf((char *)
       &serializer->buffer.data[serializer->status.buffer.size_used], buff_diff,
       NDPI_U64_FORMAT, (unsigned long long)value);
 
@@ -1506,8 +1585,10 @@ int ndpi_serialize_binary_float(ndpi_serializer *_serializer,
   u_int32_t buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
   u_int32_t needed;
 
+#ifdef OPTIMIZE_NUMERIC_KEYS
   if(ndpi_is_number(key, klen))
     return(ndpi_serialize_uint32_float(_serializer, atoi(key), value, format));
+#endif
 
   needed =
     sizeof(u_int8_t) /* type */ +
@@ -1530,9 +1611,9 @@ int ndpi_serialize_binary_float(ndpi_serializer *_serializer,
     if (!(serializer->status.flags & NDPI_SERIALIZER_STATUS_LIST)) {
       serializer->status.buffer.size_used += ndpi_json_string_escape(key, klen,
         (char *) &serializer->buffer.data[serializer->status.buffer.size_used], buff_diff);
-      buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
       serializer->buffer.data[serializer->status.buffer.size_used] = ':';
       serializer->status.buffer.size_used++;
+      buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
     }
 
     serializer->status.buffer.size_used += snprintf((char *) &serializer->buffer.data[serializer->status.buffer.size_used], buff_diff, format, value);
@@ -1598,26 +1679,31 @@ static int ndpi_serialize_binary_raw(ndpi_serializer *_serializer,
     if (!(serializer->status.flags & NDPI_SERIALIZER_STATUS_LIST)) {
       serializer->status.buffer.size_used += ndpi_json_string_escape(key, klen,
         (char *) &serializer->buffer.data[serializer->status.buffer.size_used], buff_diff);
-      buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
-      serializer->status.buffer.size_used += snprintf((char *) &serializer->buffer.data[serializer->status.buffer.size_used], buff_diff, ":");
+      serializer->buffer.data[serializer->status.buffer.size_used] = ':';
+      serializer->status.buffer.size_used++;
       buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
     }
 
     if (escape)
       serializer->status.buffer.size_used += ndpi_json_string_escape(value, vlen,
         (char *) &serializer->buffer.data[serializer->status.buffer.size_used], buff_diff);
-    else
-      serializer->status.buffer.size_used += snprintf((char *) &serializer->buffer.data[serializer->status.buffer.size_used], buff_diff,
-        value, vlen);
+    else {
+      //serializer->status.buffer.size_used += snprintf((char *) &serializer->buffer.data[serializer->status.buffer.size_used], buff_diff, value, vlen);
+      memcpy(&serializer->buffer.data[serializer->status.buffer.size_used], value, vlen);
+      serializer->status.buffer.size_used += vlen;
+    }
 
     ndpi_serialize_json_post(_serializer);
   } else if(serializer->fmt == ndpi_serialization_format_csv) {
     if (ndpi_serializer_header_string(serializer, key, klen) < 0) return(-1);
     ndpi_serialize_csv_pre(serializer);
     buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
-    serializer->status.buffer.size_used += snprintf((char *)
-      &serializer->buffer.data[serializer->status.buffer.size_used], buff_diff,
-      "%s", value);
+
+    //serializer->status.buffer.size_used += snprintf((char *)
+    //  &serializer->buffer.data[serializer->status.buffer.size_used], buff_diff, "%s", value);
+    memcpy(&serializer->buffer.data[serializer->status.buffer.size_used], value, vlen);
+    serializer->status.buffer.size_used += vlen;
+
   } else {
     serializer->buffer.data[serializer->status.buffer.size_used++] = (ndpi_serialization_string << 4) | ndpi_serialization_string;
 
@@ -1631,7 +1717,7 @@ static int ndpi_serialize_binary_raw(ndpi_serializer *_serializer,
 
 /* ********************************** */
 
-/* Key is a <string, len> pair, value is a <string, len> pair */ 
+/* Key is a <string, len> pair, value is a <string, len> pair */
 int ndpi_serialize_binary_binary(ndpi_serializer *_serializer,
 				 const char *key,
 				 u_int16_t klen,
@@ -1639,10 +1725,12 @@ int ndpi_serialize_binary_binary(ndpi_serializer *_serializer,
 				 u_int16_t vlen) {
   const char *value = _value ? _value : "";
 
+#ifdef OPTIMIZE_NUMERIC_KEYS
   if(ndpi_is_number(key, klen))
     return(ndpi_serialize_uint32_binary(_serializer, atoi(key), value, vlen));
+#endif
 
-  return ndpi_serialize_binary_raw(_serializer, key, klen, value, vlen, 1 /* escape */);
+  return(ndpi_serialize_binary_raw(_serializer, key, klen, value, vlen, 1 /* escape */));
 }
 
 /* ********************************** */
@@ -1660,6 +1748,33 @@ int ndpi_serialize_string_binary(ndpi_serializer *_serializer,
 int ndpi_serialize_string_string(ndpi_serializer *_serializer,
 				 const char *key, const char *_value) {
   const char *value = _value ? _value : "";
+  ndpi_private_serializer *serializer = (ndpi_private_serializer*)_serializer;
+
+#ifdef OPTIMIZE_CSV_SERIALIZATION
+  if(serializer->fmt == ndpi_serialization_format_csv) {
+    /* Key is ignored */
+    u_int32_t buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
+    u_int16_t needed = strlen(_value) + 1 /* CVS separator */;
+
+    if(buff_diff < needed) {
+      if(ndpi_extend_serializer_buffer(&serializer->buffer, needed - buff_diff) < 0)
+	return(-1);
+      else
+	buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
+    }
+
+    if(!(serializer->status.flags & NDPI_SERIALIZER_STATUS_HDR_DONE)) {
+      if(ndpi_serializer_header_string(serializer, key, strlen(key)) < 0)
+	return(-1);
+    }
+
+    ndpi_serialize_csv_pre(serializer);
+    needed--;
+    memcpy(&serializer->buffer.data[serializer->status.buffer.size_used], _value, needed);
+    serializer->status.buffer.size_used += needed;
+    return(0);
+  } else
+#endif
   return(ndpi_serialize_binary_binary(_serializer, key, strlen(key), value, strlen(value)));
 }
 
@@ -1673,19 +1788,20 @@ int ndpi_serialize_string_raw(ndpi_serializer *_serializer,
 
 /* ********************************** */
 
-int ndpi_serialize_string_boolean(ndpi_serializer *_serializer,
-				  const char *key, u_int8_t value) {
+int ndpi_serialize_binary_boolean(ndpi_serializer *_serializer,
+				  const char *key, u_int16_t klen, u_int8_t value) {
   ndpi_private_serializer *serializer = (ndpi_private_serializer*)_serializer;
   u_int32_t buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
-  u_int16_t klen = strlen(key);
   u_int32_t needed;
 
   if(serializer->fmt != ndpi_serialization_format_json &&
      serializer->fmt != ndpi_serialization_format_csv)
-    return -1;
+    return(-1);
 
+#ifdef OPTIMIZE_NUMERIC_KEYS
   if(ndpi_is_number(key, klen))
     return(ndpi_serialize_uint32_boolean(_serializer, atoi(key), value));
+#endif
 
   needed = klen + 16;
 
@@ -1701,9 +1817,8 @@ int ndpi_serialize_string_boolean(ndpi_serializer *_serializer,
     if (!(serializer->status.flags & NDPI_SERIALIZER_STATUS_LIST)) {
       serializer->status.buffer.size_used += ndpi_json_string_escape(key, klen,
         (char *) &serializer->buffer.data[serializer->status.buffer.size_used], buff_diff);
-      buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
-      serializer->status.buffer.size_used += snprintf((char *)
-        &serializer->buffer.data[serializer->status.buffer.size_used], buff_diff, ":");
+      serializer->buffer.data[serializer->status.buffer.size_used] = ':';
+      serializer->status.buffer.size_used++;
       buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
     }
 
@@ -1726,6 +1841,13 @@ int ndpi_serialize_string_boolean(ndpi_serializer *_serializer,
 
 /* ********************************** */
 
+int ndpi_serialize_string_boolean(ndpi_serializer *_serializer,
+				  const char *key, u_int8_t value) {
+  return(ndpi_serialize_binary_boolean(_serializer, key, strlen(key), value));
+}
+
+/* ********************************** */
+
 int ndpi_serialize_start_of_list_binary(ndpi_serializer *_serializer,
 					const char *key, u_int16_t klen) {
   ndpi_private_serializer *serializer = (ndpi_private_serializer*)_serializer;
@@ -1773,7 +1895,7 @@ int ndpi_serialize_start_of_list(ndpi_serializer *_serializer,
   const char *key = _key ? _key : "";
   u_int16_t klen = strlen(key);
 
-  return ndpi_serialize_start_of_list_binary(_serializer, key, klen);
+  return(ndpi_serialize_start_of_list_binary(_serializer, key, klen));
 }
 
 /* ********************************** */
@@ -1807,7 +1929,7 @@ int ndpi_serialize_start_of_block_binary(ndpi_serializer *_serializer,
   u_int32_t buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
   u_int32_t needed;
 
-  if(serializer->fmt != ndpi_serialization_format_json && 
+  if(serializer->fmt != ndpi_serialization_format_json &&
      serializer->fmt != ndpi_serialization_format_tlv)
     return(-1);
 
@@ -1827,7 +1949,7 @@ int ndpi_serialize_start_of_block_binary(ndpi_serializer *_serializer,
     serializer->status.buffer.size_used += snprintf((char *) &serializer->buffer.data[serializer->status.buffer.size_used], buff_diff, ": {");
     buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
     ndpi_serialize_json_post(_serializer);
-  
+
     serializer->status.flags |= NDPI_SERIALIZER_STATUS_SOB;
   } else /* ndpi_serialization_format_tlv */ {
     serializer->buffer.data[serializer->status.buffer.size_used++] = ndpi_serialization_start_of_block;
@@ -1845,7 +1967,7 @@ int ndpi_serialize_start_of_block(ndpi_serializer *_serializer,
   const char *key = _key ? _key : "";
   u_int16_t klen = strlen(key);
 
-  return ndpi_serialize_start_of_block_binary(_serializer, key, klen);
+  return(ndpi_serialize_start_of_block_binary(_serializer, key, klen));
 }
 
 /* ********************************** */
@@ -1856,7 +1978,7 @@ int ndpi_serialize_end_of_block(ndpi_serializer *_serializer) {
   u_int32_t buff_diff = serializer->buffer.size - serializer->status.buffer.size_used;
   u_int32_t needed;
 
-  if(serializer->fmt != ndpi_serialization_format_json && 
+  if(serializer->fmt != ndpi_serialization_format_json &&
      serializer->fmt != ndpi_serialization_format_tlv)
     return(-1);
 
@@ -1893,8 +2015,8 @@ void ndpi_serializer_create_snapshot(ndpi_serializer *_serializer) {
     (serializer->status.flags & NDPI_SERIALIZER_STATUS_ARRAY) ? " ARRAY" : "",
     (serializer->status.flags & NDPI_SERIALIZER_STATUS_EOR)   ? " EOR"   : ""
   );
-#endif 
- 
+#endif
+
   memcpy(&serializer->snapshot, &serializer->status, sizeof(ndpi_private_serializer_status));
   serializer->has_snapshot = 1;
 }
@@ -1903,7 +2025,7 @@ void ndpi_serializer_create_snapshot(ndpi_serializer *_serializer) {
 
 void ndpi_serializer_rollback_snapshot(ndpi_serializer *_serializer) {
   ndpi_private_serializer *serializer = (ndpi_private_serializer*)_serializer;
-  
+
   if(serializer->has_snapshot) {
     memcpy(&serializer->status, &serializer->snapshot, sizeof(ndpi_private_serializer_status));
     serializer->has_snapshot = 0;
@@ -1915,7 +2037,7 @@ void ndpi_serializer_rollback_snapshot(ndpi_serializer *_serializer) {
         serializer->buffer.data[0] = ' ';
         serializer->buffer.data[serializer->status.buffer.size_used-1] = '}';
       }
-    } 
+    }
   }
 }
 
