@@ -1,7 +1,7 @@
 /*
  * mail_imap.c
  *
- * Copyright (C) 2016-19 - ntop.org
+ * Copyright (C) 2016-22 - ntop.org
  *
  * This file is part of nDPI, an open source deep packet inspection
  * library based on the OpenDPI and PACE technology by ipoque GmbH
@@ -28,15 +28,17 @@
 
 #include "ndpi_api.h"
 
+/* #define IMAP_DEBUG 1*/
 
-static void ndpi_int_mail_imap_add_connection(struct ndpi_detection_module_struct *ndpi_struct, struct ndpi_flow_struct *flow)
-{
-  ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_MAIL_IMAP, NDPI_PROTOCOL_UNKNOWN);
+static void ndpi_int_mail_imap_add_connection(struct ndpi_detection_module_struct *ndpi_struct, struct ndpi_flow_struct *flow,
+					      u_int16_t protocol) {
+  flow->guessed_protocol_id = NDPI_PROTOCOL_UNKNOWN; /* Avoid IMAPS to be used s sub-protocol */
+  ndpi_set_detected_protocol(ndpi_struct, flow, protocol, NDPI_PROTOCOL_UNKNOWN, NDPI_CONFIDENCE_DPI);
 }
 
 void ndpi_search_mail_imap_tcp(struct ndpi_detection_module_struct *ndpi_struct, struct ndpi_flow_struct *flow)
 {
-  struct ndpi_packet_struct *packet = &flow->packet;       
+  struct ndpi_packet_struct *packet = &ndpi_struct->packet;
   u_int16_t i = 0;
   u_int16_t space_pos = 0;
   u_int16_t command_start = 0;
@@ -44,55 +46,58 @@ void ndpi_search_mail_imap_tcp(struct ndpi_detection_module_struct *ndpi_struct,
   /* const u_int8_t *command = 0; */
 
   NDPI_LOG_DBG(ndpi_struct, "search IMAP_IMAP\n");
-  
-  if (flow->l4.tcp.mail_imap_starttls == 2) {
+
+#ifdef IMAP_DEBUG
+  printf("%s() [%.*s]\n", __FUNCTION__, packet->payload_packet_len, packet->payload);
+#endif
+
+  if(flow->l4.tcp.mail_imap_starttls == 2) {
     NDPI_LOG_DBG2(ndpi_struct, "starttls detected\n");
     NDPI_ADD_PROTOCOL_TO_BITMASK(flow->excluded_protocol_bitmask, NDPI_PROTOCOL_MAIL_IMAP);
     NDPI_DEL_PROTOCOL_FROM_BITMASK(flow->excluded_protocol_bitmask, NDPI_PROTOCOL_TLS);
     return;
   }
 
-  if (packet->payload_packet_len >= 4 && ntohs(get_u_int16_t(packet->payload, packet->payload_packet_len - 2)) == 0x0d0a) {
+  if(packet->payload_packet_len >= 4 && ntohs(get_u_int16_t(packet->payload, packet->payload_packet_len - 2)) == 0x0d0a) {
     // the DONE command appears without a tag
-    if (packet->payload_packet_len == 6 && ((packet->payload[0] == 'D' || packet->payload[0] == 'd')
+    if(packet->payload_packet_len == 6 && ((packet->payload[0] == 'D' || packet->payload[0] == 'd')
 					    && (packet->payload[1] == 'O' || packet->payload[1] == 'o')
 					    && (packet->payload[2] == 'N' || packet->payload[2] == 'n')
 					    && (packet->payload[3] == 'E' || packet->payload[3] == 'e'))) {
       flow->l4.tcp.mail_imap_stage += 1;
       saw_command = 1;
     } else {
-
-      if (flow->l4.tcp.mail_imap_stage < 4) {
+      if(flow->l4.tcp.mail_imap_stage < 5) {
 	// search for the first space character (end of the tag)
 	while (i < 20 && i < packet->payload_packet_len) {
-	  if (i > 0 && packet->payload[i] == ' ') {
+	  if(i > 0 && packet->payload[i] == ' ') {
 	    space_pos = i;
 	    break;
 	  }
-	  if (!((packet->payload[i] >= 'a' && packet->payload[i] <= 'z') ||
+	  if(!((packet->payload[i] >= 'a' && packet->payload[i] <= 'z') ||
 		(packet->payload[i] >= 'A' && packet->payload[i] <= 'Z') ||
 		(packet->payload[i] >= '0' && packet->payload[i] <= '9') || packet->payload[i] == '*' || packet->payload[i] == '.')) {
 	    goto imap_excluded;
 	  }
 	  i++;
 	}
-	if (space_pos == 0 || space_pos == (packet->payload_packet_len - 1)) {
+	if(space_pos == 0 || space_pos == (packet->payload_packet_len - 1)) {
 	  goto imap_excluded;
 	}
 	// now walk over a possible mail number to the next space
 	i++;
-	if (i < packet->payload_packet_len && (packet->payload[i] >= '0' && packet->payload[i] <= '9')) {
+	if(i < packet->payload_packet_len && (packet->payload[i] >= '0' && packet->payload[i] <= '9')) {
 	  while (i < 20 && i < packet->payload_packet_len) {
-	    if (i > 0 && packet->payload[i] == ' ') {
+	    if(i > 0 && packet->payload[i] == ' ') {
 	      space_pos = i;
 	      break;
 	    }
-	    if (!(packet->payload[i] >= '0' && packet->payload[i] <= '9')) {
+	    if(!(packet->payload[i] >= '0' && packet->payload[i] <= '9')) {
 	      goto imap_excluded;
 	    }
 	    i++;
 	  }
-	  if (space_pos == 0 || space_pos == (packet->payload_packet_len - 1)) {
+	  if(space_pos == 0 || space_pos == (packet->payload_packet_len - 1)) {
 	    goto imap_excluded;
 	  }
 	}
@@ -103,23 +108,30 @@ void ndpi_search_mail_imap_tcp(struct ndpi_detection_module_struct *ndpi_struct,
 	/* command = &(packet->payload[command_start]); */
       }
 
-      if ((command_start + 3) < packet->payload_packet_len) {
-	if ((packet->payload[command_start] == 'O' || packet->payload[command_start] == 'o')
+      if((command_start + 3) < packet->payload_packet_len) {
+	if((packet->payload[command_start] == 'O' || packet->payload[command_start] == 'o')
 	    && (packet->payload[command_start + 1] == 'K' || packet->payload[command_start + 1] == 'k')
 	    && packet->payload[command_start + 2] == ' ') {
 	  flow->l4.tcp.mail_imap_stage += 1;
-	  if (flow->l4.tcp.mail_imap_starttls == 1)
+	  if(flow->l4.tcp.mail_imap_starttls == 1)
 	    flow->l4.tcp.mail_imap_starttls = 2;
 	  saw_command = 1;
-	} else if ((packet->payload[command_start] == 'U' || packet->payload[command_start] == 'u')
+	} else if((packet->payload[command_start] == 'U' || packet->payload[command_start] == 'u')
 		   && (packet->payload[command_start + 1] == 'I' || packet->payload[command_start + 1] == 'i')
 		   && (packet->payload[command_start + 2] == 'D' || packet->payload[command_start + 2] == 'd')) {
 	  flow->l4.tcp.mail_imap_stage += 1;
 	  saw_command = 1;
+	} else if((packet->payload[command_start] == 'N' || packet->payload[command_start] == 'n')
+	    && (packet->payload[command_start + 1] == 'O' || packet->payload[command_start + 1] == 'o')
+	    && packet->payload[command_start + 2] == ' ') {
+	  flow->l4.tcp.mail_imap_stage += 1;
+	  if(flow->l4.tcp.mail_imap_starttls == 1)
+	    flow->l4.tcp.mail_imap_starttls = 2;
+	  saw_command = 1;
 	}
       }
-      if ((command_start + 10) < packet->payload_packet_len) {
-	if ((packet->payload[command_start] == 'C' || packet->payload[command_start] == 'c')
+      if((command_start + 10) < packet->payload_packet_len) {
+	if((packet->payload[command_start] == 'C' || packet->payload[command_start] == 'c')
 	    && (packet->payload[command_start + 1] == 'A' || packet->payload[command_start + 1] == 'a')
 	    && (packet->payload[command_start + 2] == 'P' || packet->payload[command_start + 2] == 'p')
 	    && (packet->payload[command_start + 3] == 'A' || packet->payload[command_start + 3] == 'a')
@@ -133,8 +145,8 @@ void ndpi_search_mail_imap_tcp(struct ndpi_detection_module_struct *ndpi_struct,
 	  saw_command = 1;
 	}
       }
-      if ((command_start + 8) < packet->payload_packet_len) {
-	if ((packet->payload[command_start] == 'S' || packet->payload[command_start] == 's')
+      if((command_start + 8) < packet->payload_packet_len) {
+	if((packet->payload[command_start] == 'S' || packet->payload[command_start] == 's')
 	    && (packet->payload[command_start + 1] == 'T' || packet->payload[command_start + 1] == 't')
 	    && (packet->payload[command_start + 2] == 'A' || packet->payload[command_start + 2] == 'a')
 	    && (packet->payload[command_start + 3] == 'R' || packet->payload[command_start + 3] == 'r')
@@ -144,40 +156,66 @@ void ndpi_search_mail_imap_tcp(struct ndpi_detection_module_struct *ndpi_struct,
 	    && (packet->payload[command_start + 7] == 'S' || packet->payload[command_start + 7] == 's')) {
         flow->l4.tcp.mail_imap_stage += 1;
         flow->l4.tcp.mail_imap_starttls = 1;
-        flow->detected_protocol_stack[0] = NDPI_PROTOCOL_MAIL_IMAPS;
+        ndpi_int_mail_imap_add_connection(ndpi_struct, flow, NDPI_PROTOCOL_MAIL_IMAPS);
         saw_command = 1;
 	}
       }
-      if ((command_start + 5) < packet->payload_packet_len) {
-	if ((packet->payload[command_start] == 'L' || packet->payload[command_start] == 'l')
+      if((command_start + 5) < packet->payload_packet_len) {
+	if((packet->payload[command_start] == 'L' || packet->payload[command_start] == 'l')
 	    && (packet->payload[command_start + 1] == 'O' || packet->payload[command_start + 1] == 'o')
 	    && (packet->payload[command_start + 2] == 'G' || packet->payload[command_start + 2] == 'g')
 	    && (packet->payload[command_start + 3] == 'I' || packet->payload[command_start + 3] == 'i')
 	    && (packet->payload[command_start + 4] == 'N' || packet->payload[command_start + 4] == 'n')) {
+	  /* xxxx LOGIN "username" "password"
+	     xxxx LOGIN username password */
+	  char str[256], *user, *saveptr;
+	  u_int len = ndpi_min(packet->payload_packet_len - (command_start + 5), (int)sizeof(str) - 1);
+
+	  strncpy(str, (const char*)packet->payload + command_start + 5, len);
+	  str[len] = '\0';
+
+	  user = strtok_r(str, " \"\r\n", &saveptr);
+	  if(user) {
+	    char *pwd;
+
+	    snprintf(flow->l4.tcp.ftp_imap_pop_smtp.username,
+		     sizeof(flow->l4.tcp.ftp_imap_pop_smtp.username),
+		     "%s", user);
+
+	    ndpi_set_risk(ndpi_struct, flow, NDPI_CLEAR_TEXT_CREDENTIALS);
+
+	    pwd = strtok_r(NULL, " \"\r\n", &saveptr);
+	    if(pwd) {
+	      snprintf(flow->l4.tcp.ftp_imap_pop_smtp.password,
+		       sizeof(flow->l4.tcp.ftp_imap_pop_smtp.password),
+	               "%s", pwd);
+	    }
+	  }
+	  
 	  flow->l4.tcp.mail_imap_stage += 1;
 	  saw_command = 1;
-	} else if ((packet->payload[command_start] == 'F' || packet->payload[command_start] == 'f')
+	} else if((packet->payload[command_start] == 'F' || packet->payload[command_start] == 'f')
 		   && (packet->payload[command_start + 1] == 'E' || packet->payload[command_start + 1] == 'e')
 		   && (packet->payload[command_start + 2] == 'T' || packet->payload[command_start + 2] == 't')
 		   && (packet->payload[command_start + 3] == 'C' || packet->payload[command_start + 3] == 'c')
 		   && (packet->payload[command_start + 4] == 'H' || packet->payload[command_start + 4] == 'h')) {
 	  flow->l4.tcp.mail_imap_stage += 1;
 	  saw_command = 1;
-	} else if ((packet->payload[command_start] == 'F' || packet->payload[command_start] == 'f')
+	} else if((packet->payload[command_start] == 'F' || packet->payload[command_start] == 'f')
 		   && (packet->payload[command_start + 1] == 'L' || packet->payload[command_start + 1] == 'l')
 		   && (packet->payload[command_start + 2] == 'A' || packet->payload[command_start + 2] == 'a')
 		   && (packet->payload[command_start + 3] == 'G' || packet->payload[command_start + 3] == 'g')
 		   && (packet->payload[command_start + 4] == 'S' || packet->payload[command_start + 4] == 's')) {
 	  flow->l4.tcp.mail_imap_stage += 1;
 	  saw_command = 1;
-	} else if ((packet->payload[command_start] == 'C' || packet->payload[command_start] == 'c')
+	} else if((packet->payload[command_start] == 'C' || packet->payload[command_start] == 'c')
 		   && (packet->payload[command_start + 1] == 'H' || packet->payload[command_start + 1] == 'h')
 		   && (packet->payload[command_start + 2] == 'E' || packet->payload[command_start + 2] == 'e')
 		   && (packet->payload[command_start + 3] == 'C' || packet->payload[command_start + 3] == 'c')
 		   && (packet->payload[command_start + 4] == 'K' || packet->payload[command_start + 4] == 'k')) {
 	  flow->l4.tcp.mail_imap_stage += 1;
 	  saw_command = 1;
-	} else if ((packet->payload[command_start] == 'S' || packet->payload[command_start] == 's')
+	} else if((packet->payload[command_start] == 'S' || packet->payload[command_start] == 's')
 		   && (packet->payload[command_start + 1] == 'T' || packet->payload[command_start + 1] == 't')
 		   && (packet->payload[command_start + 2] == 'O' || packet->payload[command_start + 2] == 'o')
 		   && (packet->payload[command_start + 3] == 'R' || packet->payload[command_start + 3] == 'r')
@@ -186,8 +224,8 @@ void ndpi_search_mail_imap_tcp(struct ndpi_detection_module_struct *ndpi_struct,
 	  saw_command = 1;
 	}
       }
-      if ((command_start + 12) < packet->payload_packet_len) {
-	if ((packet->payload[command_start] == 'A' || packet->payload[command_start] == 'a')
+      if((command_start + 12) < packet->payload_packet_len) {
+	if((packet->payload[command_start] == 'A' || packet->payload[command_start] == 'a')
 	    && (packet->payload[command_start + 1] == 'U' || packet->payload[command_start + 1] == 'u')
 	    && (packet->payload[command_start + 2] == 'T' || packet->payload[command_start + 2] == 't')
 	    && (packet->payload[command_start + 3] == 'H' || packet->payload[command_start + 3] == 'h')
@@ -200,11 +238,15 @@ void ndpi_search_mail_imap_tcp(struct ndpi_detection_module_struct *ndpi_struct,
 	    && (packet->payload[command_start + 10] == 'T' || packet->payload[command_start + 10] == 't')
 	    && (packet->payload[command_start + 11] == 'E' || packet->payload[command_start + 11] == 'e')) {
 	  flow->l4.tcp.mail_imap_stage += 1;
+	  /* Authenticate phase may have multiple messages. Ignore them since they are
+	     somehow encrypted anyway. */
+	  flow->l4.tcp.mail_imap_starttls = 2;
+          ndpi_int_mail_imap_add_connection(ndpi_struct, flow, NDPI_PROTOCOL_MAIL_IMAPS);
 	  saw_command = 1;
 	}
       }
-      if ((command_start + 9) < packet->payload_packet_len) {
-	if ((packet->payload[command_start] == 'N' || packet->payload[command_start] == 'n')
+      if((command_start + 9) < packet->payload_packet_len) {
+	if((packet->payload[command_start] == 'N' || packet->payload[command_start] == 'n')
 	    && (packet->payload[command_start + 1] == 'A' || packet->payload[command_start + 1] == 'a')
 	    && (packet->payload[command_start + 2] == 'M' || packet->payload[command_start + 2] == 'm')
 	    && (packet->payload[command_start + 3] == 'E' || packet->payload[command_start + 3] == 'e')
@@ -217,26 +259,26 @@ void ndpi_search_mail_imap_tcp(struct ndpi_detection_module_struct *ndpi_struct,
 	  saw_command = 1;
 	}
       }
-      if ((command_start + 4) < packet->payload_packet_len) {
-	if ((packet->payload[command_start] == 'L' || packet->payload[command_start] == 'l')
+      if((command_start + 4) < packet->payload_packet_len) {
+	if((packet->payload[command_start] == 'L' || packet->payload[command_start] == 'l')
 	    && (packet->payload[command_start + 1] == 'S' || packet->payload[command_start + 1] == 's')
 	    && (packet->payload[command_start + 2] == 'U' || packet->payload[command_start + 2] == 'u')
 	    && (packet->payload[command_start + 3] == 'B' || packet->payload[command_start + 3] == 'b')) {
 	  flow->l4.tcp.mail_imap_stage += 1;
 	  saw_command = 1;
-	} else if ((packet->payload[command_start] == 'L' || packet->payload[command_start] == 'l')
+	} else if((packet->payload[command_start] == 'L' || packet->payload[command_start] == 'l')
 		   && (packet->payload[command_start + 1] == 'I' || packet->payload[command_start + 1] == 'i')
 		   && (packet->payload[command_start + 2] == 'S' || packet->payload[command_start + 2] == 's')
 		   && (packet->payload[command_start + 3] == 'T' || packet->payload[command_start + 3] == 't')) {
 	  flow->l4.tcp.mail_imap_stage += 1;
 	  saw_command = 1;
-	} else if ((packet->payload[command_start] == 'N' || packet->payload[command_start] == 'n')
+	} else if((packet->payload[command_start] == 'N' || packet->payload[command_start] == 'n')
 		   && (packet->payload[command_start + 1] == 'O' || packet->payload[command_start + 1] == 'o')
 		   && (packet->payload[command_start + 2] == 'O' || packet->payload[command_start + 2] == 'o')
 		   && (packet->payload[command_start + 3] == 'P' || packet->payload[command_start + 3] == 'p')) {
 	  flow->l4.tcp.mail_imap_stage += 1;
 	  saw_command = 1;
-	} else if ((packet->payload[command_start] == 'I' || packet->payload[command_start] == 'i')
+	} else if((packet->payload[command_start] == 'I' || packet->payload[command_start] == 'i')
 		   && (packet->payload[command_start + 1] == 'D' || packet->payload[command_start + 1] == 'd')
 		   && (packet->payload[command_start + 2] == 'L' || packet->payload[command_start + 2] == 'l')
 		   && (packet->payload[command_start + 3] == 'E' || packet->payload[command_start + 3] == 'e')) {
@@ -244,8 +286,8 @@ void ndpi_search_mail_imap_tcp(struct ndpi_detection_module_struct *ndpi_struct,
 	  saw_command = 1;
 	}
       }
-      if ((command_start + 6) < packet->payload_packet_len) {
-	if ((packet->payload[command_start] == 'S' || packet->payload[command_start] == 's')
+      if((command_start + 6) < packet->payload_packet_len) {
+	if((packet->payload[command_start] == 'S' || packet->payload[command_start] == 's')
 	    && (packet->payload[command_start + 1] == 'E' || packet->payload[command_start + 1] == 'e')
 	    && (packet->payload[command_start + 2] == 'L' || packet->payload[command_start + 2] == 'l')
 	    && (packet->payload[command_start + 3] == 'E' || packet->payload[command_start + 3] == 'e')
@@ -253,7 +295,7 @@ void ndpi_search_mail_imap_tcp(struct ndpi_detection_module_struct *ndpi_struct,
 	    && (packet->payload[command_start + 5] == 'T' || packet->payload[command_start + 5] == 't')) {
 	  flow->l4.tcp.mail_imap_stage += 1;
 	  saw_command = 1;
-	} else if ((packet->payload[command_start] == 'E' || packet->payload[command_start] == 'e')
+	} else if((packet->payload[command_start] == 'E' || packet->payload[command_start] == 'e')
 		   && (packet->payload[command_start + 1] == 'X' || packet->payload[command_start + 1] == 'x')
 		   && (packet->payload[command_start + 2] == 'I' || packet->payload[command_start + 2] == 'i')
 		   && (packet->payload[command_start + 3] == 'S' || packet->payload[command_start + 3] == 's')
@@ -261,7 +303,7 @@ void ndpi_search_mail_imap_tcp(struct ndpi_detection_module_struct *ndpi_struct,
 		   && (packet->payload[command_start + 5] == 'S' || packet->payload[command_start + 5] == 's')) {
 	  flow->l4.tcp.mail_imap_stage += 1;
 	  saw_command = 1;
-	} else if ((packet->payload[command_start] == 'A' || packet->payload[command_start] == 'a')
+	} else if((packet->payload[command_start] == 'A' || packet->payload[command_start] == 'a')
 		   && (packet->payload[command_start + 1] == 'P' || packet->payload[command_start + 1] == 'p')
 		   && (packet->payload[command_start + 2] == 'P' || packet->payload[command_start + 2] == 'p')
 		   && (packet->payload[command_start + 3] == 'E' || packet->payload[command_start + 3] == 'e')
@@ -274,16 +316,23 @@ void ndpi_search_mail_imap_tcp(struct ndpi_detection_module_struct *ndpi_struct,
 
     }
 
-    if (saw_command == 1) {
-      if (flow->l4.tcp.mail_imap_stage == 3 || flow->l4.tcp.mail_imap_stage == 5) {
-	NDPI_LOG_INFO(ndpi_struct, "found MAIL_IMAP\n");
-	ndpi_int_mail_imap_add_connection(ndpi_struct, flow);
+    if(saw_command == 1) {
+      if((flow->l4.tcp.mail_imap_stage == 3)
+	 || (flow->l4.tcp.mail_imap_stage == 5)
+	 || (flow->l4.tcp.mail_imap_stage == 7)
+        ) {
+	if((flow->l4.tcp.ftp_imap_pop_smtp.username[0] != '\0')
+	   || (flow->l4.tcp.mail_imap_stage >= 7)) {
+	  NDPI_LOG_INFO(ndpi_struct, "found MAIL_IMAP\n");
+	  ndpi_int_mail_imap_add_connection(ndpi_struct, flow, NDPI_PROTOCOL_MAIL_IMAP);
+	}
+	
 	return;
       }
     }
   }
 
-  if (packet->payload_packet_len > 1 && packet->payload[packet->payload_packet_len - 1] == ' ') {
+  if(packet->payload_packet_len > 1 && packet->payload[packet->payload_packet_len - 1] == ' ') {
     NDPI_LOG_DBG2(ndpi_struct,
 	     "maybe a split imap command -> need next packet and imap_stage is set to 4.\n");
     flow->l4.tcp.mail_imap_stage = 4;
@@ -294,8 +343,8 @@ void ndpi_search_mail_imap_tcp(struct ndpi_detection_module_struct *ndpi_struct,
 
   // skip over possible authentication hashes etc. that cannot be identified as imap commands or responses
   // if the packet count is low enough and at least one command or response was seen before
-  if ((packet->payload_packet_len >= 2 && ntohs(get_u_int16_t(packet->payload, packet->payload_packet_len - 2)) == 0x0d0a)
-      && flow->packet_counter < 6 && flow->l4.tcp.mail_imap_stage >= 1) {
+  if((packet->payload_packet_len >= 2 && ntohs(get_u_int16_t(packet->payload, packet->payload_packet_len - 2)) == 0x0d0a)
+      && flow->packet_counter < 8 && flow->l4.tcp.mail_imap_stage >= 1) {
     NDPI_LOG_DBG2(ndpi_struct,
 	     "no imap command or response but packet count < 6 and imap stage >= 1 -> skip\n");
     return;
