@@ -1,7 +1,7 @@
 /*
  * mail_smtp.c
  *
- * Copyright (C) 2011-21 - ntop.org
+ * Copyright (C) 2011-22 - ntop.org
  * Copyright (C) 2009-11 - ipoque GmbH
  *
  * This file is part of nDPI, an open source deep packet inspection
@@ -38,12 +38,13 @@
 #define SMTP_BIT_HELO_EHLO	0x20
 #define SMTP_BIT_MAIL		0x40
 #define SMTP_BIT_RCPT		0x80
-#define SMTP_BIT_AUTH		0x100
+#define SMTP_BIT_AUTH_LOGIN	0x100
 #define SMTP_BIT_STARTTLS	0x200
 #define SMTP_BIT_DATA		0x400
 #define SMTP_BIT_NOOP		0x800
 #define SMTP_BIT_RSET		0x1000
 #define SMTP_BIT_TlRM		0x2000
+#define SMTP_BIT_AUTH_PLAIN	0x4000
 
 /* #define SMTP_DEBUG 1 */
 
@@ -52,9 +53,11 @@ static void ndpi_int_mail_smtp_add_connection(struct ndpi_detection_module_struc
 #ifdef SMTP_DEBUG
   printf("**** %s()\n", __FUNCTION__);
 #endif
-  
+
+  flow->guessed_protocol_id = NDPI_PROTOCOL_MAIL_SMTP; /* Avoid SMTPS to be used s sub-protocol */
+
   ndpi_set_detected_protocol(ndpi_struct, flow,
-			     NDPI_PROTOCOL_MAIL_SMTP, NDPI_PROTOCOL_UNKNOWN);
+			     NDPI_PROTOCOL_MAIL_SMTP, NDPI_PROTOCOL_UNKNOWN, NDPI_CONFIDENCE_DPI);
 }
 
 /* **************************************** */
@@ -63,9 +66,56 @@ static void smtpInitExtraPacketProcessing(struct ndpi_flow_struct *flow);
 
 /* **************************************** */
 
+static void get_credentials_auth_plain(struct ndpi_detection_module_struct *ndpi_struct,
+				       struct ndpi_flow_struct *flow,
+				       const u_int8_t *line, u_int16_t line_len)
+{
+  u_int8_t buf[255];
+  u_char *out;
+  size_t i, out_len;
+  unsigned int user_len = 0;
+
+  /* AUTH PLAIN XXXXXX */
+  if(line_len <= 11)
+    return;
+
+  line += 11;
+  line_len -= 11;
+
+  ndpi_user_pwd_payload_copy(buf, sizeof(buf), 0, line, line_len);
+  out = ndpi_base64_decode(buf, strlen((char *)buf), &out_len);
+  if(!out)
+    return;
+  /* No guarantee that out is null terminated:
+     UTF8NUL authcid UTF8NUL passwd */
+  for(i = 1; i < out_len; i++) {
+    if(out[i] == '\0')
+      user_len = i - 1;
+  }
+  if(user_len > 0) {
+    user_len = ndpi_min(user_len, sizeof(flow->l4.tcp.ftp_imap_pop_smtp.username) - 1);
+
+    memcpy(flow->l4.tcp.ftp_imap_pop_smtp.username, out + 1, user_len);
+    flow->l4.tcp.ftp_imap_pop_smtp.username[user_len] = '\0';
+
+    ndpi_set_risk(ndpi_struct, flow, NDPI_CLEAR_TEXT_CREDENTIALS);
+
+    if(1 + user_len + 1 < out_len) {
+      unsigned int pwd_len;
+
+      pwd_len = ndpi_min(out_len - (1 + user_len + 1), sizeof(flow->l4.tcp.ftp_imap_pop_smtp.password) - 1);
+      memcpy(flow->l4.tcp.ftp_imap_pop_smtp.password, out + 1 + user_len + 1, pwd_len);
+      flow->l4.tcp.ftp_imap_pop_smtp.password[pwd_len] = '\0';
+    }
+  }
+  ndpi_free(out);
+}
+
+/* **************************************** */
+
 void ndpi_search_mail_smtp_tcp(struct ndpi_detection_module_struct *ndpi_struct,
 			       struct ndpi_flow_struct *flow) {
-  struct ndpi_packet_struct *packet = &flow->packet;
+  struct ndpi_packet_struct *packet = &ndpi_struct->packet;
 
   NDPI_LOG_DBG(ndpi_struct, "search mail_smtp\n");
 
@@ -83,6 +133,34 @@ void ndpi_search_mail_smtp_tcp(struct ndpi_detection_module_struct *ndpi_struct,
       if(packet->line[a].len >= 3) {
 	if(memcmp(packet->line[a].ptr, "220", 3) == 0) {
 	  flow->l4.tcp.smtp_command_bitmask |= SMTP_BIT_220;
+
+	  if(flow->host_server_name[0] == '\0') {
+	    if(packet->line[a].len > 4) {
+	      int i;
+	      unsigned int len;
+
+	      if(packet->line[a].ptr[4] != '(') {
+		for(i=5; (i<packet->line[a].len-1) && (packet->line[a].ptr[i] != ' '); i++)
+		  ;
+
+		if((packet->line[a].ptr[i+1] != '\r')
+		   && (packet->line[a].ptr[i+1] != '\n')) {
+		  len = i-4;
+		  /* Copy result for nDPI apps */
+		  ndpi_hostname_sni_set(flow, &packet->line[a].ptr[4], len);
+
+		  if (ndpi_match_hostname_protocol(ndpi_struct, flow, NDPI_PROTOCOL_MAIL_SMTP,
+						   flow->host_server_name,
+						   strlen(flow->host_server_name))) {
+		    /* We set the protocols; we need to initialize extra dissection
+		       to search for credentials */
+		    NDPI_LOG_DBG(ndpi_struct, "SMTP: hostname matched\n");
+		    smtpInitExtraPacketProcessing(flow);
+		  }
+		}
+	      }
+	    }
+	  }
 	} else if(memcmp(packet->line[a].ptr, "250", 3) == 0) {
 	  flow->l4.tcp.smtp_command_bitmask |= SMTP_BIT_250;
 	} else if(memcmp(packet->line[a].ptr, "235", 3) == 0) {
@@ -104,21 +182,25 @@ void ndpi_search_mail_smtp_tcp(struct ndpi_detection_module_struct *ndpi_struct,
 	   && (packet->line[a].ptr[3] == 'O' || packet->line[a].ptr[3] == 'o')
 	   && packet->line[a].ptr[4] == ' ') {
 	  flow->l4.tcp.smtp_command_bitmask |= SMTP_BIT_HELO_EHLO;
-	  flow->protos.ftp_imap_pop_smtp.auth_found = 0;
+	  flow->l4.tcp.ftp_imap_pop_smtp.auth_found = 0;
 	} else if((packet->line[a].ptr[0] == 'M' || packet->line[a].ptr[0] == 'm')
 		  && (packet->line[a].ptr[1] == 'A' || packet->line[a].ptr[1] == 'a')
 		  && (packet->line[a].ptr[2] == 'I' || packet->line[a].ptr[2] == 'i')
 		  && (packet->line[a].ptr[3] == 'L' || packet->line[a].ptr[3] == 'l')
 		  && packet->line[a].ptr[4] == ' ') {
 	  flow->l4.tcp.smtp_command_bitmask |= SMTP_BIT_MAIL;
-	  flow->protos.ftp_imap_pop_smtp.auth_found = 0;
+	  flow->l4.tcp.ftp_imap_pop_smtp.auth_found = 0;
+	  /* We shouldn't be here if there are credentials */
+	  flow->l4.tcp.ftp_imap_pop_smtp.auth_done = 1;
 	} else if((packet->line[a].ptr[0] == 'R' || packet->line[a].ptr[0] == 'r')
 		  && (packet->line[a].ptr[1] == 'C' || packet->line[a].ptr[1] == 'c')
 		  && (packet->line[a].ptr[2] == 'P' || packet->line[a].ptr[2] == 'p')
 		  && (packet->line[a].ptr[3] == 'T' || packet->line[a].ptr[3] == 't')
 		  && packet->line[a].ptr[4] == ' ') {
 	  flow->l4.tcp.smtp_command_bitmask |= SMTP_BIT_RCPT;
-	  flow->protos.ftp_imap_pop_smtp.auth_found = 0;
+	  flow->l4.tcp.ftp_imap_pop_smtp.auth_found = 0;
+	  /* We shouldn't be here if there are credentials */
+	  flow->l4.tcp.ftp_imap_pop_smtp.auth_done = 1;
 	} else if((packet->line[a].ptr[0] == 'A' || packet->line[a].ptr[0] == 'a')
 		  && (packet->line[a].ptr[1] == 'U' || packet->line[a].ptr[1] == 'u')
 		  && (packet->line[a].ptr[2] == 'T' || packet->line[a].ptr[2] == 't')
@@ -127,17 +209,28 @@ void ndpi_search_mail_smtp_tcp(struct ndpi_detection_module_struct *ndpi_struct,
 #ifdef SMTP_DEBUG
 	  printf("%s() AUTH [%.*s]\n", __FUNCTION__, packet->line[a].len, packet->line[a].ptr);
 #endif
-
-	  flow->l4.tcp.smtp_command_bitmask |= SMTP_BIT_AUTH;
-	  flow->protos.ftp_imap_pop_smtp.auth_found = 1;
+	  flow->l4.tcp.ftp_imap_pop_smtp.auth_found = 1;
+	  if(packet->line[a].len >= 6) {
+            if(packet->line[a].ptr[5] == 'L' || packet->line[a].ptr[5] == 'l') {
+	      flow->l4.tcp.smtp_command_bitmask |= SMTP_BIT_AUTH_LOGIN;
+	      /* AUTH LOGIN: Username and pwd on the next messages */
+	    } else if(packet->line[a].ptr[5] == 'P' || packet->line[a].ptr[5] == 'p') {
+	      flow->l4.tcp.smtp_command_bitmask |= SMTP_BIT_AUTH_PLAIN;
+	      /* AUTH PLAIN: username and pwd here */
+	      get_credentials_auth_plain(ndpi_struct, flow,
+					 packet->line[a].ptr, packet->line[a].len);
+	      flow->l4.tcp.ftp_imap_pop_smtp.auth_done = 1;
+	    }
+	  }
 	} else {
 	  if(packet->line[a].ptr[3] != ' ') {
 #ifdef SMTP_DEBUG
 	    printf("%s() => [%.*s]\n", __FUNCTION__, packet->line[a].len, packet->line[a].ptr);
 #endif
-	    
-	    if(flow->protos.ftp_imap_pop_smtp.auth_found) {
-	      if(flow->protos.ftp_imap_pop_smtp.username[0] == '\0') {
+
+	    if(flow->l4.tcp.ftp_imap_pop_smtp.auth_found &&
+	       (flow->l4.tcp.smtp_command_bitmask & SMTP_BIT_AUTH_LOGIN)) {
+	      if(flow->l4.tcp.ftp_imap_pop_smtp.username[0] == '\0') {
 		/* Username */
 		u_int8_t buf[48];
 		u_char *out;
@@ -147,20 +240,22 @@ void ndpi_search_mail_smtp_tcp(struct ndpi_detection_module_struct *ndpi_struct,
 					   packet->line[a].ptr, packet->line[a].len);
 
 #ifdef SMTP_DEBUG
-		printf("%s() => [auth: %u] (username) [%s]\n", __FUNCTION__, flow->protos.ftp_imap_pop_smtp.auth_found, buf);
+		printf("%s() => [auth: %u] (username) [%s]\n", __FUNCTION__, flow->l4.tcp.ftp_imap_pop_smtp.auth_found, buf);
 #endif
 
 		out = ndpi_base64_decode((const u_char*)buf, (size_t)strlen((const char*)buf), &out_len);
 
 		if(out) {
-		  size_t len = ndpi_min(out_len, sizeof(flow->protos.ftp_imap_pop_smtp.username) - 1);
+		  size_t len = ndpi_min(out_len, sizeof(flow->l4.tcp.ftp_imap_pop_smtp.username) - 1);
+
+		  memcpy(flow->l4.tcp.ftp_imap_pop_smtp.username, out, len);
+		  flow->l4.tcp.ftp_imap_pop_smtp.username[len] = '\0';
 
-		  memcpy(flow->protos.ftp_imap_pop_smtp.username, out, len);
-		  flow->protos.ftp_imap_pop_smtp.username[len] = '\0';
-		  
 		  ndpi_free(out);
 		}
-	      } else if(flow->protos.ftp_imap_pop_smtp.password[0] == '\0') {
+		
+		ndpi_set_risk(ndpi_struct, flow, NDPI_CLEAR_TEXT_CREDENTIALS);
+	      } else if(flow->l4.tcp.ftp_imap_pop_smtp.password[0] == '\0') {
 		/* Password */
 		u_int8_t buf[48];
 		u_char *out;
@@ -170,20 +265,25 @@ void ndpi_search_mail_smtp_tcp(struct ndpi_detection_module_struct *ndpi_struct,
 					   packet->line[a].ptr, packet->line[a].len);
 
 #ifdef SMTP_DEBUG
-		printf("%s() => [auth: %u] (password) [%s]\n", __FUNCTION__, flow->protos.ftp_imap_pop_smtp.auth_found, buf);
+		printf("%s() => [auth: %u] (password) [%s]\n", __FUNCTION__, flow->l4.tcp.ftp_imap_pop_smtp.auth_found, buf);
 #endif
 
 		out = ndpi_base64_decode((const u_char*)buf, (size_t)strlen((const char*)buf), &out_len);
 
 		if(out) {
-		  size_t len = ndpi_min(out_len, sizeof(flow->protos.ftp_imap_pop_smtp.password) - 1);
+		  size_t len = ndpi_min(out_len, sizeof(flow->l4.tcp.ftp_imap_pop_smtp.password) - 1);
 
-		  memcpy(flow->protos.ftp_imap_pop_smtp.password, out, len);
-		  flow->protos.ftp_imap_pop_smtp.password[len] = '\0';
+		  memcpy(flow->l4.tcp.ftp_imap_pop_smtp.password, out, len);
+		  flow->l4.tcp.ftp_imap_pop_smtp.password[len] = '\0';
 
 		  ndpi_free(out);
 		}
+
+		ndpi_set_risk(ndpi_struct, flow, NDPI_CLEAR_TEXT_CREDENTIALS);
+
+		flow->l4.tcp.ftp_imap_pop_smtp.auth_done = 1;
 	      } else {
+		flow->host_server_name[0] = '\0';
 		NDPI_EXCLUDE_PROTO(ndpi_struct, flow);
 		return;
 	      }
@@ -202,6 +302,8 @@ void ndpi_search_mail_smtp_tcp(struct ndpi_detection_module_struct *ndpi_struct,
 	   && (packet->line[a].ptr[6] == 'L' || packet->line[a].ptr[6] == 'l')
 	   && (packet->line[a].ptr[7] == 'S' || packet->line[a].ptr[7] == 's')) {
 	  flow->l4.tcp.smtp_command_bitmask |= SMTP_BIT_STARTTLS;
+	  flow->l4.tcp.ftp_imap_pop_smtp.auth_tls = 1;
+	  flow->l4.tcp.ftp_imap_pop_smtp.auth_done = 1;
 	}
       }
 
@@ -239,11 +341,15 @@ void ndpi_search_mail_smtp_tcp(struct ndpi_detection_module_struct *ndpi_struct,
 
 #ifdef SMTP_DEBUG
       printf("%s() [bit_count: %u][%s]\n", __FUNCTION__,
-	     bit_count, flow->protos.ftp_imap_pop_smtp.password);
+	     bit_count, flow->l4.tcp.ftp_imap_pop_smtp.password);
 #endif
 
-      ndpi_int_mail_smtp_add_connection(ndpi_struct, flow);
-      smtpInitExtraPacketProcessing(flow);      
+      /* Only if we don't have already set the protocol via hostname matching */
+      if(flow->detected_protocol_stack[0] == NDPI_PROTOCOL_UNKNOWN &&
+         flow->detected_protocol_stack[1] == NDPI_PROTOCOL_UNKNOWN) {
+        ndpi_int_mail_smtp_add_connection(ndpi_struct, flow);
+        smtpInitExtraPacketProcessing(flow);
+      }
       return;
     }
 
@@ -270,14 +376,14 @@ void ndpi_search_mail_smtp_tcp(struct ndpi_detection_module_struct *ndpi_struct,
 int ndpi_extra_search_mail_smtp_tcp(struct ndpi_detection_module_struct *ndpi_struct,
 				    struct ndpi_flow_struct *flow) {
   int rc;
-  
+
   ndpi_search_mail_smtp_tcp(ndpi_struct, flow);
 
-  rc = (flow->protos.ftp_imap_pop_smtp.password[0] == '\0') ? 1 : 0;
-  
+  rc = (flow->l4.tcp.ftp_imap_pop_smtp.password[0] == '\0') ? 1 : 0;
+
 #ifdef SMTP_DEBUG
   printf("**** %s() [rc: %d]\n", __FUNCTION__, rc);
-#endif  
+#endif
 
   return(rc);
 }
@@ -287,12 +393,12 @@ int ndpi_extra_search_mail_smtp_tcp(struct ndpi_detection_module_struct *ndpi_st
 static void smtpInitExtraPacketProcessing(struct ndpi_flow_struct *flow) {
 #ifdef SMTP_DEBUG
   static u_int num = 0;
-  
+
   printf("**** %s(%u)\n", __FUNCTION__, ++num);
 #endif
-  
+
   flow->check_extra_packets = 1;
-  /* At most 7 packets should almost always be enough */
+  /* At most 12 packets should almost always be enough */
   flow->max_extra_packets_to_check = 12;
   flow->extra_packets_func = ndpi_extra_search_mail_smtp_tcp;
 }
