@@ -1,7 +1,7 @@
 /*
  * mining.c [Bitcoin, Ethereum, ZCash, Monero]
  *
- * Copyright (C) 2018-20 - ntop.org
+ * Copyright (C) 2018-22 - ntop.org
  *
  * This file is part of nDPI, an open source deep packet inspection
  * library based on the OpenDPI and PACE technology by ipoque GmbH
@@ -26,9 +26,19 @@
 
 /* ************************************************************************** */
 
+static void cacheMiningHostTwins(struct ndpi_detection_module_struct *ndpi_struct,
+				 u_int32_t host_keys /* network byte order */) {
+  if(ndpi_struct->mining_cache == NULL) ndpi_struct->mining_cache = ndpi_lru_cache_init(1024);
+  
+  if(ndpi_struct->mining_cache)
+    ndpi_lru_add_to_cache(ndpi_struct->mining_cache, host_keys, NDPI_PROTOCOL_MINING);
+}
+
+/* ************************************************************************** */
+
 void ndpi_search_mining_udp(struct ndpi_detection_module_struct *ndpi_struct,
 			    struct ndpi_flow_struct *flow) {
-  struct ndpi_packet_struct *packet = &flow->packet;
+  struct ndpi_packet_struct *packet = &ndpi_struct->packet;
   u_int16_t source = ntohs(packet->udp->source);
   u_int16_t dest = ntohs(packet->udp->dest);
 
@@ -39,7 +49,8 @@ void ndpi_search_mining_udp(struct ndpi_detection_module_struct *ndpi_struct,
      Ethereum P2P Discovery Protocol
      https://github.com/ConsenSys/ethereum-dissectors/blob/master/packet-ethereum-disc.c
   */
-  if((packet->payload_packet_len > 98)
+  if(packet->udp
+     && (packet->payload_packet_len > 98)
      && (packet->payload_packet_len < 1280)
      && ((source == 30303) || (dest == 30303))
      && (packet->payload[97] <= 0x04 /* NODES */)
@@ -50,7 +61,9 @@ void ndpi_search_mining_udp(struct ndpi_detection_module_struct *ndpi_struct,
       ;
     else {
       snprintf(flow->flow_extra_info, sizeof(flow->flow_extra_info), "%s", "ETH");
-      ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_MINING, NDPI_PROTOCOL_UNKNOWN);
+      ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_MINING, NDPI_PROTOCOL_UNKNOWN, NDPI_CONFIDENCE_DPI);
+      if(packet->iph) /* TODO: ipv6 */
+        cacheMiningHostTwins(ndpi_struct, packet->iph->saddr + packet->iph->daddr);
       return;
     }
   }
@@ -60,15 +73,20 @@ void ndpi_search_mining_udp(struct ndpi_detection_module_struct *ndpi_struct,
 
 /* ************************************************************************** */
 
+static u_int8_t isEthPort(u_int16_t dport) {
+  return(((dport >= 30300) && (dport <= 30305)) ? 1 : 0);
+}
+
+/* ************************************************************************** */
+
 void ndpi_search_mining_tcp(struct ndpi_detection_module_struct *ndpi_struct,
 			    struct ndpi_flow_struct *flow) {
-  struct ndpi_packet_struct *packet = &flow->packet;
+  struct ndpi_packet_struct *packet = &ndpi_struct->packet;
 
   NDPI_LOG_DBG(ndpi_struct, "search MINING TCP\n");
 
   /* Check connection over TCP */
-  if(packet->payload_packet_len > 10) {
-
+  if(packet->tcp && (packet->payload_packet_len > 10)) {
     if(packet->tcp->source == htons(8333)) {
       /*
 	Bitcoin
@@ -79,16 +97,26 @@ void ndpi_search_mining_tcp(struct ndpi_detection_module_struct *ndpi_struct,
       
       if((*to_match == magic) || (*to_match == magic1)) {
 	snprintf(flow->flow_extra_info, sizeof(flow->flow_extra_info), "%s", "ETH");
-	ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_MINING, NDPI_PROTOCOL_UNKNOWN);
+	ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_MINING, NDPI_PROTOCOL_UNKNOWN, NDPI_CONFIDENCE_DPI);
+	if(packet->iph) /* TODO: ipv6 */
+	  cacheMiningHostTwins(ndpi_struct, packet->iph->saddr + packet->iph->daddr);
+	return;
       }
     }
 
-    if((packet->payload_packet_len > 450)
+    if((packet->payload_packet_len > 300)
        && (packet->payload_packet_len < 600)
-       && (packet->tcp->dest == htons(30303) /* Ethereum port */)
        && (packet->payload[2] == 0x04)) {
-      snprintf(flow->flow_extra_info, sizeof(flow->flow_extra_info), "%s", "ETH");
-      ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_MINING, NDPI_PROTOCOL_UNKNOWN);
+
+      if(isEthPort(ntohs(packet->tcp->dest)) /* Ethereum port */) {
+	snprintf(flow->flow_extra_info, sizeof(flow->flow_extra_info), "%s", "ETH");
+	ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_MINING, NDPI_PROTOCOL_UNKNOWN, NDPI_CONFIDENCE_DPI);
+	if(packet->iph) /* TODO: ipv6 */
+	  cacheMiningHostTwins(ndpi_struct, packet->iph->saddr + packet->iph->daddr);
+	return;
+      } else
+	flow->guessed_protocol_id = NDPI_PROTOCOL_MINING;
+      
     } else if(ndpi_strnstr((const char *)packet->payload, "{", packet->payload_packet_len)
 	 && (
 	   ndpi_strnstr((const char *)packet->payload, "\"eth1.0\"", packet->payload_packet_len)
@@ -103,7 +131,10 @@ void ndpi_search_mining_tcp(struct ndpi_detection_module_struct *ndpi_struct,
 	{"worker": "", "jsonrpc": "2.0", "params": [], "id": 3, "method": "eth_getWork"}
       */
       snprintf(flow->flow_extra_info, sizeof(flow->flow_extra_info), "%s", "ETH");
-      ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_MINING, NDPI_PROTOCOL_UNKNOWN);
+      ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_MINING, NDPI_PROTOCOL_UNKNOWN, NDPI_CONFIDENCE_DPI);
+      if(packet->iph) /* TODO: ipv6 */
+        cacheMiningHostTwins(ndpi_struct, packet->iph->saddr + packet->iph->daddr);
+      return;
     } else if(ndpi_strnstr((const char *)packet->payload, "{", packet->payload_packet_len)
 	      && (ndpi_strnstr((const char *)packet->payload, "\"method\":", packet->payload_packet_len)
 		  || ndpi_strnstr((const char *)packet->payload, "\"blob\":", packet->payload_packet_len)
@@ -124,7 +155,10 @@ void ndpi_search_mining_tcp(struct ndpi_detection_module_struct *ndpi_struct,
 	{"method":"submit","params":{"id":"479059546883218","job_id":"722134174127131","nonce":"98024001","result":"c9be9381a68d533c059d614d961e0534d7d8785dd5c339c2f9596eb95f320100"},"id":1}
       */
       snprintf(flow->flow_extra_info, sizeof(flow->flow_extra_info), "%s", "ZCash/Monero");
-      ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_MINING, NDPI_PROTOCOL_UNKNOWN);
+      ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_MINING, NDPI_PROTOCOL_UNKNOWN, NDPI_CONFIDENCE_DPI);
+      if(packet->iph) /* TODO: ipv6 */
+        cacheMiningHostTwins(ndpi_struct, packet->iph->saddr + packet->iph->daddr);
+      return;
     }
   }
 
