@@ -1,7 +1,7 @@
 /*
  * quic.c
  *
- * Copyright (C) 2012-21 - ntop.org
+ * Copyright (C) 2012-22 - ntop.org
  *
  * This module is free software: you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -83,7 +83,8 @@ static int is_version_quic(uint32_t version)
   return version == V_1 ||
     ((version & 0xFFFFFF00) == 0xFF000000) /* IETF Drafts*/ ||
     ((version & 0xFFFFF000) == 0xfaceb000) /* Facebook */ ||
-    ((version & 0x0F0F0F0F) == 0x0a0a0a0a) /* Forcing Version Negotiation */;
+    ((version & 0x0F0F0F0F) == 0x0a0a0a0a) /* Forcing Version Negotiation */ ||
+    ((version & 0xFFFFFF00) == 0xFF020000) /* V2 IETF Drafts */;
 }
 static int is_version_valid(uint32_t version)
 {
@@ -112,6 +113,12 @@ static uint8_t get_u8_quic_ver(uint32_t version)
      only latest drafts... */
   if ((version & 0x0F0F0F0F) == 0x0a0a0a0a)
     return 29;
+
+  /* QUIC Version 2 */
+  /* For the time being use 100 + draft as a number for V2 */
+  if ((version >> 8) == 0xff0200)
+    return 100 + (uint8_t)version;
+
   return 0;
 }
 #ifdef HAVE_LIBGCRYPT
@@ -181,6 +188,15 @@ int is_version_with_ietf_long_header(uint32_t version)
     ((version & 0xFFFFFF00) == 0x51303500) /* Q05X */ ||
     ((version & 0xFFFFFF00) == 0x54303500) /* T05X */;
 }
+#ifdef HAVE_LIBGCRYPT
+int is_version_with_v1_labels(uint32_t version)
+{
+  if(((version & 0xFFFFFF00) == 0x51303500)  /* Q05X */ ||
+     ((version & 0xFFFFFF00) == 0x54303500)) /* T05X */
+    return 1;
+  return is_quic_ver_less_than(version, 33);
+}
+#endif
 
 int quic_len(const uint8_t *buf, uint64_t *value)
 {
@@ -196,7 +212,7 @@ int quic_len(const uint8_t *buf, uint64_t *value)
     *value = ntohl(*(uint32_t *)buf) & 0x3FFFFFFF;
     return 4;
   case 3:
-    *value = ndpi_ntohll(*(uint64_t *)buf) & 0x3FFFFFFFFFFFFFFF;
+    *value = ndpi_ntohll(get_u_int64_t(buf, 0)) & 0x3FFFFFFFFFFFFFFF;
     return 8;
   default: /* No Possible */
     return 0;
@@ -223,7 +239,7 @@ static uint16_t gquic_get_u16(const uint8_t *buf, uint32_t version)
 {
   if(version >= V_Q039)
     return ntohs(*(uint16_t *)buf);
-  return (*(uint16_t *)buf);
+  return le16toh(*(uint16_t *)buf);
 }
 
 
@@ -531,29 +547,34 @@ static void quic_ciphers_reset(quic_ciphers *ciphers)
  * and initialize cipher with the new key.
  */
 static int quic_hp_cipher_init(quic_hp_cipher *hp_cipher, int hash_algo,
-			       uint8_t key_length, uint8_t *secret)
+			       uint8_t key_length, uint8_t *secret,
+			       uint32_t version)
 {
   uint8_t hp_key[256/8]; /* Maximum key size is for AES256 cipher. */
   uint32_t hash_len = gcry_md_get_algo_dlen(hash_algo);
+  char *label = is_version_with_v1_labels(version) ? "quic hp" : "quicv2 hp";
 
-  if(!quic_hkdf_expand_label(hash_algo, secret, hash_len, "quic hp", hp_key, key_length)) {
+  if(!quic_hkdf_expand_label(hash_algo, secret, hash_len, label, hp_key, key_length)) {
     return 0;
   }
 
   return gcry_cipher_setkey(hp_cipher->hp_cipher, hp_key, key_length) == 0;
 }
 static int quic_pp_cipher_init(quic_pp_cipher *pp_cipher, int hash_algo,
-			       uint8_t key_length, uint8_t *secret)
+			       uint8_t key_length, uint8_t *secret,
+			       uint32_t version)
 {
   uint8_t write_key[256/8]; /* Maximum key size is for AES256 cipher. */
   uint32_t hash_len = gcry_md_get_algo_dlen(hash_algo);
+  char *key_label = is_version_with_v1_labels(version) ? "quic key" : "quicv2 key";
+  char *iv_label = is_version_with_v1_labels(version) ? "quic iv" : "quicv2 iv";
 
   if(key_length > sizeof(write_key)) {
     return 0;
   }
 
-  if(!quic_hkdf_expand_label(hash_algo, secret, hash_len, "quic key", write_key, key_length) ||
-     !quic_hkdf_expand_label(hash_algo, secret, hash_len, "quic iv", pp_cipher->pp_iv, sizeof(pp_cipher->pp_iv))) {
+  if(!quic_hkdf_expand_label(hash_algo, secret, hash_len, key_label, write_key, key_length) ||
+     !quic_hkdf_expand_label(hash_algo, secret, hash_len, iv_label, pp_cipher->pp_iv, sizeof(pp_cipher->pp_iv))) {
     return 0;
   }
 
@@ -579,7 +600,7 @@ static int quic_get_pn_cipher_algo(int cipher_algo, int *hp_cipher_mode)
  * If the optional base secret is given, then its length MUST match the hash
  * algorithm output.
  */
-static int quic_hp_cipher_prepare(quic_hp_cipher *hp_cipher, int hash_algo, int cipher_algo, uint8_t *secret)
+static int quic_hp_cipher_prepare(quic_hp_cipher *hp_cipher, int hash_algo, int cipher_algo, uint8_t *secret, u_int32_t version)
 {
 #if 0
   /* Clear previous state (if any). */
@@ -604,7 +625,7 @@ static int quic_hp_cipher_prepare(quic_hp_cipher *hp_cipher, int hash_algo, int
 
   if(secret) {
     uint32_t cipher_keylen = (uint8_t)gcry_cipher_get_algo_keylen(cipher_algo);
-    if(!quic_hp_cipher_init(hp_cipher, hash_algo, cipher_keylen, secret)) {
+    if(!quic_hp_cipher_init(hp_cipher, hash_algo, cipher_keylen, secret, version)) {
       quic_hp_cipher_reset(hp_cipher);
 #ifdef DEBUG_CRYPT
       printf("Failed to derive key material for HP cipher\n");
@@ -615,7 +636,7 @@ static int quic_hp_cipher_prepare(quic_hp_cipher *hp_cipher, int hash_algo, int
 
   return 1;
 }
-static int quic_pp_cipher_prepare(quic_pp_cipher *pp_cipher, int hash_algo, int cipher_algo, int cipher_mode, uint8_t *secret)
+static int quic_pp_cipher_prepare(quic_pp_cipher *pp_cipher, int hash_algo, int cipher_algo, int cipher_mode, uint8_t *secret, u_int32_t version)
 {
 #if 0
   /* Clear previous state (if any). */
@@ -632,7 +653,7 @@ static int quic_pp_cipher_prepare(quic_pp_cipher *pp_cipher, int hash_algo, int
 
   if(secret) {
     uint32_t cipher_keylen = (uint8_t)gcry_cipher_get_algo_keylen(cipher_algo);
-    if(!quic_pp_cipher_init(pp_cipher, hash_algo, cipher_keylen, secret)) {
+    if(!quic_pp_cipher_init(pp_cipher, hash_algo, cipher_keylen, secret, version)) {
       quic_pp_cipher_reset(pp_cipher);
 #ifdef DEBUG_CRYPT
       printf("Failed to derive key material for PP cipher\n");
@@ -643,10 +664,10 @@ static int quic_pp_cipher_prepare(quic_pp_cipher *pp_cipher, int hash_algo, int
 
   return 1;
 }
-static int quic_ciphers_prepare(quic_ciphers *ciphers, int hash_algo, int cipher_algo, int cipher_mode, uint8_t *secret)
+static int quic_ciphers_prepare(quic_ciphers *ciphers, int hash_algo, int cipher_algo, int cipher_mode, uint8_t *secret, u_int32_t version)
 {
-  return quic_hp_cipher_prepare(&ciphers->hp_cipher, hash_algo, cipher_algo, secret) &&
-         quic_pp_cipher_prepare(&ciphers->pp_cipher, hash_algo, cipher_algo, cipher_mode, secret);
+  return quic_hp_cipher_prepare(&ciphers->hp_cipher, hash_algo, cipher_algo, secret, version) &&
+         quic_pp_cipher_prepare(&ciphers->pp_cipher, hash_algo, cipher_algo, cipher_mode, secret, version);
 }
 /**
  * Given a header protection cipher, a buffer and the packet number offset,
@@ -864,6 +885,10 @@ static int quic_derive_initial_secrets(uint32_t version,
 						0x38, 0x76, 0x2c, 0xf7, 0xf5, 0x59, 0x34, 0xb3, 0x4d, 0x17,
 						0x9a, 0xe6, 0xa4, 0xc8, 0x0c, 0xad, 0xcc, 0xbb, 0x7f, 0x0a
   };
+  static const uint8_t handshake_salt_v2_draft_00[20] = {
+						     0xa7, 0x07, 0xc2, 0x03, 0xa5, 0x9b, 0x47, 0x18, 0x4a, 0x1d,
+						     0x62, 0xca, 0x57, 0x04, 0x06, 0xea, 0x7a, 0xe3, 0xe5, 0xd3
+  };
   gcry_error_t err;
   uint8_t secret[HASH_SHA2_256_LENGTH];
 #ifdef DEBUG_CRYPT
@@ -894,10 +919,14 @@ static int quic_derive_initial_secrets(uint32_t version,
     err = hkdf_extract(GCRY_MD_SHA256, handshake_salt_draft_29,
 		       sizeof(handshake_salt_draft_29),
                        cid, cid_len, secret);
-  } else {
+  } else if (is_quic_ver_less_than(version, 33)) {
     err = hkdf_extract(GCRY_MD_SHA256, handshake_salt_v1,
 		       sizeof(handshake_salt_v1),
                        cid, cid_len, secret);
+  } else {
+    err = hkdf_extract(GCRY_MD_SHA256, handshake_salt_v2_draft_00,
+		       sizeof(handshake_salt_v2_draft_00),
+                       cid, cid_len, secret);
   }
   if(err) {
 #ifdef DEBUG_CRYPT
@@ -929,11 +958,11 @@ static uint8_t *decrypt_initial_packet(struct ndpi_detection_module_struct *ndpi
 				       uint32_t *clear_payload_len)
 {
   uint64_t token_length, payload_length, packet_number;
-  struct ndpi_packet_struct *packet = &flow->packet;
+  struct ndpi_packet_struct *packet = &ndpi_struct->packet;
   uint8_t first_byte;
   uint32_t pkn32, pn_offset, pkn_len, offset;
   quic_ciphers ciphers; /* Client initial ciphers */
-  quic_decrypt_result_t decryption = {0};
+  quic_decrypt_result_t decryption = { 0, 0};
   uint8_t client_secret[HASH_SHA2_256_LENGTH];
 
   memset(&ciphers, '\0', sizeof(ciphers));
@@ -947,7 +976,7 @@ static uint8_t *decrypt_initial_packet(struct ndpi_detection_module_struct *ndpi
      Initial packets are protected with AEAD_AES_128_GCM. */
   if(!quic_ciphers_prepare(&ciphers, GCRY_MD_SHA256,
                            GCRY_CIPHER_AES128, GCRY_CIPHER_MODE_GCM,
-                           client_secret)) {
+                           client_secret, version)) {
     NDPI_LOG_DBG(ndpi_struct, "Error quic_cipher_prepare\n");
     return NULL;
   }
@@ -956,8 +985,9 @@ static uint8_t *decrypt_initial_packet(struct ndpi_detection_module_struct *ndpi
   pn_offset = 1 + 4 + 1 + dest_conn_id_len + 1 + source_conn_id_len;
   pn_offset += quic_len(&packet->payload[pn_offset], &token_length);
   pn_offset += token_length;
-  /* Checks: quic_len reads 8 bytes, at most; quic_decrypt_header reads other 20 bytes */
-  if(pn_offset + 8 + (4 + 16) >= packet->payload_packet_len) {
+  /* Checks: quic_len reads 8 bytes, at most; quic_decrypt_header reads other 20 bytes.
+     Promote to uint64_t to avoid unsigned wrapping */
+  if((uint64_t)pn_offset + 8 + (4 + 16) >= (uint64_t)packet->payload_packet_len) {
     quic_ciphers_reset(&ciphers);
     return NULL;
   }
@@ -1010,7 +1040,7 @@ static int __reassemble(struct ndpi_flow_struct *flow, const u_int8_t *frag,
                         uint64_t frag_len, uint64_t frag_offset,
                         const u_int8_t **buf, u_int64_t *buf_len)
 {
-  const int max_quic_reasm_buffer_len = 4096; /* Let's say a couple of full-MTU packets... */
+  const uint64_t max_quic_reasm_buffer_len = 4096; /* Let's say a couple of full-MTU packets... */
 
   /* TODO: at the moment, this function is only a little more than a stub.
      We should reassemble the fragments, but nDPI lacks any proper generic
@@ -1235,7 +1265,7 @@ static uint8_t *get_clear_payload(struct ndpi_detection_module_struct *ndpi_stru
 				  struct ndpi_flow_struct *flow,
 				  uint32_t version, uint32_t *clear_payload_len)
 {
-  struct ndpi_packet_struct *packet = &flow->packet;
+  struct ndpi_packet_struct *packet = &ndpi_struct->packet;
   u_int8_t *clear_payload;
   u_int8_t dest_conn_id_len;
 #ifdef HAVE_LIBGCRYPT
@@ -1286,7 +1316,7 @@ static void process_tls(struct ndpi_detection_module_struct *ndpi_struct,
 			const u_int8_t *crypto_data, uint32_t crypto_data_len,
 			uint32_t version)
 {
-  struct ndpi_packet_struct *packet = &flow->packet;
+  struct ndpi_packet_struct *packet = &ndpi_struct->packet;
 
   /* Overwriting packet payload */
   u_int16_t p_len;
@@ -1297,7 +1327,7 @@ static void process_tls(struct ndpi_detection_module_struct *ndpi_struct,
   packet->payload_packet_len = crypto_data_len;
 
   processClientServerHello(ndpi_struct, flow, version);
-  flow->l4.tcp.tls.hello_processed = 1; /* Allow matching of custom categories */
+  flow->protos.tls_quic.hello_processed = 1; /* Allow matching of custom categories */
 
   /* Restore */
   packet->payload = p;
@@ -1307,13 +1337,13 @@ static void process_tls(struct ndpi_detection_module_struct *ndpi_struct,
      this way we lose JA3S and negotiated ciphers...
      Negotiated version is only present in the ServerHello message too, but
      fortunately, QUIC always uses TLS version 1.3 */
-  flow->protos.tls_quic_stun.tls_quic.ssl_version = 0x0304;
+  flow->protos.tls_quic.ssl_version = 0x0304;
 
   /* DNS-over-QUIC: ALPN is "doq" or "doq-XXX" (for drafts versions) */
-  if(flow->protos.tls_quic_stun.tls_quic.alpn &&
-     strncmp(flow->protos.tls_quic_stun.tls_quic.alpn, "doq", 3) == 0) {
-    NDPI_LOG_DBG(ndpi_struct, "Found DOQ (ALPN: [%s])\n", flow->protos.tls_quic_stun.tls_quic.alpn);
-    ndpi_int_change_protocol(ndpi_struct, flow, NDPI_PROTOCOL_DOH_DOT, NDPI_PROTOCOL_QUIC);
+  if(flow->protos.tls_quic.alpn &&
+     strncmp(flow->protos.tls_quic.alpn, "doq", 3) == 0) {
+    NDPI_LOG_DBG(ndpi_struct, "Found DOQ (ALPN: [%s])\n", flow->protos.tls_quic.alpn);
+    ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_DOH_DOT, NDPI_PROTOCOL_QUIC, NDPI_CONFIDENCE_DPI);
   }
 }
 static void process_chlo(struct ndpi_detection_module_struct *ndpi_struct,
@@ -1324,7 +1354,7 @@ static void process_chlo(struct ndpi_detection_module_struct *ndpi_struct,
   uint32_t i;
   uint16_t num_tags;
   uint32_t prev_offset;
-  uint32_t tag_offset_start, offset, len, sni_len;
+  uint32_t tag_offset_start, offset, len;
   ndpi_protocol_match_result ret_match;
   int sni_found = 0, ua_found = 0;
 
@@ -1336,7 +1366,7 @@ static void process_chlo(struct ndpi_detection_module_struct *ndpi_struct,
 #endif
     return;
   }
-  num_tags = (*(uint16_t *)&crypto_data[4]);
+  num_tags = le16toh(*(uint16_t *)&crypto_data[4]);
 
   tag_offset_start = 8 + 8 * num_tags;
   prev_offset = 0;
@@ -1344,34 +1374,32 @@ static void process_chlo(struct ndpi_detection_module_struct *ndpi_struct,
     if(8 + 8 * i + 8 >= crypto_data_len)
       break;
     tag = &crypto_data[8 + 8 * i];
-    offset = *((u_int32_t *)&crypto_data[8 + 8 * i + 4]);
+    offset = le32toh(*((u_int32_t *)&crypto_data[8 + 8 * i + 4]));
     if(prev_offset > offset)
       break;
     len = offset - prev_offset;
-    if(tag_offset_start + prev_offset + len > crypto_data_len)
+    /* Promote to uint64_t to avoid unsigned wrapping */
+    if((uint64_t)tag_offset_start + prev_offset + len > (uint64_t)crypto_data_len)
       break;
 #if 0
-    printf("crypto_data_len %u prev_offset %u offset %u len %d\n",
-	   crypto_data_len, prev_offset, offset, len);
+    printf("crypto_data_len %u tag_offset_start %u prev_offset %u offset %u len %u\n",
+	   crypto_data_len, tag_offset_start, prev_offset, offset, len);
 #endif
-    if((memcmp(tag, "SNI\0", 4) == 0) &&
-       (tag_offset_start + prev_offset + len < crypto_data_len)) {
-      sni_len = MIN(len, sizeof(flow->protos.tls_quic_stun.tls_quic.client_requested_server_name) - 1);
-      memcpy(flow->protos.tls_quic_stun.tls_quic.client_requested_server_name,
-             &crypto_data[tag_offset_start + prev_offset], sni_len);
-      flow->protos.tls_quic_stun.tls_quic.client_requested_server_name[sni_len] = '\0';
+    if(memcmp(tag, "SNI\0", 4) == 0) {
+
+      ndpi_hostname_sni_set(flow, &crypto_data[tag_offset_start + prev_offset], len);
 
       NDPI_LOG_DBG2(ndpi_struct, "SNI: [%s]\n",
-                    flow->protos.tls_quic_stun.tls_quic.client_requested_server_name);
+                    flow->host_server_name);
 
       ndpi_match_host_subprotocol(ndpi_struct, flow,
-                                  (char *)flow->protos.tls_quic_stun.tls_quic.client_requested_server_name,
-                                  strlen((const char*)flow->protos.tls_quic_stun.tls_quic.client_requested_server_name),
+                                  flow->host_server_name,
+                                  strlen(flow->host_server_name),
                                   &ret_match, NDPI_PROTOCOL_QUIC);
-      flow->l4.tcp.tls.hello_processed = 1; /* Allow matching of custom categories */
+      flow->protos.tls_quic.hello_processed = 1; /* Allow matching of custom categories */
 
       ndpi_check_dga_name(ndpi_struct, flow,
-                          flow->protos.tls_quic_stun.tls_quic.client_requested_server_name, 1);
+                          flow->host_server_name, 1);
 
       sni_found = 1;
       if (ua_found)
@@ -1381,15 +1409,13 @@ static void process_chlo(struct ndpi_detection_module_struct *ndpi_struct,
     if(memcmp(tag, "UAID", 4) == 0) {
       u_int uaid_offset = tag_offset_start + prev_offset;
             
-      if((uaid_offset + len) < crypto_data_len) {      
-	NDPI_LOG_DBG2(ndpi_struct, "UA: [%.*s]\n", len, &crypto_data[uaid_offset]);
+      NDPI_LOG_DBG2(ndpi_struct, "UA: [%.*s]\n", len, &crypto_data[uaid_offset]);
 	
-	http_process_user_agent(ndpi_struct, flow, &crypto_data[uaid_offset], len); /* http.c */
-	ua_found = 1;
+      http_process_user_agent(ndpi_struct, flow, &crypto_data[uaid_offset], len); /* http.c */
+      ua_found = 1;
 	
-	if (sni_found)
-	  return;
-      }
+      if (sni_found)
+        return;
     }
 
     prev_offset = offset;
@@ -1398,7 +1424,7 @@ static void process_chlo(struct ndpi_detection_module_struct *ndpi_struct,
     NDPI_LOG_DBG(ndpi_struct, "Something went wrong in tags iteration\n");
 
   /* Add check for missing SNI */
-  if(flow->protos.tls_quic_stun.tls_quic.client_requested_server_name[0] == '\0') {
+  if(flow->host_server_name[0] == '\0') {
     /* This is a bit suspicious */
     ndpi_set_risk(ndpi_struct, flow, NDPI_TLS_MISSING_SNI);
   }
@@ -1409,7 +1435,7 @@ static int may_be_initial_pkt(struct ndpi_detection_module_struct *ndpi_struct,
 			      struct ndpi_flow_struct *flow,
 			      uint32_t *version)
 {
-  struct ndpi_packet_struct *packet = &flow->packet;
+  struct ndpi_packet_struct *packet = &ndpi_struct->packet;
   u_int8_t first_byte;
   u_int8_t pub_bit1, pub_bit2, pub_bit3, pub_bit4, pub_bit5, pub_bit7, pub_bit8;
   u_int8_t dest_conn_id_len, source_conn_id_len;
@@ -1476,7 +1502,7 @@ static int may_be_initial_pkt(struct ndpi_detection_module_struct *ndpi_struct,
       grease_quic_bit transport parameter." */
   if((*version & 0x0F0F0F0F) == 0x0a0a0a0a &&
      !(pub_bit1 == 1 && pub_bit2 == 1)) {
-    NDPI_LOG_DBG2(ndpi_struct, "Version 0x%x with first byte 0x%x\n", first_byte);
+    NDPI_LOG_DBG2(ndpi_struct, "Version 0x%x with first byte 0x%x\n", *version, first_byte);
     return 0;
   }
 
@@ -1510,7 +1536,7 @@ static int eval_extra_processing(struct ndpi_detection_module_struct *ndpi_struc
    */
 
   if((version == V_Q046 &&
-      flow->protos.tls_quic_stun.tls_quic.client_requested_server_name[0] == '\0') ||
+      flow->host_server_name[0] == '\0') ||
      is_ch_reassembler_pending(flow)) {
     NDPI_LOG_DBG2(ndpi_struct, "We have further work to do\n");
     return 1;
@@ -1529,7 +1555,7 @@ static void ndpi_search_quic(struct ndpi_detection_module_struct *ndpi_struct,
 static int ndpi_search_quic_extra(struct ndpi_detection_module_struct *ndpi_struct,
 				  struct ndpi_flow_struct *flow)
 {
-  struct ndpi_packet_struct *packet = &flow->packet;
+  struct ndpi_packet_struct *packet = &ndpi_struct->packet;
 
   /* We are elaborating a packet following the initial CHLO/ClientHello.
      Two cases:
@@ -1543,7 +1569,10 @@ static int ndpi_search_quic_extra(struct ndpi_detection_module_struct *ndpi_stru
 
   if (is_ch_reassembler_pending(flow)) {
     ndpi_search_quic(ndpi_struct, flow);
-    return is_ch_reassembler_pending(flow);
+    if(is_ch_reassembler_pending(flow))
+      return 1;
+    flow->extra_packets_func = NULL;
+    return 0;
   }
 
   /* RTP/RTCP stuff */
@@ -1566,7 +1595,7 @@ static int ndpi_search_quic_extra(struct ndpi_detection_module_struct *ndpi_stru
       packet->payload[1] == 200 || /* RTCP, Sender Report */
       is_valid_rtp_payload_type(packet->payload[1] & 0x7F)) /* RTP */) {
     NDPI_LOG_DBG(ndpi_struct, "Found RTP/RTCP over QUIC\n");
-    ndpi_int_change_protocol(ndpi_struct, flow, NDPI_PROTOCOL_SNAPCHAT_CALL, NDPI_PROTOCOL_QUIC);
+    ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_SNAPCHAT_CALL, NDPI_PROTOCOL_QUIC, NDPI_CONFIDENCE_DPI);
   } else {
     /* Unexpected traffic pattern: we should investigate it... */
     NDPI_LOG_INFO(ndpi_struct, "To investigate...\n");
@@ -1580,7 +1609,7 @@ static void ndpi_search_quic(struct ndpi_detection_module_struct *ndpi_struct,
 {
   u_int32_t version;
   u_int8_t *clear_payload;
-  uint32_t clear_payload_len;
+  uint32_t clear_payload_len = 0;
   const u_int8_t *crypto_data;
   uint64_t crypto_data_len;
   int is_quic;
@@ -1610,7 +1639,7 @@ static void ndpi_search_quic(struct ndpi_detection_module_struct *ndpi_struct,
    */
 
   NDPI_LOG_INFO(ndpi_struct, "found QUIC\n");
-  ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_QUIC, NDPI_PROTOCOL_UNKNOWN);
+  ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_QUIC, NDPI_PROTOCOL_UNKNOWN, NDPI_CONFIDENCE_DPI);
 
   /*
    * 3) Skip not supported versions
