@@ -1,7 +1,7 @@
 /*
  * rx.c
  *
- * Copyright (C) 2012-20 - ntop.org
+ * Copyright (C) 2012-22 - ntop.org
  *
  * Giovanni Mascellani <gio@debian.org>
  *
@@ -48,19 +48,19 @@ struct ndpi_rx_header {
 } PACK_OFF;
 
 /* Type values */
-#define DATA	           1
-#define	ACK	           2
-#define	BUSY	           3	
-#define	ABORT	           4	
-#define	ACKALL	           5	
-#define	CHALLENGE          6	
-#define	RESPONSE           7	
-#define	DEBUG	           8	
-#define	PARAM_1            9	
-#define	PARAM_2           10	
-#define	PARAM_3           11	
-#define	PARAMS_4          12	
-#define	VERS	          13
+#define RX_DATA	           1
+#define	RX_ACK	           2
+#define	RX_BUSY	           3
+#define	RX_ABORT	   4
+#define	RX_ACKALL	   5
+#define	RX_CHALLENGE       6
+#define	RX_RESPONSE        7
+#define	RX_DEBUG           8
+#define	RX_PARAM_1         9
+#define	RX_PARAM_2        10
+#define	RX_PARAM_3        11
+#define	RX_PARAMS_4       12
+#define	RX_VERS	          13
 
 /* Flags values */
 #define EMPTY              0
@@ -72,13 +72,14 @@ struct ndpi_rx_header {
 #define PLUS_2             6
 #define MORE_1             9
 #define CLIENT_INIT_2     33
+#define PLUS_3            34
 
 
 
 void ndpi_check_rx(struct ndpi_detection_module_struct *ndpi_struct,
                    struct ndpi_flow_struct *flow)
 {
-  struct ndpi_packet_struct *packet = &flow->packet;
+  struct ndpi_packet_struct *packet = &ndpi_struct->packet;
   u_int32_t payload_len = packet->payload_packet_len;
 
   NDPI_LOG_DBG2(ndpi_struct, "RX: pck: %d, dir[0]: %d, dir[1]: %d\n",
@@ -108,7 +109,7 @@ void ndpi_check_rx(struct ndpi_detection_module_struct *ndpi_struct,
   **/
   
   /* TYPE field */
-  if((header->type < DATA) || (header->type > VERS)) {
+  if((header->type < RX_DATA) || (header->type > RX_VERS)) {
     NDPI_EXCLUDE_PROTO(ndpi_struct, flow);
     return;
   }
@@ -118,48 +119,48 @@ void ndpi_check_rx(struct ndpi_detection_module_struct *ndpi_struct,
      header->flags == PLUS_0 || header->flags == PLUS_1 ||
      header->flags == PLUS_2 || header->flags == REQ_ACK ||
      header->flags == MORE_1 || header->flags == CLIENT_INIT_1 ||
-     header->flags == CLIENT_INIT_2) {
+     header->flags == CLIENT_INIT_2 || header->flags == PLUS_3) {
 
     /* TYPE and FLAGS combo */
     switch(header->type)
     {
-      case DATA:
+      case RX_DATA:
 	if(header->flags == LAST_PKT || header->flags == EMPTY ||
 	   header->flags == PLUS_0 || header->flags == PLUS_1 ||
 	   header->flags == PLUS_2 || header->flags == REQ_ACK ||
 	   header->flags == MORE_1)
 	  goto security;
 	/* Fall-through */
-      case ACK:
+      case RX_ACK:
 	if(header->flags == CLIENT_INIT_1 || header->flags == CLIENT_INIT_2 ||
-	   header->flags == EMPTY)
+	   header->flags == EMPTY || header->flags == PLUS_3)
 	  goto security;
 	/* Fall-through */
-      case CHALLENGE:
+      case RX_CHALLENGE:
 	if(header->flags == EMPTY || header->call_number == 0)
 	  goto security;
 	/* Fall-through */
-      case RESPONSE:
+      case RX_RESPONSE:
 	if(header->flags == EMPTY || header->call_number == 0)
 	  goto security;
 	/* Fall-through */
-      case ACKALL:
+      case RX_ACKALL:
 	if(header->flags == EMPTY)
 	  goto security;
 	/* Fall-through */
-      case BUSY:
+      case RX_BUSY:
 	goto security;
-      case ABORT:
+      case RX_ABORT:
 	goto security;
-      case DEBUG:
+      case RX_DEBUG:
 	goto security;
-      case PARAM_1:
+      case RX_PARAM_1:
 	goto security;
-      case PARAM_2:
+      case RX_PARAM_2:
         goto security;
-      case PARAM_3:
+      case RX_PARAM_3:
 	goto security;
-      case VERS:
+      case RX_VERS:
 	goto security;
       default:
 	NDPI_EXCLUDE_PROTO(ndpi_struct, flow);
@@ -187,7 +188,7 @@ void ndpi_check_rx(struct ndpi_detection_module_struct *ndpi_struct,
 	flow->l4.udp.rx_conn_id == header->conn_id)
     {
       NDPI_LOG_INFO(ndpi_struct, "found RX\n");
-      ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_RX, NDPI_PROTOCOL_UNKNOWN);
+      ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_RX, NDPI_PROTOCOL_UNKNOWN, NDPI_CONFIDENCE_DPI);
     }
     /* https://www.central.org/frameless/numbers/rxservice.html. */
     else
@@ -198,20 +199,14 @@ void ndpi_check_rx(struct ndpi_detection_module_struct *ndpi_struct,
   } else {
     flow->l4.udp.rx_conn_epoch = header->conn_epoch;
     flow->l4.udp.rx_conn_id = header->conn_id;
-    {
-      NDPI_LOG_INFO(ndpi_struct, "found RX\n");
-      ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_RX, NDPI_PROTOCOL_UNKNOWN);
-    }
   }
 }
 
 void ndpi_search_rx(struct ndpi_detection_module_struct *ndpi_struct,
                     struct ndpi_flow_struct *flow)
 {
-  struct ndpi_packet_struct *packet = &flow->packet;
-
   NDPI_LOG_DBG(ndpi_struct, "search RX\n");
-  if (packet->detected_protocol_stack[0] != NDPI_PROTOCOL_RX) {
+  if (flow->detected_protocol_stack[0] != NDPI_PROTOCOL_RX) {
     ndpi_check_rx(ndpi_struct, flow);
   }
 }
@@ -223,7 +218,7 @@ void init_rx_dissector(struct ndpi_detection_module_struct *ndpi_struct,
   ndpi_set_bitmask_protocol_detection("RX", ndpi_struct, detection_bitmask, *id,
 				      NDPI_PROTOCOL_RX,
 				      ndpi_search_rx,
-				      NDPI_SELECTION_BITMASK_PROTOCOL_UDP_WITH_PAYLOAD,
+				      NDPI_SELECTION_BITMASK_PROTOCOL_V4_V6_UDP_WITH_PAYLOAD,
 				      SAVE_DETECTION_BITMASK_AS_UNKNOWN,
 				      ADD_TO_DETECTION_BITMASK);
 
