@@ -83,7 +83,7 @@ enum DEFAULT_PROTOCOL_PORTS{
 static void ndpi_int_someip_add_connection (struct ndpi_detection_module_struct *ndpi_struct,
 					    struct ndpi_flow_struct *flow)
 {
-  ndpi_set_detected_protocol(ndpi_struct,flow,NDPI_PROTOCOL_SOMEIP,NDPI_PROTOCOL_UNKNOWN);
+  ndpi_set_detected_protocol(ndpi_struct,flow,NDPI_PROTOCOL_SOMEIP,NDPI_PROTOCOL_UNKNOWN, NDPI_CONFIDENCE_DPI);
   NDPI_LOG_INFO(ndpi_struct, "found SOME/IP\n");
 }
 
@@ -101,7 +101,7 @@ static u_int32_t someip_data_cover_32(const u_int8_t *data)
 void ndpi_search_someip (struct ndpi_detection_module_struct *ndpi_struct,
 			 struct ndpi_flow_struct *flow)
 {
-  const struct ndpi_packet_struct *packet = &flow->packet;
+  const struct ndpi_packet_struct *packet = &ndpi_struct->packet;
   
   if (packet->payload_packet_len < 16) {
     NDPI_LOG(NDPI_PROTOCOL_SOMEIP, ndpi_struct, NDPI_LOG_DEBUG,
@@ -114,7 +114,7 @@ void ndpi_search_someip (struct ndpi_detection_module_struct *ndpi_struct,
 
   NDPI_LOG_DBG(ndpi_struct, "search SOME/IP\n");
 
-  if (packet->detected_protocol_stack[0] != NDPI_PROTOCOL_UNKNOWN) {
+  if (flow->detected_protocol_stack[0] != NDPI_PROTOCOL_UNKNOWN) {
     return;
   }
  
@@ -204,13 +204,13 @@ void ndpi_search_someip (struct ndpi_detection_module_struct *ndpi_struct,
   //Filtering by port. 
   //This check is NOT a 100% thing - these ports are mentioned in the documentation but the documentation also states they haven't been approved by IANA yet, and that the user is free to use different ports.
   //This is is PURELY for demo purposes and the rest of the check must be filled in later on!
-  if (packet->l4_protocol == IPPROTO_UDP){
+  if (flow->l4_proto == IPPROTO_UDP){
     if ((packet->udp->dest == ntohs(PORT_DEFAULT_CLIENT)) || (packet->udp->dest == ntohs(PORT_DEFAULT_SERVER)) || (packet->udp->dest == ntohs(PORT_DEFAULT_SD))) {
       ndpi_int_someip_add_connection(ndpi_struct, flow);
       return;
     }
   }
-  if (packet->l4_protocol == IPPROTO_TCP){
+  if (flow->l4_proto == IPPROTO_TCP){
     if ((packet->tcp->dest == ntohs(PORT_DEFAULT_CLIENT)) || (packet->tcp->dest == ntohs(PORT_DEFAULT_SERVER))) {
       ndpi_int_someip_add_connection(ndpi_struct, flow);
       return;
